//bump.cpp

#include "bump.h"
#include "main.h"
#include "utility.h"


Bump::Bump() {
	size=128;
	bumparray=0;
	freq=1;
	return;
};

Bump::~Bump() {};


/*
//-------------------------------------------------------------------------------------
// create sphere of size, tesselated by tesx/y,
//   with texcoord starting at texoff* - texmag*, and bumped
void solid_sphereb(double size, int tesx, int tesy, double texmagx, double texmagy, 
                   double texoffx, double texoffy,
						 double bumpheight) {
   int aix, ix,iy, iy2;
   double rx,ry, cx, cy,cy2, dx,dy,dz, dx2,dy2,dz2, fx,fy,fy2, texx, texy, texy2, height;
   height=1;
   rx=pi2/ tesx;
   ry=pi1/(tesy-1);
   for    (iy=0; iy<(tesy-1); iy++) {
      iy2=iy+1;
      fy=iy*ry;
      fy2=iy2*ry;
	  dy = -cos(fy );
	  dy2= -cos(fy2);
	  cy=(double)iy /tesy;
	  cy2=(double)iy2 /tesy;
	  texy =texoffy+texmagy*cy;
	  texy2=texoffy+texmagy*cy2;

      glBegin(GL_TRIANGLE_STRIP);
      for (ix=0; ix<=tesx; ix++) {
		  cx=(double)ix /tesx;
            aix=(ix==tesx) ? 0 : ix;
            fx=aix*rx;
            dx =(sin(fx) * sin(fy ));   dz = (cos(fx) * sin(fy ));
            dx2=(sin(fx) * sin(fy2));   dz2= (cos(fx) * sin(fy2));
            
			texx =texoffx+texmagx*cx;

			if (bumpheight) height=get_bump(cx,cy2,bump)*bumpheight+1;
    	    glNormal3d(dx2,dy2,dz2);
     	    glTexCoord2d(texx, texy2);
			glVertex3d(dx2*size*height, dy2*size*height, dz2*size*height);

			if (bumpheight) height=get_bump(cx,cy,bump)*bumpheight+1;
            glNormal3d(dx ,dy ,dz);
    	    glTexCoord2d(texx, texy);
			glVertex3d(dx *size*height, dy *size*height, dz *size*height);
         }
         glEnd();
		}
	return;
}

//------------------------------------------------------------------------------*/

int Bump::load(const char *picname, float mag, float off) {
	FILE * file;
	int x,y, addr, red,green,blue, sizex, sizey;
	float height;

	file= fopen(picname, "rb");
    if (file == 0) return 1;

    fseek(file, 18, SEEK_SET);
	 sizex=fgetc(file)+fgetc(file)*256;
    fseek(file, 22, SEEK_SET);
	 sizey=fgetc(file)+fgetc(file)*256;
	 if (sizex*sizey>65536) return 0;
	 size=sizex;

    fseek(file, 54, SEEK_SET);
    for     (y = 0; y <size; y++) {
        for (x = 0; x <size; x++) {
			addr=x+(size-y-1)*size;
			blue =fgetc(file);
			green=fgetc(file);
			red  =fgetc(file);
			height=(green/128.-1)*mag+off;//(red+green+blue)/(3*256.);
			if (addr<65536) bumparray[addr] = height;
		}
	}
	fclose(file);
	return 0;
}

//---------------------------------------------------------------------------

int Bump::make(int mode, int insize, float mag, float off) {
	int x,y, addr;
	float height, s2, dist;
	size=insize;
	s2=size*.5;
	for     (y = 0; y <size; y++) {
		for (x = 0; x <size; x++) {
			addr=(size-x-1)+(size-y-1)*size;
			switch(mode) {
			case 0:
				height=-cos((x-s2)*.12*freq)*
					   -cos((y-s2)*.12*freq);
				break;
			case 1:
				dist=sqrt((x-s2)*(x-s2)+(y-s2)*(y-s2));
				height=sin(dist*.3*freq);
				if (height>.25) height=.25;
				break;
			case 4:
				if (y<20) { // big v ramp
					if (x<111) height = cos((((131.75/128.)*x)/size)*pi2-.3)*12-7;
					else height=0;
				}
				else if ((x<20)&&(y<84)&&(y>=20)){ // ramp up
					height=cos_interp(5,0,((float)(y-20)/64));
				}
				else if (y<28) { // wall
					if (x<111) height=5; else height=0;
				}
				else if (y<36) { // curve
					if (x<111) height=-sin((y-28)*.225)*5+5; else height=0;
				}
				else if ((y>90)&&(x>20)&&(x<108)) 
					if (x<64) height=(cos((y-90)*.18)-1)*2; // trough
					else      height=(cos((y-90)*.18)-1)*2 * (-cos((x-21)*.14612)+1)*.2;
				else height=0;

				dist=sqrt((x-s2+5)*(x-s2+5)+(y-s2+2)*(y-s2+2))/s2; 
				if (dist<.3.33) {
					dist*=3;
					height=(-cos(dist*pi1)-1)*3;
				}
				if ((x>90)&&(x<110)&&(y>55)&&(y<65)) {
					height=(cos((x-90.)/20.*pi2)-1)*1;
				}
				if (((127-x)+(127-y))<10) {
					height=(10-((127-x)+(127-y)))*.2;
				}
				if ((x+(127-y))<10) {
					height=(10-(x+(127-y)))*.2;
				}
				break;
			case 5:	height=0; break;
			}
			if (addr<65536) bumparray[addr]=height*mag+off;
		}
	}
	return 0;
}

//---------------------------------------------------------------------------

float Bump::get(float x, float y) { // 0-1, 0-1 -> 0-1
	int ix, iy, addr;
	float v=0, v0=0, xi,yi;

	while (x<0) x+=1;
	while (y<0) y+=1;
	while (x>1) x-=1;
	while (y>1) y-=1;
	ix=x*size; 
	iy=y*size;
	xi=x*size-ix;
	yi=y*size-iy;
	if (ix>=size) ix=size-1;
	if (iy>=size) iy=size-1;
	//if (xi>=1) xi=0;
	//if (yi>=1) yi=0;

	addr=ix+iy*size;
	if (addr<65536) v0=bumparray[addr];
	v=v0;

	addr=((ix+1)%size)+iy         *size;
	if (addr<65536) v+=(bumparray[addr]-v0)*xi;

	addr=  ix+       ((iy+1)%size)*size;
	if (addr<65536) v+=(bumparray[addr]-v0)*yi;
	return v;
}

//--------------------------------------------------------------------

void Bump::getv(float x, float y, float *xi, float *zi) {
	float v;

	v=get(x,y);
	*xi=-(get(x+.01,y    )-v);
	*zi=-(get(x,    y+.01)-v);
}

//--------------------------------------------------------------------

int Bump::make2(int mode, int insize) {
	int addr, x,y;
	float height, s2, h,fx,fy, f;

	size=insize;
	s2=size*.5;
	for    (y = 0; y <size; y++) {
		for (x = 0; x <size; x++) {
			addr=(size-x-1)+(size-y-1)*size;
			height=(float)y/size;
			if (addr<65536) {
				bumparray[addr]=height;
				bumparray2[addr]=height;
			}
		}
	}
	for (f=0; f<1; f+=.0005) {
		int s=10;
		float lfx, lfy, dx,dy;

		fx=(cos(f*2*pi2)*(1-f)+1)*50+14;
		fy=f*100+14;
		dx=pow(fx-lfx,2);
		dy=pow(fy-lfy,2);
		if (sqrt(dx+dy)<.05) {
			continue;
		}
		height=(float)y/size;
		for    (x=-s; x<s; x++) {
			for (y=-s; y<s; y++) {
				addr=(size-(x+(int)fx)-1)+(size-(y+(int)fy)-1)*size;
				h=sqrt(x*x+y*y);
				if (h>=s) continue;
				h=cos_interp(-20,0,h*(1./s));
				if (h<height-12) h=height-12;
				if (addr<65536) {
					if ((bumparray[addr]+h)<bumparray2[addr])
						  bumparray2[addr]=  bumparray[addr]+h;
				}
			}
		lfx=fx;
		lfy=fy;
		}
	}
	for    (y = 0; y <size; y++) {
		for (x = 0; x <size; x++) {
			addr=(size-x-1)+(size-y-1)*size;
			if (addr<65536) bumparray[addr]=bumparray2[addr];
		}
	}
	return 0;
}

//---------------------------------------------------------------------

void Bump::depress(int inx, int iny, int insize, int indepth) {
	int addr, x,y, xa,ya;
	float dist, s2, h;

	s2=insize*.5;
	for (addr=0; addr<65536; addr++) bumparray2[addr]=bumparray[addr];

	for     (x=-s2; x<s2; x++) {
		for (y=-s2; y<s2; y++) {
			xa=(x+inx)%size;
			ya=(y+iny)%size;
			addr=(size-xa-1)+(size-ya-1)*size;
			dist=sqrt(x*x+y*y);
			if (dist>=s2) continue;
			h=cos_interp(indepth,0,dist/s2);
			if (addr<65536) {
				if ((bumparray[addr]-h)<bumparray2[addr])
					 bumparray2[addr]=  bumparray[addr]-h;
			}
		}
	}
	for (addr=0; addr<65536; addr++) bumparray[addr]=bumparray2[addr];
	return;
}

//---------------------------------------------------------------------

