//-------------------------------------------------------------------
//	File:		   game.c
//	Comments:   by George Ziniewicz
//-------------------------------------------------------------------

#include "game.h"

extern float mesh_grav(Object *object, int fall);
extern void announce(char *s);
extern void make_bump(void);
extern void setc(void);

//------------------------------------------------------------------------------------

Character:: Character() {};
Character::~Character() {};

//------------------------------------------------------------------------------------

Pack:: Pack() {};
Pack::~Pack() {};

//------------------------------------------------------------------------------------

Game::Game() {
	hand=1;
	handHeight=-.5;
	mybullet=0;
	hisbullet=0;
	splat=0;
	bulletSpeed=1000;
	maxlives=5;
	maxbullet=25;
	lastBulletTime=0;

	him.alphafunc=.005;
	him.size=50;
	him.sizex=him.size;
	him.sizey=him.size;
	him.sizez=him.size;
	him.faceScreen=1;
	him.blend=0;

	pack.size=30;
	pack.sizex=pack.size;
	pack.sizey=pack.size;
	pack.sizez=pack.size;
	pack.alphafunc=.01;
	pack.faceScreen=1;
	pack.blend=1;
	pack.r=0;
	pack.g=1;
	pack.b=0;
	pack.a=.9;

	port.size=30;
	port.sizex=port.size;
	port.sizey=port.size;
	port.sizez=port.size;
	port.alphafunc=.01;
	port.alphatest=1;
	port.faceScreen=1;
	port.blend=1;
	port.r=1;
	port.g=1;
	port.b=0;
	port.a=.9;

	med.size=30;
	med.sizex=med.size;
	med.sizey=med.size;
	med.sizez=med.size;
	med.alphafunc=.01;
	med.faceScreen=1;
	med.blend=1;
	med.r=1;
	med.g=1;
	med.b=1;
	med.a=.9;

	aggr=0;
	difficulty=1;
	lastoption=-1; lastimage=-1; lastuniverse=-1; lastb0=-1; lastb1=-1;
	lastTauntTime=0;
	on=0;
};

Game::~Game() {};

//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------

void Game::init(void) {
	me.health=100;
	me.nlives=maxlives;
	me.nbullet=maxbullet;

	him.health=100;
	him.nlives=maxlives;
	him.nbullet=9999999;

	pack.x=rndn(1000)-500;
	pack.y=5000;
	pack.z=rndn(1000)-500;
	pack.nbullet=maxbullet;

	port.x=rndn(1000)-500;
	port.y=5000;
	port.z=rndn(1000)-500;

	med.x=rndn(1000)-500;
	med.y=5000;
	med.z=rndn(1000)-500;

	on=0;
}

//---------------------------------------------------------------------------------------

void Game::boost(void) {
	if (hw->time->fDeltaTime<0) return;
	me.health+= 1*hw->time->fDeltaTime; if (me.health >100) me.health =100;
	him.health+=1*hw->time->fDeltaTime; if (him.health>100) him.health=100;
}

//---------------------------------------------------------------------------------------

void Game::update(int dialogs) {
	float x,y,z, dist;
	int i,n;
	Object o;

	if ((!dialogs)&&(hw->mouse->lbutton)&&(me.nbullet)&&
		 ((hw->camera->y+hw->camera->centery)>=waterHeight)) { // new mybullet ----
		if (fire_bullet()>-1) me.nbullet--;
	}

	//mybullet->update(); // update my bullets ---------------------------------
	if (mybullet->collision(him.x,
		                    him.y,
							him.z,him.size*.4)) { // hit?
		him.health-=10;
		n=rndn(20)+10;
		for(i=0; i<n; i++) {
			n=bloodp->create(him.x, him.y, him.z);
			//if (n>-1) bloodp->p[n].slickness=.5;
		}

		if (him.health<0) {
			ouch();
			announce("He Died...");
			him.health=100;
			him.x=mesh->rnd(1);
			him.y=12000;
			him.z=mesh->rnd(1);
			him.nlives--;
			if (him.nlives<=0) {
				him.nlives=0;
				him.health=0;
				me.health=100;
				announce("You Won the Match!");
				on=0;
			}
		}
	}
	//mybullet->draw(); // ----------------------------------------------------


	dist=sqrt(pow((hw->camera->x-him.x),2)+ // ----------- his -------------
			  pow((hw->camera->y-him.y),2)+
			  pow((hw->camera->z-him.z),2));
	aggr=(250/dist) + (1./(him.health*.0066+.33)) + difficulty -2;

	if ((!dialogs)&&(rndn(1)<aggr*2*hw->time->fDeltaTime)&&(him.y<200)&&
		((him.y+.5*him.size)>=waterHeight)) { // new his bullet
		x=him.x;
		y=him.y;
		z=him.z;
		n=hisbullet->create(x,y,z);
		if (n>-1) {
			Vector v;
			float d;

			hw->sound->play(hisfire,him.x,him.y,him.z);
			if (hw->sound->lastChannel>=0) {
				if (!hw->sound->hasReverb) {
					d=(1/(dist/500.))*.5;
					if (d>1) d=1;
					hw->sound->set_volume(d);
				}
				hw->sound->set_frequency(15000);
			}
			v.x=(hw->camera->x+rndn(1)-.5)-him.x;
			v.y=(hw->camera->y+rndn(6)-3)-him.y;
			v.z=(hw->camera->z+rndn(1)-.5)-him.z;
			v=v.normalize();
			hisbullet->p[n].xi=v.x*bulletSpeed;
			hisbullet->p[n].yi=v.y*bulletSpeed;
			hisbullet->p[n].zi=v.z*bulletSpeed;
		}
	}
	//hisbullet->update();

	if (hisbullet->collision(hw->camera->x,
		                     hw->camera->y,
							 hw->camera->z,6)) { // hit?
		me.health-=10;
		hw->camera->shakeampl=.5;
		hw->sound->play("hit.wav");

		n=rndn(20)+10;
		for(i=0; i<n; i++) {
			n=bloodp->create(hw->camera->x,
		                     hw->camera->y+hw->camera->centery,
							 hw->camera->z);
			if (n>-1) { // spurt forward
				bloodp->p[n].xi+=hw->camera->forwardx*50;
				bloodp->p[n].yi+=hw->camera->forwardy*50;
				bloodp->p[n].zi+=hw->camera->forwardz*50;
			}
		}

		if (me.health<0) {
			me.health=100;
			me.nlives--;
			announce("You Lost a Life...");
			if (me.nlives<=0) {
				me.nlives=0;
				announce("You Lost the Match!");
				on=0;
				beat_taunt();
			}
		}
	}
	//hisbullet->draw(); // -------------------------

	mobility=aggr*20;
	if (rndn(1)<20*aggr*hw->time->fDeltaTime) { 
		if (rndn(1)<.5) {if (rndn(1)<.5) him.xi+=mobility; else him.xi-=mobility;}
		else            {if (rndn(1)<.5) him.zi+=mobility; else him.zi-=mobility;} 
	}
	him.move(hw->time->fDeltaTime, hw->pref);
	him.drag(.01);
	mesh_grav(&him,0);
	draw_him(); // ----------------------------------------------------------

	o=*hw->camera;
	o.size=.1*o.size;
	if (me.nbullet<100) { // ammo pack --------------------------------
		if (pack.collision(&o,hw->pref)) {
			hw->sound->play(reload);
			me.nbullet+=pack.nbullet;
			if (me.nbullet>100) me.nbullet=100;
			pack.x=mesh->rnd(1);
			pack.y=15000;
			pack.z=mesh->rnd(1);
		}
	}
	if (me.nbullet<100) {
		pack.move(hw->time->fDeltaTime, hw->pref);
		mesh_grav(&pack,0);
		glPushMatrix();
			glTranslatef(0,.2*pack.size,0);
			pack.draw(hw->camera, hw->pref);
			glPopMatrix();
	} //-----------------------------------

	if (me.health<100) { // --------------------------medpak --------------
		if (med.collision(&o,hw->pref)) {
			hw->sound->play(chimes);
			med.x=mesh->rnd(1);
			med.y=2000;
			med.z=mesh->rnd(1);
			me.health=100;
		}
		med.move(hw->time->fDeltaTime, hw->pref);
		mesh_grav(&med,0);
		glPushMatrix();
			glTranslatef(0,.4*med.size,0);
			med.draw(hw->camera, hw->pref);
			glPopMatrix();
	} // ---------------------------------------------------
}

//-----------------------------------------------------------------------------

void Game::draw_him(void) {
	glPushMatrix();
		glTranslatef(0,.1*him.size,0);
		him.g=him.health*.01;
		him.b=him.health*.01;
		him.draw(hw->camera, hw->pref);
		glPopMatrix();
	return;
}

//-----------------------------------------------------------------------------

void Game::idle_taunt(void) {
	int r=-1;
	static int lastr=-1;

	while ((r==lastr)||(r==-1)) r=rndn(16);
	lastr=r; lastTauntTime=hw->time->fTime;
	switch(r) {
		case 0: hw->sound->play("allwrong.wav"); break;
		case 1: hw->sound->play("comeout.wav");  break;
		case 2: hw->sound->play("qstallin.wav"); break;
		case 3:	hw->sound->play("toads.wav"); break;
		case 4: hw->sound->play("sam17.wav"); break;//quit stallin roastin
		case 5: hw->sound->play("sam31.wav"); break;//sorry intrude
		case 6: hw->sound->play("sam41.wav"); break;//oo mm
		case 7: hw->sound->play("sam45.wav"); break;//i want u baby
		case 8: hw->sound->play("sam49.wav"); break;//come 2 papa
		case 9: hw->sound->play("sam62.wav"); break;//i c u never dealt w/ viking
		case 10: hw->sound->play("ltys_030.wav"); break;// getta going
		case 11: hw->sound->play("ltys_031.wav"); break;//quick do something
		case 12: hw->sound->play("ltys_032.wav"); break;//come back
		case 13: hw->sound->play("ltys_041.wav"); break;//i can c u keyhole
		case 14: hw->sound->play("ltys_058.wav"); break;//gimme that hat
		case 15: hw->sound->play("ltys_061.wav"); break;//getting warm in here
	}
}

//---------------------------------------------------------------------------------

void Game::taunt(void) {
	int r=-1;
	static int lastr=-1;

	while ((r==lastr)||(r==-1)) r=rndn(19);
	lastr=r; lastTauntTime=hw->time->fTime;
	switch(r) {
		case 0: hw->sound->play("allwrong.wav"); break;
		case 1:	hw->sound->play("gotcha.wav"); break;
		case 2:	hw->sound->play("mouthsht.wav"); break;
		case 3: hw->sound->play("nogood.wav"); break;
		case 4:	hw->sound->play("ooooh.wav"); break;
		case 5: hw->sound->play("qstallin.wav"); break;
		case 6:	hw->sound->play("toads.wav"); break;
		case 7:	hw->sound->play("what.wav"); break;
		case 8:	hw->sound->play("yankee.wav"); break;
		case 9:	hw->sound->play("yowwwww.wav"); break;
		case 10:	hw->sound->play("sam16.wav"); break;// missed agin
		case 11: hw->sound->play("sam36.wav"); break;//shutup bang bang
		case 12: hw->sound->play("sam41.wav"); break;//oo mm
		case 13:hw->sound->play("sam47.wav"); break;//u durn dude
		case 14: hw->sound->play("sam62.wav"); break;//i c u never dealt w/ viking
		case 15:hw->sound->play("ltys_005.wav"); break;//i hate you
		case 16: hw->sound->play("ltys_030.wav"); break;// getta going
		//case 11: hw->sound->play("ltys_031.wav"); break;//quick do something
		case 17: hw->sound->play("ltys_041.wav"); break;//i can c u keyhole
		case 18: hw->sound->play("ltys_058.wav"); break;//gimme that hat
	}
	
}

//---------------------------------------------------------------------------------

void Game::beat_taunt(void) {
	static int lastr=-1;
	int r=-1;

	while ((r==lastr)||(r==-1)) r=rndn(7);
	lastr=r; lastTauntTime=hw->time->fTime;
	switch(r) {
		case 0:	hw->sound->play("gdnight.wav"); break;
		case 1:	hw->sound->play("gotcha.wav"); break;
		case 2:	hw->sound->play("mouthsht.wav"); break;
		case 3:	hw->sound->play("sam14.wav"); break;//20 years tryin
		case 4:	hw->sound->play("sam16.wav"); break;// missed agin
		case 5:	hw->sound->play("sam38.wav"); break;//lay down arms
		case 6:	hw->sound->play("sam62.wav"); break;//i cu u never dealt w/ viking
	}
}

//--------------------------------------------------------------------------------

void Game::start(void) {
	static int lastr=-1;
	int r=-1;

	on=1;
	hw->camera->y=10;
	him.y=400;
	while ((r==lastr)||(r==-1)) r=rndn(11);
	lastr=r; lastTauntTime=hw->time->fTime;
	switch(r) {
		case 0:	hw->sound->play("blast.wav"); break;
		case 1:	hw->sound->play("charge.wav"); break;
		case 2:	hw->sound->play("doom.wav"); break;
		case 3:	hw->sound->play("ready2go.wav"); break;
		case 4:	hw->sound->play("sayprays.wav"); break;
		case 5:	hw->sound->play("sam21.wav"); break;//say ur prayers
		case 6:	hw->sound->play("sam46.wav"); break;//im ready
		case 7:	hw->sound->play("sam50.wav"); break;//ur eatin lead im warnin u
		case 8:	hw->sound->play("sam52.wav"); break;//lily liver
		case 9:	hw->sound->play("sam54.wav"); break;//stranger you upped 4 a hole
		case 10:hw->sound->play("sam57.wav"); break;//well see who chickens out 1st
	}
}

//---------------------------------------------------------------------------

void Game::ouch(void) {
	static int lastr=-1;
	int r=-1;

	while ((r==lastr)||(r==-1)) r=rndn(17);
	lastr=r; lastTauntTime=hw->time->fTime;
		switch(r) {
			case 0:	hw->sound->play("blashide.wav"); break;
			case 1:	hw->sound->play("furbear.wav"); break;
			case 2:	hw->sound->play("keelhaul.wav"); break;
			case 3:	hw->sound->play("mangy.wav"); break;
			case 4:	hw->sound->play("ooooh.wav"); break;
			case 5:	hw->sound->play("what.wav"); break;
			case 6:	hw->sound->play("yankee.wav"); break;
			case 7:	hw->sound->play("youllpay.wav"); break;
			case 8:	hw->sound->play("yowwwww.wav"); break;
			case 9: hw->sound->play("sam36.wav"); break;//shutup bang bang
			case 10:hw->sound->play("sam41.wav"); break;//oo mm
			case 11:hw->sound->play("sam47.wav"); break;//u durn dude
			case 12:hw->sound->play("sam55.wav"); break;//no more gentlemen stuff
			case 13:hw->sound->play("sam60.wav"); break;//oo ill blast ur head off
			case 14:hw->sound->play("ltys_005.wav"); break;//i hate you
			case 15:hw->sound->play("ltys_059.wav"); break;//u crazy galoot
			case 16:hw->sound->play("nogood.wav"); break;
		}
}

//---------------------------------------------------------------------------

void Game::end_taunt(void) {
	int r=rndn(5);
	switch(r) {
		case 0:	hw->sound->play("adios.wav"); break;
		case 1:	hw->sound->play("carcass.wav"); break;
		case 2:	hw->sound->play("gdnght2.wav"); break;
		case 3:	hw->sound->play("solong.wav"); break;
		case 4: hw->sound->play("ltys_039.wav"); break;//o what a night
	}
}

//--------------------------------------------------------------------------

void Game::annot(void) {
	float bsize=.04*hw->screen->height;
	char s[80];
	gen->alphatest=1; gen->alphafunc=.1;
	gen->italic=4;

	if (on) {
		gen->color(1,me.health/100.,me.health/100.); // my health
		sprintf(s,"%d",(int)me.health);
		gen->print(s,40,47,bsize);
		gen->color(1,1,1);
		gen->print("% MyHealth",40-10,12,10);

		gen->color(1,him.health/100.,him.health/100.); // his health
		sprintf(s,"%d",(int)him.health);
		gen->print(s,hw->screen->width*.76,47,bsize);
		gen->color(1,1,1);
		gen->print("% HisHealth",hw->screen->width*.76-10,12,10);


		gen->color(1,(float)me.nbullet/maxbullet,(float)me.nbullet/maxbullet); // my ammo
		sprintf(s,"%d",(int)me.nbullet);
		gen->print(s,hw->screen->width-80,47,bsize);
		gen->color(1,1,1);
		gen->print("Rounds",hw->screen->width-80-10,12,10);
	}

	if ((him.nlives!=maxlives)||(me.nlives!=maxlives)||on) {
		gen->color(1,(float)me.nlives/maxlives,(float)me.nlives/maxlives); // my lives
		sprintf(s,"%d",(int)me.nlives);
		gen->print(s,hw->screen->width*.33,47,bsize);//220
		gen->color(1,1,1);
		gen->print("My Lives",hw->screen->width*.33-35,12,10);

		gen->color(1,(float)him.nlives/maxlives,(float)him.nlives/maxlives); // his lives
		sprintf(s,"%d",(int)him.nlives);
		gen->print(s,hw->screen->width*.66,47,bsize);//400
		gen->color(1,1,1);
		gen->print("His Lives",hw->screen->width*.66-35,12,10);
	}
}

//-----------------------------------------------------------------------

int Game::fire_bullet(void) {
	float x,y,z;
	int n;
	/*
	x=hw->camera->x+hw->camera->rightx  *hand*20+
			        hw->camera->upx     *handHeight*20+
					hw->camera->forwardx*20;
	y=hw->camera->y+hw->camera->righty  *hand*20+
			        hw->camera->upy     *handHeight*20+hw->camera->centery+
					hw->camera->forwardy*20;
	z=hw->camera->z+hw->camera->rightz  *hand*20+
			        hw->camera->upz     *handHeight*20+
					hw->camera->forwardz*20;*/
	x=hand*20;
	y=handHeight*20;
	z=20;
	matmult(&x,&y,&z);
	x+=hw->camera->x;
	y+=hw->camera->y+hw->camera->centery;
	z+=hw->camera->z;

	n=mybullet->create(x,y,z);
	if (n>-1) {
		lastBulletTime=hw->time->fTime;
		hw->sound->play(myfire, hw->camera->x, hw->camera->y, hw->camera->z);
		if (!hw->sound->hasReverb) hw->sound->set_volume(1);
		//hw->sound->set_frequency(11000);
		mybullet->p[n].xi=hw->camera->forwardx*bulletSpeed;
		mybullet->p[n].yi=hw->camera->forwardy*bulletSpeed;
		mybullet->p[n].zi=hw->camera->forwardz*bulletSpeed;
	}
	return n;
}

//-----------------------------------------------------------------------------

void Game::draw_gun(void) {
	float z, t;

	t=(hw->time->fTime-lastBulletTime);
	if (t<.25) z=(.25-(hw->time->fTime-lastBulletTime))/.25;
	else       z=0;
	z=z*z;
	glPushMatrix();
		glTranslatef(hw->camera->x,
			         hw->camera->y+hw->camera->centery,
					 hw->camera->z);// orient to player
		glRotatef(-hw->camera->heading,0,1,0);
		glRotatef(-hw->camera->pitch,1,0,0);

		glTranslatef(10*hand,10*handHeight,-5+z*5);// recoil
		glRotatef(z*4,1,0,0);
		
		glColor3f(.25,.25,.25); glTranslatef(0, 0,-20); geom->draw_pill(1,10);// barrel
		glColor3f(.5,.25,0);    glTranslatef(0,-1, 20); geom->draw_pill(2,30);// mid
		glColor3f(.5,.5,.5);    glTranslatef(0,-4,  0); geom->draw_pill(4,15);// chamber
		glColor3f(.25,.125,.125);
		glTranslatef(0,-2, 15); 
		glRotatef(15,1,0,0);
		geom->draw_pill(3,25);// stock

		glPopMatrix();
}

//-----------------------------------------------------------------------------

void Game::draw_water(void) {
	float size=mesh->sizex*.5;

	glColor4f(0,1,1,hw->pref->a);
	glEnable(GL_BLEND);
	set_blend(hw->pref->blendFunc0,hw->pref->blendFunc1);
	if (hw->pref->textures) {
		glEnable(GL_TEXTURE_2D);
		waterTex->bind();
	}
	geom->texmagx=3*hw->pref->a1;
	geom->texmagy=3*hw->pref->a2;
	geom->texoffx=-hw->time->fTime*.05;
	geom->texoffy=-hw->time->fTime*.05;

	waterHeight=app->get(4,3,18,-20);//qqq
	glPushMatrix();
		glTranslatef(0,waterHeight,0);
		//geom.x0=-size; geom.z0=-size;	//geom.x1= size; geom.z1=-size;
		//geom.x2= size; geom.z2= size;	//geom.x3=-size; geom.z3= size;
		geom->draw_quad(-size,0,-size,size,0,-size,size,0,size,-size,0,size);
		glPopMatrix();
	glDisable(GL_BLEND);
	if (hw->pref->textures) glDisable(GL_TEXTURE_2D);
	return;
}

//-----------------------------------------------------------------------------

int Game::proc_teleport(void) {
	int newlev=0;
	Object o;

	o=*hw->camera;
	o.size*=.2;
	if (port.collision(&o,hw->pref)) { //  teleport ----------------
		hw->sound->play(cosmic);
		hw->pref->reverbMode=((int)rndn(26));
		hw->sound->set_reverb(hw->pref->reverbMode);
		new_level();
		port.x=((int)rndn(2)*2-1)*mesh->size*.5;
		port.y=15000;
		port.z=((int)rndn(2)*2-1)*mesh->size*.5;

		hw->camera->x=mesh->rnd(1);
		hw->camera->y=300;
		hw->camera->z=mesh->rnd(1);
		hw->camera->heading=rndn(360);
		newlev=1;
	}
	port.move(hw->time->fDeltaTime, hw->pref);
	mesh_grav(&port,0);
	hw->sound->set_loc(teleportChannel, port.x, port.y, port.z);
	glPushMatrix();
		glTranslatef(0,.2*port.size,0);
		port.draw(hw->camera, hw->pref);
		glPopMatrix(); // ----------------------------------
	return newlev;
}

//-------------------------------------------------------------------------------

void Game::draw_crosshair(void) {
	float y=-hw->camera->centery*.2;// offset for height
	if (on) {
		glColor3f(1,0,0);
		glLineWidth(2);
		glBegin(GL_LINES);
			glVertex2d(.45*hw->screen->width,.5*hw->screen->height+y); // crosshair x
			glVertex2d(.48*hw->screen->width,.5*hw->screen->height+y);
			glVertex2d(.52*hw->screen->width,.5*hw->screen->height+y);
			glVertex2d(.55*hw->screen->width,.5*hw->screen->height+y);

			glVertex2d(.5*hw->screen->width,.44*hw->screen->height+y); // crosshair y
			glVertex2d(.5*hw->screen->width,.47*hw->screen->height+y);
			glVertex2d(.5*hw->screen->width,.53*hw->screen->height+y);
			glVertex2d(.5*hw->screen->width,.56*hw->screen->height+y);

			glEnd();
		glLineWidth(1);
	}
}

//-------------------------------------------------------------------------------

void Game::new_level(void) {

	hw->pref->page   =rndn(.84)+.66;
	hw->pref->bracket=rndn(1.75)+.75;
	hw->pref->option=-1;
	while ((hw->pref->option    ==lastoption)  ||(hw->pref->option==-1))     hw->pref->option=rndn(maxoption+1);
	lastoption=hw->pref->option;

	hw->pref->image=-1;
	while ((hw->pref->image     ==lastimage)   ||(hw->pref->image==-1))      hw->pref->image =rndn(maximage+1);
	lastimage=hw->pref->image;

	hw->pref->universe=-1;
	while ((hw->pref->universe  ==lastuniverse)||(hw->pref->universe==-1))   hw->pref->universe=(int)rndn(3)+1;
	lastuniverse=hw->pref->universe;

	hw->pref->blendFunc0=-1;
	while ((hw->pref->blendFunc0==lastb0)      ||(hw->pref->blendFunc0==-1)) hw->pref->blendFunc0=rndn(9);
	lastb0=hw->pref->blendFunc0;

	hw->pref->blendFunc1=-1;
	while ((hw->pref->blendFunc1==lastb1)      ||(hw->pref->blendFunc1==-1)) hw->pref->blendFunc1=rndn(8);
	lastb1=hw->pref->blendFunc1;
	if (hw->pref->universe==1) {
		hw->pref->blank=rndn(2);
		hw->pref->c2=rndn(.6)+.3;
	}
	//setc();
	//make_bump();
}

//-------------------------------------------------------------------------------

void Game::matmult(float *inx, float *iny, float *inz) {
	float x,y,z;
	x=hw->camera->rightx  **inx+
	  hw->camera->upx     **iny+
	  hw->camera->forwardx**inz;
	y=hw->camera->righty  **inx+
	  hw->camera->upy     **iny+
	  hw->camera->forwardy**inz;
	z=hw->camera->rightz  **inx+
	  hw->camera->upz     **iny+
	  hw->camera->forwardz**inz;
	*inx=x;
	*iny=y;
	*inz=z;
}

//-------------------------------------------------------------------------------






