// Copyright (C) 2001 by George Ziniewicz
//
//  A continuing experiment in graphics and programming
// "main.cpp" has code specific/unique to THIS graphic or effect
// otherwise all common/generic routines (main, init, callbacks)
// that rarely change are "included" using "standard.cpp"
//----------------------------------------------------------------------

//#define new DEBUG_NEW

//#define  ortho

#define  hasstars
#define  hasparticles

#include "main.h"
#include "object.h"
#include "text.h"
#include "utility.h"
#include "texture.h"
#include "counter.h"

#if defined hasparticles
#include "particle.h"
#endif

#include "dialog.h"
#include "vector.h" 
#include "starfield.h"
#include "skybox.h"
#include "mesh.h"
#include "bump.h"
#include "geom.h"
#include "game.h"
#include "snd.h"

//------------------------------------------------------------------------------------
#define delayms 0
#define maxlight 0
#define canBlank 0

#define mintess 20
#define maxtess 59

#define maxn 13

enum dialognames {MAIN,OPTIONS,SAVE,LOAD,
                  THIS,COLORLIGHTING,SOUND,LIGHTMAP,GENERAL};

//extern void perlin_do(float persistence, int level);

// this app global objects ------------------------------------------------
char titlestr[]="YosemiteZ";

Object sun, lightmap;
Text annot, title, check, gen, dialogText;
Clock clock;
Pref pref;
Camera camera;
Time time;
Mouse mouse;
Key key;
Counter cameraRotate, cameraMove, lightRotate, app, textEffect;
Light light;
Screen screen;
Skybox skybox;
Mesh mesh;
Bump bump;
Geom geom;
Game game;
Snd sound;

#if defined hasparticles
Particle mybullet, hisbullet, blood, grass, dust, spark, splash;
#endif

Texture font, smoke, sphere, sand, perlin, metal, star, sunMap, shatter;
Texture starMap, controls, images[9];
Texture logo, lmsphere, figure, cloud, grassTex, cross;

Dialog dialogObject, *dialogCurrent=&dialogObject;

Hardware hardware;

# if defined (hasstars)
Starfield starfield;
#endif

float beginTime=5, clearance, skip=0;
int prefview=1, zoomon=0;

float bumparray[65536];
float bumparray2[65536];

struct detail{//texture info: mag, mesh colors, particle colors, blends
	float texmagx,texmagy, r,g,b, rp,gp,bp, b0,b1;
} image_details[10]={
	20,20,.60,.30,.10, .6, .3, .1, 2,3, //sand
	 7, 7,  1,  1,  1, .5,.25, .1, 2,3, //dirt
	10,10,.75,.60,.20,  0,  1,  0, 2,3, //grass
	20,20,  1,  1,  1, .6,.60,.10, 2,3, //floor
	20,20,.95,.90,.90,  1,  1,  1, 2,3, //marble
	40,40, 1,1,1,       1,1,1,     2,3,// rock
	20,20, 1,1,1,      .6,.6,.6,   2,3,// metal
	20,20, 1,1,1,       0,1,1,     2,3,// brass
	10,10, 1,1,1,       1,1,1,     2,3 // check
};

// this app protos ------------------------------------------------------------

void dialog_run(void);
void set_suncolor(void);
void info(char *titlestr);
void lm(void);
void intro(void);
void doc_lightmap(void);
void make_bump(void);
float mesh_grav(Object *object,int fall);
void announce(char *s);
void update_mesh(void);
void update_lm(void);
void test_mesh(Particle *particle);
void test_mesh_stick(Particle *particle);
void setc(void);
void zoom(void);
void frame(float x0, float y0, float x1, float y1);
void change_world(void);

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------

// main has code specific to THIS graphic effect
// otherwise all common routines (main, init, callbacks) are in standard.cpp
#include "standard.cpp"

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//---- only code exclusive to "this app" from here down ---------------------

void do_title(void) {
   short x=20,y, yinc=-15, size=8;

   y=screen.height*.92;//.09;
	gen.reset();
	gen.blend=0;
	gen.alphatest=0;

   gen.print(titlestr, x,y, 15);
   x+=10; y+=yinc*1.5;
	gen.print("Special Controls for this (32 bitcolor!) simulation:",        x,y,size); y+=yinc;
 	//gen.print("           1-9,0  Adjust # * 100, 0=1000",                 x,y,size); y+=yinc;
	gen.print(" <PageUp/Down>  Adjust Ground Height",                       x,y,size); y+=yinc;
	gen.print("            []  Adjust Ground Size",                         x,y,size); y+=yinc;
	gen.print("      shift <>  Adjust Ground Tesselation",                  x,y,size); y+=yinc;
	gen.print("             i  Images: Cycle Ground Textures",             x,y,size); y+=yinc;
	gen.print("             o  Options: Cycle Cycle Ground Heightmaps",           x,y,size); y+=yinc;
	gen.print("<F1/F2><F3/F4>  Adjust Ground Texture X/Y Magnification",    x,y,size); y+=yinc;
	gen.print("           b/B  BlendFunc0/1: Cycle Blends",            x,y,size); y+=yinc;
	y-=5;
	gen.print("  Manual Camera:",                                           x,y,size); y+=yinc;
	gen.print("    <MouseX/Y>  MouseLook",     x,y,size); y+=yinc;
	gen.print("     <Up/Down>  Move Forward/Reverse, precede with <control> for Up/Down",x,y,size); y+=yinc;
	gen.print("  <Left/Right>  Move (Sidestep) Left/Right",                             x,y,size); y+=yinc;
	y-=5;
	gen.print("  FullScreen Lightmaps:",                                           x,y,size); y+=yinc;
	gen.print("      shift  L  Lightmaps: Toggle On/Off",                  x,y,size); y+=yinc;
	gen.print("  shift-alt  L  Lightmaps: Cycle None/Vignette/X-Y-Z Noise/X-Y Grating",  x,y,size); y+=yinc;
	gen.print("    <Home/End>  Adjust Lightmap Noise Level",                   x,y,size); y+=yinc;
	gen.print("      <F9/F10>  Adjust Lightmap Vignette Size",                 x,y,size); y+=yinc;

	//gen.print("      <MouseX/Y>  MouseLook in Manual ViewCam",     x,y,size); y+=yinc;
	//gen.print("    <LeftButton>  Move Forward in Manual ViewCam",                              x,y,size); y+=yinc;
	//gen.print("        <Arrows>  Move in Manual ViewCam, Rotate in Auto ViewCam",           x,y,size); y+=yinc;
	//gen.print("               m  Matrix: Demo Effect",                         x,y,size); y+=yinc;
# if defined (hasstars)
	gen.print("             u  Universe: Cycle None/Stars/FixedSkyBox/MovableSkyBox",          x,y,size); y+=yinc;
#endif
	gen.print("             x  Explode Screen Toggle (try with Hyper<Space>!)",x,y,size); y+=yinc;
	//gen.print("           1-3  Adjust # of ",                              x,y,size); y+=yinc;
	y-=5;
	gen.print("Standard Controls in most simulations where applicable:",     x,y,size); y+=yinc;
	gen.print("`/<RightButton> Toggle Current Popup Menu",                   x,y,size); y+=yinc;
 	gen.print(" <MiddleButton> Toggle Sniper Zoom",                          x,y,size); y+=yinc;
	//gen.print("     <control>+ Convert ArrowKey x/z to x/y temporarily",     x,y,size); y+=yinc;
	//gen.print("         <alt>+ Enable 10x Thrust, <alt>+Arrows",             x,y,size); y+=yinc;
	//gen.print("       <F5/F6>  Adjust Fog Start Distance",                   x,y,size); y+=yinc;
	//gen.print("       <F7/F8>  Adjust Fog End Distance",                     x,y,size); y+=yinc;
 	//gen.print("   <LeftButton> Move Forward (in Manual Camera)",               x,y,size); y+=yinc;
	gen.print("     <F11/F12>  Adjust Screen Fade Effect (0-1)",             x,y,size); y+=yinc;
	//y+=yinc*.3;
	//gen.print("  unshifted -+  Adjust Universe Rotate Speed",                x,y,size); y+=yinc;
	//gen.print("    shift   -+  Adjust Universe Movement Speed",              x,y,size); y+=yinc;
	if (maxlight>-1) {
	//gen.print("    shift   ()  Adjust Light Rotate Speed",                   x,y,size); y+=yinc;
	}
	gen.print("  unshifted <>  Adjust Screen Magnification",                 x,y,size); y+=yinc;
	gen.print("    shift   {}  Adjust Closeness (Camera Z)",                 x,y,size); y+=yinc;
	gen.print("             c  Clock: Toggle Frame-Per-Seconds",             x,y,size); y+=yinc;
	gen.print("             f  FullScreen/Window Toggle",                    x,y,size); y+=yinc;
	gen.print("             l  Letterbox: Adjust Size 0-6",                  x,y,size); y+=yinc;
	gen.print("             p  Pause: Toggle On/Off",                        x,y,size); y+=yinc;
	gen.print("    <ESC> or q  EXIT after fade, hit twice for quick exit",   x,y,size); y+=yinc;
	if (maxlight>-1) {
	gen.print("             r  Reset/Randomize all parameters              ",x,y,size); y+=yinc;
	gen.print("             s  Spotlite: Toggle On-Off",                     x,y,size); y+=yinc;}
	gen.print("             t  Texture:  Toggle On-Off",                     x,y,size); y+=yinc;
	gen.print("             v  ViewCam: Cycle Control: Auto/Manual/Hold/Auto-Centered",x,y,size); y+=yinc;//Auto
	gen.print("             w  Wireframe/Points/Polygon Mode Cycle",        x,y,size); y+=yinc;
	gen.print("     shift   !  Save Preferences",                            x,y,size); y+=yinc;
	gen.print("     shift   @  Load Preferences",                            x,y,size); y+=yinc;
	gen.print("       <SPACE>  Toggle HyperSpace",                           x,y,size); y+=yinc;
	gen.print("   -------------------------------------------------------- ",x,y,size); y+=yinc;
	gen.print("   Copyright (C) 2001 by George Ziniewicz    zin@zintel.com", x,y,size); y+=yinc;
	gen.print("         www.zintel.com for 3-d graphics shareware",          x,y,size); y+=yinc;
	gen.print("   <?> to return",                                            x,y,size); y+=yinc;
	return;
	}

//----------------------------------------------------------------------------

void key_this(unsigned char inkey) {
	static float lastpage=1, lasthome=1;

	switch(inkey) {
			//case '1': case '2': case '3': case '4': case '5': 
			//case '6': case '7': case '8': case '9': case '0':
			//if (inkey=='0') n=1000;
			//else            n=(inkey-'0')*100;
			//annot.hold("# Birds: %d",n); break;
			//break;

		case 'b':
			pref.blendFunc0=(pref.blendFunc0>=8)? 0 :pref.blendFunc0+1;
			doc_blend(0,pref.blendFunc0,&annot);
			break;
		case 'B':
			pref.blendFunc1=(pref.blendFunc1>=7)? 0 :pref.blendFunc1+1;
			doc_blend(1,pref.blendFunc1,&annot);
			break;

		case 'i':
			mesh.tex=&images[pref.image];
			setc();
			break;

		case 'L':
			if (!key.alt) pref.lightmap=-pref.lightmap;
			else if (pref.lightmap>-1) {
				pref.lightmap=((pref.lightmap)+1) % 64;
				doc_lightmap();
			}
			if (pref.lightmap>0) annot.hold("Lightmap On");
			else                 annot.hold("Lightmap Off");
			break;

		case 'n':// general purpose int toggle
			pref.n=(pref.n>=maxn)? 0 : pref.n+1;
			annot.hold("n: %d",pref.n);
			break;
		case 'N':
			pref.n2=(pref.n2>=9)? 0 : pref.n2+1;
			annot.hold("n2: %d",pref.n2);
			break;

		case 'o':
			make_bump();
			update_mesh();
			doc_option();
			break;

		case 'u': case 'U':
			pref.universe=(pref.universe+1)%4;
			switch(pref.universe) {
				case 0: annot.hold("Universe Off");      set_fogcolor(0,0,0,1); break;
				case 1: annot.hold("Star Universe On");  set_fogcolor(0,0,0,1); break;
				case 2: annot.hold("Fixed SkyBox On");   set_fogcolor(1,1,1,1); break;
				case 3: annot.hold("Movable SkyBox On"); set_fogcolor(1,1,1,1); break;
			}
			break;

		case 'x': case 'X':
			if (capture==2) {
				capture=0;
				screen.needblank=1;
			}
			else capture=1;
			break;

		case '[': case ']':
			change_world();
			break;

		case '<': case '>':
			mesh.resx=pref.tess;
			mesh.resy=pref.tess;
			update_mesh();
			break;

		case '@':
			doc_option();
			switch(pref.universe) {
				case 0: set_fogcolor(0,0,0,1); break;
				case 1: set_fogcolor(0,0,0,1); break;
				case 2: set_fogcolor(1,1,1,1); break;
				case 3: set_fogcolor(1,1,1,1); break;
			}
			screen.needblank=1;
			pref.pause=0;
			camera.fov=pref.fov;
			mesh.tex=&images[pref.image];
			make_bump();
			update_mesh();
			break;

		case '/': case '?':
			dialogs=0; infoOn=0;
			mouse.set_cursor(&pref, dialogs);
			break;

		case 127:  // delete key
			camera.yi+=40000*time.fDeltaTime;
			break;

	} // of switch(key)

	if (pref.page != lastpage) {
		lastpage=pref.page;
		update_mesh();
	}
	if ((pref.home != lasthome)&&(introDone)) {
		lasthome=pref.home;
		//sound.lastChannel=game.teleportChannel;
		//sound.set_frequency(10000*pref.home);
	}


	if (inkey) {
		if (inkey=='r') dialog_init(dialogCurrent->ID,1); // replace dialog
		else            dialog_init(dialogCurrent->ID,0); // update dialog
	}

	if (ex) {
		FSOUND_StopSound(FSOUND_ALL);
		sound.hasReverb=0;
		game.end_taunt();
		game.on=0;
	}
	return;
}

//---------------------------------------------------------------------------

void init_pref(void) {
	static int init=0;

	log_message("Init pref...");

	pref.arrowx=1;
	pref.arrowy=1;
	pref.arrownx=0;
	pref.arrowny=0;
	pref.brace=0;
	pref.bracket=1;
	pref.fclock=0;
	pref.fog=0;
	pref.fov=90;
	//pref.image=0;
	pref.letterbox=2;
	pref.lights=1;
	pref.lmInterval=.075;
	pref.mouseSens=1;
	pref.moveSens=1;
	pref.n=0;
	//pref.option=0;
	pref.pan=.5;
	pref.page=1;
	pref.paren=1;
	pref.pause=0;
	pref.reverbMode=0;
	pref.spd1=1;
	pref.spd2=1;
	pref.tess=30;
	pref.textures=1;
	pref.volume=1;
	pref.pan=.5;
	pref.wireframe=0;

	pref.a1=1;
	pref.a2=1; 
	pref.b1=.1*FARZ;
	pref.b2=FARZ;    //worldSize;  // fog dist                           
	pref.c2=.3; // fade effect

	pref.brightness=1;
	pref.map=0;

	pref.ambient=.4;    pref.diffuse=.5;    pref.specular=.99;
	pref.reflective=.99;pref.shininess=10;
	pref.cutoff=180;    pref.exponent=1;

	pref.r=.75; pref.g=.75; pref.b=.75; pref.a=1;

	//pref.toggle[0]=0;
	pref.value[0]=1;
	pref.value[1]=1;

	if (!init) {
		pref.blank=0;
		pref.c1=.7;
		pref.home=4;
		pref.lightmap=8;
		pref.lmtextureMagx=1;
		pref.lmtextureMagy=1;
		pref.universe=1;
		pref.view=2;
		init=1;
	}
	else {
		pref.blank=1;
		pref.c1=1;
		pref.home=1;
		pref.lightmap=4;
		pref.lmtextureMagx=3;
		pref.lmtextureMagy=3;
		pref.view=prefview;
		game.new_level();
		change_world();

		pref.blendFunc0=4;
		pref.blendFunc1=2;
		setc();
	}
	return;
}

//-----------------------------------------------------------------------------

void init_once(void) { // onetime inits
	int err=0;

	log_message("Init Once...");

	worldSize=FARZ*pref.bracket;
	screen.width =glutGet(GLUT_SCREEN_WIDTH);
	screen.height=glutGet(GLUT_SCREEN_HEIGHT);
	screen.set_letterbox(pref.letterbox);
	screen.set_viewport();

	font.loadtexInt("font_256.bmp", -1,   0, 0);
	//metal.loadtex("metalplate128.bmp",255,0, 1);
	sphere.maketextypeint(  SPHERE,0,64,0);     // NOTSPHERE
	lmsphere.maketextypeint(SPHERE,0,64,200);
	shatter.maketextype2(SPHERE,0);

#if defined (hasparticles)
	smoke.maketextype(SMOKE,0); // procedural smoke
	//sunMap.maketextype2(SPHERE,0); // procedural sphere/sun overwrite
//	sunflare(sunMap.texID);
#endif

	//.maketextype(GRADIENT,0); // procedural vertical gradient
	//.maketextype(HALO,0);     // procedural halo
	//.maketextype(GRID,0);     // procedural grid
	//.maketextype(CHECK,0);    // procedural checkerboard
	//starMap.maketextype(STARFIELD,0);// procedural starfield

	//star.maketextype(STAR,  0); // procedural star
	//sand.maketextype(SAND,  0); // procedural sand
	perlin.maketextype(PERLIN,0);   // procedural noise
	cross.maketextype(CROSS,0,80,0);   // procedural noise
	
	//                       file                alpha If > threshold mip? size
	//loadtex("metalplate128.bmp", -1,        0,        1);

	controls.loadtex("controls_128.bmp", -1,0, 1);
	//spark.loadtex("cloudbw2128.bmp", -1,0, 1);
	//mainTex.loadtex("external.bmp",     -1,0, 0);
	//spark.maketextype(SPARK,0);
	
	//images[0].loadtex("sand.bmp",   -1,0, 0);
	images[0]=perlin;
	images[1].loadtex("dirt128.bmp",-1,0, 1);
	images[2].loadtex("grass.bmp",  -1,0, 1);
	images[3].loadtex("floor.bmp",  -1,0, 1);
	images[4].loadtex("marblewhite128.bmp",  -1,0, 1);
	images[5].loadtex("rocks128.bmp",  -1,0, 1);
	images[6].loadtex("metalplate128.bmp",  -1,0, 1);
	images[7].loadtex("brasstarnish128.bmp",  -1,0, 1);
	images[8].maketextype(CHECK,0);

	sunMap.loadtex("sun_128.bmp",-1,0,0);
	logo.loadtex("zin24.bmp",-1,0,0);
	figure.loadtex("yos_sam_1.bmp",-1,0,0);
	cloud.loadtex("cloudbw2128.bmp",-1,0,0);
	grassTex.loadtex("grass_blend.bmp",     -1,0, 1);

# if defined (hasstars)
	starfield.nlevel=2;//1-3
	starfield.size=600;
	starfield.hollow=400;
	starfield.blend=1;
	starfield.depthtest=0;  
	starfield.depthwrite=0;
	starfield.type=1; // 0round/1cube/2dome
	starfield.init(2000/starfield.nlevel);// init volume univ
#endif

	hardware.camera=&camera;
	hardware.counter=&app;
	hardware.key=&key;
	hardware.mouse=&mouse;
	hardware.pref=&pref;
	hardware.sound=&sound;
	hardware.screen=&screen;
	hardware.gen=&gen;
	hardware.time=&time;

	Object::hw=&hardware;

	Widget::tex=&controls;// init dialog textureIDs
	Widget::text=&dialogText;
	dialogText.x=0;
	dialogText.y=0;
	//dialogText.alphatest=0;
	dialogText.alphafunc=.05;
	dialogText.blend=0;

	dialogCurrent->key=&key;
	dialogCurrent->mouse=&mouse;
	dialogCurrent->text=&dialogText;
	dialogCurrent->time=&time;
	dialog_init(MAIN,1);

	Text::tex=&font;
	Text::time=&time;
	Text::counter=&textEffect;
	Text::screen=&screen;

	annot.x=20;
	annot.y=screen.height-20;
	annot.size=10;
	annot.alphatest=0;
	annot.alphafunc=.1;
	annot.blend=0;

	check.x=.4*screen.width;
	check.y=screen.height-20;
	check.color(1,1,0);

	title.bloom=1;
	//title.bend=1;
	title.drift=1;
	title.color(0,1,0);
	title.bloomamplx=1;
	title.bloomamply=1;

	camera.speedLimit=1000;
	camera.maxthrust=FARZ*.3;
	camera.moveRangex=180;
	camera.moveRangey=40;
	camera.moveRangez=180;
	camera.pitchRange=40;
	camera.headingRange=720;
	camera.rollRange=400;
	camera.moveSpeed=8;
	camera.rotateSpeed=1.5;
	camera.centerx=0;
	//camera.centery=10;
	camera.centerz=0;
	camera.pref =&pref;
	camera.time =&time;
	camera.mouse=&mouse;
	camera.cameraMove  =&cameraMove;
	cameraMove.init(6); // setup all random counters
	camera.cameraRotate=&cameraRotate;
	cameraRotate.init(3);
	camera.cameraRotate->toggleRate=.2;
	camera.cameraRotate->newRate=.2;
	camera.cameraRotate->damp=.2;
	camera.cameraMove->toggleRate=.2;
	camera.cameraMove->newRate=.04;
	camera.cameraMove->damp=.2;

	lightRotate.init(4);
	lightRotate.fixed[1]=1;
	textEffect.init(8);

	lightmap.alphatest=0;
	lightmap.depthtest=0;
	lightmap.depthwrite=0;
	lightmap.sizex=screen.width*.5;
	lightmap.sizey=screen.height*.5;//lightmap.sizex;
	lightmap.x=.5*screen.width;
	lightmap.y=.5*screen.height;
	lightmap.z=1;
	lightmap.tex=&lmsphere;
	lightmap.faceScreen=0;
	//lightmap.rolli=.03;
	lightmap.blendFunc0=6;
	lightmap.blendFunc1=0;

	light.r=1;
	light.g=1;
	light.b=1;

#if defined hasparticles
	Particle::time  =&time;
	Particle::pref  =&pref;
	Particle::camera=&camera;
	particle_init();
#endif

	screen.camera=&camera;
	screen.pref=&pref;
	screen.r=0;
	screen.g=0;
	screen.b=0;

	sun.blend=0;
	sun.depthtest=1;
	sun.depthwrite=1;
	sun.sizex=10;
	sun.sizey=sun.sizex;
	sun.sizez=sun.sizex;
	sun.tex=&sunMap;
	sun.faceScreen=1;
	sun.b=0;

	skybox.hw=&hardware;
	skybox.init();

	bump.bumparray=bumparray;
	bump.bumparray2=bumparray2;

	mesh.blend=0;
	mesh.alphatest=0;
	mesh.resx=pref.tess;
	mesh.resy=pref.tess;
	mesh.r=1;
	mesh.g=1;
	mesh.b=1;
	mesh.x=0;
	mesh.y=0;
	mesh.z=0;
	mesh.size=1000;
	mesh.sizex=mesh.size;
	mesh.sizey=mesh.size;
	mesh.texOffsetx=0;
	mesh.texOffsety=0;
	mesh.tex=&sphere;

	//make_bump();	update_mesh();

	game.gen=&gen;
	game.him.tex=&figure;// textures
	game.pack.tex=&sphere;
	game.port.tex=&sphere;
	game.med.tex=&cross;
	game.waterTex=&cloud;
	game.mesh=&mesh;
	game.geom=&geom;
	game.app=&app;

	game.mybullet =&mybullet;// particles
	game.hisbullet=&hisbullet;
	game.bloodp   =&blood;

	sound.hw=&hardware;
	sound.worldSize=mesh.size;
	err=sound.init();
	if (!err) {
		game.myfire  =FSOUND_Sample_Load(FSOUND_FREE,"cannon.wav", FSOUND_2D,0);
		game.hisfire  =FSOUND_Sample_Load(FSOUND_FREE,"cannon.wav", sound.type,0);
		//game.fire  =FSOUND_Sample_Load(FSOUND_FREE,"weapon.wav",  sound.type,0);
		game.splash=FSOUND_Sample_Load(FSOUND_FREE,"little_splash.wav",sound.type,0);
		game.rico1 =FSOUND_Sample_Load(FSOUND_FREE,"rico2.wav",   sound.type,0);
		game.rico2 =FSOUND_Sample_Load(FSOUND_FREE,"ricochet.wav",sound.type,0);
		game.twing =FSOUND_Sample_Load(FSOUND_FREE,"twing.wav",   sound.type,0);
		game.chimes=FSOUND_Sample_Load(FSOUND_FREE,"chimes.wav",  FSOUND_2D,0);
		game.cosmic=FSOUND_Sample_Load(FSOUND_FREE,"cosmicl.wav", sound.type,0);
		game.reload=FSOUND_Sample_Load(FSOUND_FREE,"weapon1.wav", sound.type,0);
		FSOUND_SetSFXMasterVolume(255);

		game.wind  =FSOUND_Sample_Load(FSOUND_FREE,"engine.wav",  sound.type,0);//wind2//engine
		FSOUND_Sample_SetLoopMode(game.wind,FSOUND_LOOP_NORMAL);
		FSOUND_Sample_SetMinMaxDistance(game.wind, 1,1000000);// set sample volume range
		FSOUND_Sample_SetDefaults(game.wind,10000,60,-1,-1);

		sound.play("wind2.wav",0,0,0);
	}
	return;
}

//--------------------------------------------------------------------

void shutdown_this(void) {
	if (sound.inited) {
		sound.free_sample(game.myfire);
		sound.free_sample(game.hisfire);
		sound.free_sample(game.splash);
		sound.free_sample(game.rico1);
		sound.free_sample(game.rico2);
		sound.free_sample(game.twing);
		sound.free_sample(game.chimes);
		sound.free_sample(game.cosmic);
		sound.free_sample(game.reload);
		sound.free_sample(game.wind);
		sound.deinit();
	}
	//score.finalSave(titlestr);
	return;
}

//--------------------------------------------------------------------

#if defined hasparticles

void particle_init(void) {
	Particle p;

	log_message("Particle Init...");

	//------------------------------------------------ dust
	p.lifeTime=2; p.interval=0; p.lifeTimeRnd=.1;
	p.size0=1;    p.size1=25;  p.sizeRnd=.1; p.length=0;
	p.windx=0;    p.windy=0;   p.windz=0;  p.dispersex=10; p.dispersey=0; p.dispersez=10;
	p.xi=0;       p.yi=20;      p.zi=0;    p.canroll=1;   p.rollspeed=80; 
	
	p.tex=&smoke;
	p.r0=.5;     p.g0=.25;   p.b0=.1;  p.colorRnd=0; // color
	p.r1=.5;	 p.g1=.25;   p.b1=.1;  p.inten0=1; p.inten1=1;
	p.alpha0=1; p.alpha1=0; p.alphatest=1; p.alphafunc=.01;
	p.blend=1;     p.blendFunc0=2;  p.blendFunc1=3;
	p.depthtest=1; p.depthwrite=0;	p.init=1;
	dust=p;

	//------------------------------------------------ spark
	p.lifeTime=2; p.interval=0; p.lifeTimeRnd=.1;
	p.size0=2.5;  p.size1=1;    p.sizeRnd=2; p.length=0;
	p.windx=0;    p.windy=-200; p.windz=0;   p.dispersex=60; p.dispersey=40; p.dispersez=20;
	p.xi=0;       p.yi=100;      p.zi=0;      p.canroll=0;   p.rollspeed=0;

	p.tex=0;
	p.r0=1;     p.g0=.5;   p.b0=.25;  p.colorRnd=.4;
	p.r1=1;	    p.g1=.5;   p.b1=.25;  p.inten0=1; p.inten1=1;
	p.alpha0=1; p.alpha1=0; p.alphatest=0; p.alphafunc=.01;
	p.blend=1;     p.blendFunc0=2;  p.blendFunc1=3;
	p.depthtest=1; p.depthwrite=0;	p.init=1;
	spark=p;
	//------------------------------------------------ splash
	p.lifeTime=1; p.interval=0; p.lifeTimeRnd=.1;
	p.size0=2.5;  p.size1=1;    p.sizeRnd=2; p.length=0;
	p.windx=0;    p.windy=-200; p.windz=0;   p.dispersex=70; p.dispersey=50; p.dispersez=40;
	p.xi=0;       p.yi=100;      p.zi=0;      p.canroll=0;    p.rollspeed=0;

	p.tex=0;
	p.r0=.5;     p.g0=.75;   p.b0=1;  p.colorRnd=.1; 
	p.r1=.5;	 p.g1=.75;   p.b1=1;  p.inten0=1; p.inten1=1;
	p.alpha0=1; p.alpha1=0; p.alphatest=0; p.alphafunc=.01;
	p.blend=1;     p.blendFunc0=2;  p.blendFunc1=3;
	p.depthtest=1; p.depthwrite=0;	p.init=1;
	splash=p;
	//------------------------------------------------ bullet
	p.lifeTime=2.5; p.interval=0;  p.lifeTimeRnd=.1; // life
	p.size0=8;     p.size1=1;     p.sizeRnd=0;     p.length=30;// space related
	p.windx=0;     p.windy=-1;     p.windz=0; p.dispersex=0; p.dispersey=0; p.dispersez=0;
	p.xi=0;        p.yi=0;        p.zi=0; p.canroll=0;   p.rollspeed=0;

	p.tex=0; // image related
	p.r0=1;     p.g0=.5;   p.b0=.25;  p.colorRnd=0; // color
	p.r1=1;	    p.g1=.5;   p.b1=.25;  p.inten0=1; p.inten1=1;
	p.alpha0=1;    p.alpha1=1;	p.alphatest=1; p.alphafunc=.1;
	p.blend=0;     p.blendFunc0=2;  p.blendFunc1=3;
	p.depthtest=1; p.depthwrite=0;  p.init=1;// misc
	mybullet=p;  mybullet.interval=.25;
	hisbullet=p; hisbullet.interval=.25;
	//------------------------------------------------ grass
	p.lifeTime=0; p.interval=0;  p.lifeTimeRnd=0; // life
	p.size0=4;    p.size1=4;    p.sizeRnd=0; p.length=0;// space related
	p.windx=0;     p.windy=0;   p.windz=0; p.dispersex=0; p.dispersey=0; p.dispersez=0;
	p.xi=0;        p.yi=0;        p.zi=0;  p.canroll=0;   p.rollspeed=0;

	p.tex=&grassTex;       // image related
	p.tx0=.01;	p.ty0=.01;
	p.tx1=.99;	p.ty1=.99;
	p.r0=1;     p.g0=1;   p.b0=1;  p.colorRnd=0; // color
	p.r1=1;	    p.g1=1;   p.b1=1;  p.inten0=1; p.inten1=1;
	p.alpha0=1;    p.alpha1=1;	p.alphatest=1; p.alphafunc=.1; // transparency
	p.blend=0;     p.blendFunc0=1;  p.blendFunc1=0;
	p.depthtest=1; p.depthwrite=1; p.init=1;// misc
	grass=p;

	//------------------------------------------------ blood
	p.lifeTime=3; p.interval=0; p.lifeTimeRnd=.1;
	p.size0=2;  p.size1=.1;    p.sizeRnd=2; p.length=2;
	p.windx=0;    p.windy=-300; p.windz=0;   p.dispersex=100; p.dispersey=300; p.dispersez=100;
	p.xi=0;       p.yi=40;      p.zi=0;      p.canroll=0;    p.rollspeed=0;

	p.tex=&sphere;
	p.r0=1;     p.g0=0;   p.b0=0;  p.colorRnd=.1; 
	p.r1=1;	    p.g1=0;   p.b1=0;  p.inten0=1; p.inten1=1;
	p.alpha0=1; p.alpha1=0; p.alphatest=0; p.alphafunc=.01;
	p.blend=1;     p.blendFunc0=2;  p.blendFunc1=3;
	p.depthtest=1; p.depthwrite=0;	p.init=1;
	blood=p;

	return;
}/*
	//------------------------------------------------ flame
	p.lifeTime=1; p.interval=.05; p.lifeTimeRnd=.1;
	p.size0=1;    p.size1=10;  p.sizeRnd=.1; p.length=0;
	p.windx=0;    p.windy=0;   p.windz=0;  p.dispersex=20; p.dispersey=20; p.dispersez=20;
	p.xi=0;       p.yi=0;      p.zi=0;
	

	p.tex=&smoke;
	p.r0=1;     p.g0=.5;   p.b0=.25;  p.colorRnd=.4;
	p.r1=1;	    p.g1=.5;   p.b1=.25;
	p.alpha0=1; p.alpha1=0; p.alphatest=1; p.alphafunc=.01; p.inten0=1; p.inten1=1;

	p.blend=1;     p.blendFunc0=2;  p.blendFunc1=3;
	p.depthtest=1; p.depthwrite=0;
	p.canroll=1;   p.rollspeed=5;
	p.init=1;
	flame=p;
*/

#endif

//-----------------------------------------------------------------------------

void init_vars(void) { // repeatable reset
	log_message("Init Vars...");
	static int init=0;
//.............................................standard stuff
	srand(time.msTime);

	dialogs=0; ex=0; capture=0;

	camera.x=0;     camera.y=0;        camera.z=0;
	camera.xi=0;    camera.yi=0;       camera.zi=0;
	camera.pitch=0; camera.heading=0;  camera.roll=0;

//.............................................this stuff

	app.init(9);
	//app.on[   0]=1; // constant counter move
	//app.inc[  0]=.01;
	//app.fixed[0]=1;

	cameraMove.init(6);
	cameraRotate.init(3);

	if (!init) {
		camera.x=0;
		camera.y=0;
		camera.z=0;
		camera.pitch=0;
		camera.heading=0;
		camera.roll=0;
		camera.centery=0;
		init=1;
	} else {
		camera.x=0;
		camera.y=0;
		camera.z=0;
		camera.xi=0;
		camera.yi=0;
		camera.zi=0;
		camera.pitch=0;
		camera.heading=0;
		camera.roll=0;
		camera.centery=0;
	}
	screen.farz=FARZ*pref.bracket;
	game.init();
	mouse.set_cursor(&pref, dialogs);
	set_fog(pref.universe);
	mesh.size =1000*pref.bracket;
	mesh.sizex=mesh.size;
	mesh.sizey=mesh.size;
	make_bump();
	update_mesh();
	return;
}

//-----------------------------------------------------------------------------

void doc_image(void) {
	switch(pref.image) {
		case 0:
			if (maximage==0) annot.hold("No Image Options");
		else     annot.hold("Image 0/8: Sand");
		break;
		case 1:  annot.hold("Image 1/8: Dirt");      break;
		case 2:  annot.hold("Image 2/8: Grass");     break;
		case 3:  annot.hold("Image 3/8: Floor");     break;
		case 4:  annot.hold("Image 4/8: Marble");    break;
		case 5:  annot.hold("Image 5/8: Rock");      break;
		case 6:  annot.hold("Image 6/8: MetalPlate");      break;
		case 7:  annot.hold("Image 7/8: Tarnished Brass"); break;
		case 8:  annot.hold("Image 8/8: Checkerboard");    break;
			}
	return;
}

//------------------------------------------------------

void doc_option(void) {
	switch(pref.option) {
		case 0:
			if (maxoption==0) annot.hold("No Options"); 
			else annot.hold("Option 0/10: Sine Bumps"); 
			break;
		case 1:  annot.hold("Option 1/10: Atlantis");   break;
		case 2:  annot.hold("Option 2/10: Rocky");        break;
		case 3:  annot.hold("Option 3/10: Pyramids");     break;
		case 4:  annot.hold("Option 4/10: Playground");   break;
		case 5:  annot.hold("Option 5/10: Variable Crater");   break;
		case 6:  annot.hold("Option 6/10: cloud.bmp");   break;
		case 7:  annot.hold("Option 7/10: dirt.bmp");   break;
		case 8:  annot.hold("Option 8/10: metalplate.bmp");   break;
		case 9:  annot.hold("Option 9/10: brasstarnish.bmp");   break;
		case 10: annot.hold("Option 10/10: floor.bmp");   break;
	}
	return;
}

//-------------------------------------------------------

void doc_lightmap(void) {
	char s[80];
	int lightmap=abs(pref.lightmap);
	sprintf(s,"%d/64: ",lightmap);
	if (lightmap&1) strcat(s,"X ");
	if (lightmap&2) strcat(s,"Y ");
	if (lightmap&4) strcat(s,"Z ");
	if (lightmap&8) strcat(s," Vignette ");
	if (lightmap&16) strcat(s," X-grating");
	if (lightmap&32) strcat(s," Y-grating");
	annot.hold(s);
}

//-------------------------------------------------------

void dialog_init(int ID, int mode) {
	int x,y, yinc, width, height;
	static int logged=0;

	if (!logged) { logged=1; log_message("Dialog Init..."); }
	dialogText.bend=0;
	dialogText.alphatest=0;

	x=30; y=.7*screen.height;
	if (mode==1) { // init 1 vs. update 0?
		dialogCurrent->clear();
		dialogCurrent->ID=ID;
	}
	//-------------------------------------------------------------------------------------------------------
	switch(ID) {
	case THIS:
	Widget::textSize=10; yinc=-24; width=30; height=30;
	//                     name   label                  ID   x     y  width height    currentValue  minValue  maxValue  truncate?  log?
	dialogCurrent->control(TITLE, titlestr,               0,  x,    y, width,height);
	dialogCurrent->control(BACK,  "",                    93,  x+200,y, width,height);
	y-=50; width=140; height=24;

	dialogCurrent->control(SLIDER,"Orneriness",           200,x,y,  width,height, game.difficulty, .1, 5,0, 0); y+=yinc;
	dialogCurrent->control(SLIDER,"Fire Interval Seconds",201,x,y,width,height, mybullet.interval,.04,   1);y+=yinc;	
	dialogCurrent->control(SLIDER,"# Lives",              202,x,y,width,height, game.maxlives,     1,  20.9,1,0);y+=yinc;
	dialogCurrent->control(SLIDER,"Gun X Position",203,x,y,width,height, game.hand,      -1, 1,0,1);y+=yinc;	
	dialogCurrent->control(SLIDER,"Gun Y Position",204,x,y,width,height, game.handHeight,-1, 1,0,1);y+=yinc;	
	y+=-15;
	dialogCurrent->control(SLIDER,"Images: Ground Textures",   40,x,y,width,height,  pref.image,      0,  maximage+.9,1,1);y+=yinc;
	dialogCurrent->control(SLIDER,"Options: Height Maps",       41,x,y,width,height,  pref.option,      0,  maxoption+.9,1,1);y+=yinc;
	dialogCurrent->control(SLIDER,"[] Size: World",           53,x,y,width,height,  pref.bracket,   .1,   6);y+=yinc;
	dialogCurrent->control(SLIDER,"PageUp/Dn: Height",54,x,y,  width,height,    pref.page,      .1,  4); y+=yinc;
	dialogCurrent->control(SLIDER,"<> Tesselation: Ground",    50,x,y,width,height,  pref.tess,  mintess,    maxtess+.9,1,1);y+=yinc;
	y+=-15;
	dialogCurrent->control(SLIDER,"Mouse Sensitivity",                 71,x,y,width,height,pref.mouseSens, 0,2,0,1);y+=yinc;
	dialogCurrent->control(SLIDER,"ArrowMove/Click Thrust Sensitivity",72,x,y,width,height,pref.moveSens,  0,2,0,1);y+=yinc;
	y+=-15;
	dialogCurrent->control(SLIDER,"Universe: None/Stars/FixedSkyBox/MovableSkyBox",42,x,y,width,height,  pref.universe,    0,  3.9,1,1);y+=yinc;
	y+=-15;
	//dialogCurrent->control(SLIDER,"Arrowx", 75,x,y,width,height,pref.arrownx,        -40,             40,0,1);y+=yinc;
	//dialogCurrent->control(SLIDER,"Arrowy", 76,x,y,width,height,pref.arrowny,         10,             90,0,1);y+=yinc;
	//dialogCurrent->control(SLIDER,"-= Universe Rotate Speed",     56,  x,   y, width,height,    pref.spd1,       0, 20); y+=yinc;
	//dialogCurrent->control(SLIDER,"_+ Universe Move Speed",       57,  x,   y, width,height,    pref.spd2,       0, 20); y+=yinc;
	//y+=-15;

	dialogCurrent->control(SLIDER,"F1/F2 Texture X Magnification",26,x,y, width,height,  pref.a1,      0,    20,0,0);y+=yinc;
	dialogCurrent->control(SLIDER,"F3/F4 Texture Y Magnification",27,x,y, width,height,  pref.a2,      0,    20,0,0);y+=yinc;

	//dialogCurrent->control(SLIDER,"Persistence",100,x,   y, width,height, persistence,  0,    1,0,1);y+=yinc;
	//dialogCurrent->control(SLIDER,"# Octaves",  101,x,   y, width,height, noctaves,     0,    8,1,1);y+=yinc;
	break;
	//-----------------------------------------------------------------------------------------------------
	case GENERAL:
	Widget::textSize=10; yinc=-24; width=30; height=30;
	dialogCurrent->control(TITLE, "General",          0,   x,    y, width,height);
	dialogCurrent->control(BACK,"",                   93,  x+200,y, width,height);
	y-=50; width=140; height=24;
	//dialogCurrent->control(SLIDER,"-= RotateSpeed", 56,  x,   y, width,height,    pref.spd1,       0, 20); y+=yinc;
	//dialogCurrent->control(SLIDER,"_+ Move Speed",  57,  x,   y, width,height,    pref.spd2,       0, 20); y+=yinc;
	dialogCurrent->control(SLIDER,",. Magnification/FOV",51,x,y, width,height,    pref.fov,        5,  178,1,1);y+=yinc;
	dialogCurrent->control(SLIDER,"{} CameraZ",     52,  x,   y, width,height,    pref.brace,     -.4*worldSize,    .4*worldSize,0,1);y+=yinc;
	dialogCurrent->control(SLIDER,"   CameraY",    521,  x,   y, width,height,    camera.centery,-.05*worldSize,.05*worldSize,0,1);y+=yinc;
	y+=-15;
	dialogCurrent->control(SLIDER,"F5/F6 Fog Start Distance",   28,x,y, width,height,  pref.b1,      0,   5*worldSize,0,0);y+=yinc;
	dialogCurrent->control(SLIDER,"F7/F8 Fog End Distance",     29,x,y, width,height,  pref.b2,      0,   5*worldSize,0,0);y+=yinc;
	y+=-15;
	dialogCurrent->control(SLIDER,"F11/F12 Fade Effect", 55,  x,   y, width,height,    pref.c2,       .01,    1);y+=yinc;
	dialogCurrent->control(SLIDER,"Letterbox",           59,  x,   y, width,height,    pref.letterbox,  0,  6.8, 1,1);y+=yinc;
	dialogCurrent->control(SLIDER,"W PolygonMode",       58,  x,   y, width,height,    pref.wireframe,  0,  2.9, 1,1);y+=yinc;
	y+=-15;
	yinc=-35; x+=40; width=30; height=30;
	dialogCurrent->control(TOGGLE, "Screen Clear",96 ,x,   y, width,height,    pref.blank);y+=yinc;
	dialogCurrent->control(TOGGLE, "Pause",       97, x   ,y, width,height,    pref.pause);y+=yinc;
	dialogCurrent->control(BUTTON, "Reset/Randomize",98, x ,y, width,height);y+=yinc-5;
	break;
	//-----------------------------------------------------------------------------------------------------
	case SOUND:
	Widget::textSize=10; yinc=-30;  width=30; height=30;
	dialogCurrent->control(TITLE, "Sounds",             0,  x,    y, width,height);
	dialogCurrent->control(BACK,"",                    93,  x+200,y, width,height);
	y-=50;
	dialogCurrent->control(TOGGLE, "Sounds",       70, x   ,y, width,height,    sound.on);
	y-=40;
	width=140; height=24; yinc=-24; 
	dialogCurrent->control(SLIDER,"SoundVolume",   71, x,   y, width,height,  pref.volume,      0,   1,0,1);y+=yinc;
	if (!sound.hasReverb) {
		dialogCurrent->control(SLIDER,"Balance",  72, x,   y, width,height,  pref.pan,      0,   1,0,1);y+=yinc;
		y-=15;
	}
	//dialogCurrent->control(SLIDER,"WorldScale",   711, x,   y, width,height,  sound.worldScale,      0,   100,0,1);y+=yinc;
	//if (sound.hasReverb) 
	//	dialogCurrent->control(SLIDER,"ReverbMode",73, x,   y, width,height,  pref.reverbMode,  0,  26.8, 1,1);y+=yinc;
	break;
	//-----------------------------------------------------------------------------------------------------
	case LIGHTMAP:
	Widget::textSize=10; yinc=-24;  width=30; height=30;
	dialogText.bend=0; dialogText.alphatest=0;
	dialogCurrent->control(TITLE, "Lightmap",           0,  x,    y, width,height);
	dialogCurrent->control(BACK,"",                    93,  x+200,y, width,height);
	y-=50; width=140; height=24; yinc=-24; 
	dialogCurrent->control(SLIDER,"Lightmaps: None/Vignette/Noise/Grating Combinations",61,x,y,width,height,  abs(pref.lightmap),    0,  63.9,1,1);y+=yinc;
	dialogCurrent->control(SLIDER,"Home/End:Lightmap Noise Level",60,  x,   y, width,height,    pref.home,   0,  50,1,0); y+=yinc;
	dialogCurrent->control(SLIDER,"F9/F10 Vignette Size", 73,x,y,width,height,  pref.c1,          0,             3,0,1);y+=yinc;
	dialogCurrent->control(SLIDER,"Lightmap Refresh Interval",   74, x,   y, width,height,  pref.lmInterval,      0,   1,0,0);y+=yinc;
	y-=15;
	dialogCurrent->control(SLIDER,"Texture X Magnification",30,x,y, width,height,  pref.lmtextureMagx,      0,    20,0,0);y+=yinc;
	dialogCurrent->control(SLIDER,"Texture Y Magnification",31,x,y, width,height,  pref.lmtextureMagy,      0,    20,0,0);y+=yinc;

	Widget::textSize=10; yinc=-35;  width=30; height=30;
	y-=20;
	dialogCurrent->control(TOGGLE, "Film Noise",  21, x,y, width,height, pref.lightmap&1);y+=yinc;
	dialogCurrent->control(TOGGLE, "TV Line Noise",  22, x,y, width,height, pref.lightmap&2);y+=yinc;
	dialogCurrent->control(TOGGLE, "Z Pixel Noise", 23, x,y, width,height, pref.lightmap&4);y+=yinc;
	dialogCurrent->control(TOGGLE, "X Grating",     25, x,y, width,height, pref.lightmap&16);y+=yinc;
	dialogCurrent->control(TOGGLE, "Y Grating",     26, x,y, width,height, pref.lightmap&32);y+=yinc;
	dialogCurrent->control(TOGGLE, "Vignette",      24, x,y, width,height, pref.lightmap&8);y+=yinc;
	break;
	//-----------------------------------------------------------------------------------------------------
	case MAIN:
	Widget::textSize=20; yinc=-40; width=30; height=30;
	dialogText.bend=1; dialogText.alphatest=1;
	dialogCurrent->control(TITLE, "Main Menu",    0,  x,   y, width,height);
	y-=60;
	if (game.on) {
		dialogCurrent->control(BUTTON,"Re-Start Game", 5, x  ,y, width,height);y+=yinc;
		dialogCurrent->control(BUTTON,"Stop Game",     51,x  ,y, width,height);y+=yinc;
	}
	else {
		dialogCurrent->control(BUTTON,"Start New Game",5,x   ,y, width,height);y+=yinc;
	}
	y-=15;
	dialogCurrent->control(BUTTON, "Options...",  3,  x   ,y, width,height);y+=yinc;
	dialogCurrent->control(BUTTON, "Save...",     1,  x   ,y, width,height);y+=yinc;
	dialogCurrent->control(BUTTON, "Load...",     2,  x   ,y, width,height);y+=yinc;
	dialogCurrent->control(TOGGLE, "Info",       80,  x,   y, width,height, infoOn);y+=yinc;
	y-=15;
	dialogCurrent->control(BUTTON, "Quit",       99,  x   ,y, width,height);y+=yinc;
	break;
	//------------------------------------------------------------------------------------------------------
	case COLORLIGHTING:
	Widget::textSize=10;	yinc=-20;  width=30; height=30;
	dialogText.bend=0;
	dialogCurrent->control(TITLE, "Color/Lighting",    0,  x,   y, width,height);
	dialogCurrent->control(BACK,  "",                 93,  x+200,y, width,height);
	y-=50; width=140; height=24;
	dialogCurrent->control(SLIDER,"Ambient",     10,  x,   y, width,height,  pref.ambient,        0,    1,0,1);y+=yinc;
	dialogCurrent->control(SLIDER,"Diffuse",     11,  x,   y, width,height,  pref.diffuse,        0,    1,0,1);y+=yinc;
	dialogCurrent->control(SLIDER,"Specular",    12,  x,   y, width,height,  pref.specular,       0,    1,0,1);y+=yinc;
	dialogCurrent->control(SLIDER,"Reflective",  13,  x,   y, width,height,  pref.reflective,     0,    1,0,1);y+=yinc;
	y+=-15;
	dialogCurrent->control(SLIDER,"Shininess",   14,  x,   y, width,height,  pref.shininess,0,  128,    1,1);  y+=yinc;
	dialogCurrent->control(SLIDER,"Cutoff",      15,  x,   y, width,height,  pref.cutoff   ,0,  180,    1,1);  y+=yinc;
	dialogCurrent->control(SLIDER,"Exponent",    16,  x,   y, width,height,  pref.exponent, 0,   10,    1);    y+=yinc;
	y+=-15;
	dialogCurrent->control(SLIDER,"Red",       20,  x,  y, width,height, pref.r,        0, 1,0,1);y+=yinc;
	dialogCurrent->control(SLIDER,"Green",     21,  x,  y, width,height, pref.g,        0, 1,0,1);y+=yinc;
	dialogCurrent->control(SLIDER,"Blue",      22,  x,  y, width,height, pref.b,        0, 1,0,1);y+=yinc;
	y+=-15;
	dialogCurrent->control(SLIDER,"Color Scale", 24,x,  y, width,height, pref.value[0],.01,2,0,1);y+=yinc;
	dialogCurrent->control(SLIDER,"Color Offset",25,x,  y, width,height, pref.value[1],.01,2,0,1);y+=yinc;
	y+=-15;
	dialogCurrent->control(SLIDER,"Alpha",       23,x,  y, width,height,  pref.a,      0,    1,0,1);y+=yinc;
	y+=-15;
	dialogCurrent->control(SLIDER,"BlendFunc0",  31,x,  y, width,height,  pref.blendFunc0,  0,  8.5,1,1);y+=yinc;
	dialogCurrent->control(SLIDER,"BlendFunc1",  32,x,  y, width,height,  pref.blendFunc1,  0,  7.5,1,1);y+=yinc;
	break;
	//-----------------------------------------------------------------------------------------------------
	case OPTIONS:
	Widget::textSize=20; yinc=-50; width=30; height=30;
	dialogText.bend=1; dialogText.alphatest=1;
	dialogCurrent->control(TITLE, "Options",    0,  x,   y, width,height);
	dialogCurrent->control(BACK,"",                    93,  x+200,y, width,height);
	y-=60;
	dialogCurrent->control(BUTTON, "Game...",           1,  x   ,y, width,height);y+=yinc;
	dialogCurrent->control(BUTTON, "Lighting/Color...", 2,  x   ,y, width,height);y+=yinc;
	dialogCurrent->control(BUTTON, "Sound...",          3,  x   ,y, width,height);y+=yinc;
	dialogCurrent->control(BUTTON, "Lightmap...",       4,  x   ,y, width,height);y+=yinc;
	dialogCurrent->control(BUTTON, "General...",        5,  x   ,y, width,height);y+=yinc;
	break;
	//-----------------------------------------------------------------------------------------------------
	case SAVE:
	Widget::textSize=20; yinc=-60; width=30; height=30;
	dialogText.bend=1; dialogText.alphatest=1;
	dialogCurrent->control(TITLE,"Save Menu",    0,  x,    y, width,height);
	dialogCurrent->control(BACK,  "",            93, x+200,y,width,height);
	y-=60;
	dialogCurrent->control(BUTTON,"! Save Prefs", 1, x    ,y, width,height);y+=yinc;
	break;
	//-----------------------------------------------------------------------------------------------------
	case LOAD:
	Widget::textSize=20;	yinc=-60; width=30; height=30;
	dialogText.bend=1;	dialogText.alphatest=1;
	dialogCurrent->control(TITLE,"Load Menu",    0, x,    y, width,height);
	dialogCurrent->control(BACK,  "",           93, x+200,y, width,height);
	y-=60;
	dialogCurrent->control(BUTTON,"@ Load Prefs",1, x    ,y, width,height);y+=yinc;
	break;
	}
	return;
}

//-----------------------------------------------------------------------------

void dialog_do(void) {
	float value;
	static float last0=1,last1=1,last2=0,last3=0;
	int refresh=0;

	value=dialogCurrent->activeValue;
	if (dialogCurrent->activeID==92) { // alt
		if (dialogCurrent->ID==MAIN) dialogCurrent->init(THIS,0);
		else                         dialogCurrent->init(MAIN,0);
		return;
	}
	else if (dialogCurrent->activeID==93) { // back
		float ox=dialogCurrent->ox;
		float oy=dialogCurrent->oy;
		dialogCurrent->pop();
		dialogCurrent->ox=ox;
		dialogCurrent->oy=oy;
		return;
	}
	else if (dialogCurrent->activeID==99) { // exit app
		ex=time.fTime;
		game.end_taunt();
		dialogs=0;
		capture=3;
		return;
	}
	//--------------------------------------------------------
	switch(dialogCurrent->ID) {
	case THIS:
	switch(dialogCurrent->activeID) {

		case 26: pref.a1=value; annot.hold("Texture X Magnification: %4.2f",pref.a1);break;
		case 27: pref.a2=value; annot.hold("Texture Y Magnification: %4.2f",pref.a2);break;

		case 40:
			pref.image=value;
			mesh.tex=&images[pref.image];
			setc();
			doc_image();
			break;
		case 41: pref.option=value; 
			make_bump();
			update_mesh();
			doc_option();
			break;
		case 42: pref.universe=value; 
			switch(pref.universe) {
				case 0: annot.hold("Blank Universe"); break;
				case 1: annot.hold("Star Universe"); break;
				case 2: annot.hold("Fixed SkyBox"); break;
				case 3: annot.hold("Movable SkyBox"); break;
			}
			break;
		case 50:
			pref.tess=value;
			annot.hold("Tesselation:: %d",(int)value); 
			mesh.resx=pref.tess;
			mesh.resy=pref.tess;
			update_mesh();
			break;

		case 53: pref.bracket=value; annot.hold("Size: %4.2f",pref.bracket);
			change_world();
			break;

		case 54: pref.page=value; annot.hold("<PageUp/Down> Main Height: %4.2f",pref.page);
			update_mesh();
			break;
		case 56: pref.spd1=value;   annot.hold("Move Speed: %4.2f",pref.spd1);  break;
		case 57: pref.spd2=value;   annot.hold("Rotate Speed: %4.2f",pref.spd2);break;

		case 71: pref.mouseSens=value; annot.hold("Mouse Sensitivity: %4.2f",value); break;
		case 72: pref.moveSens =value; annot.hold("Move Sensitivity: %4.2f",value);  break;


		//case 75: pref.arrownx  =value; annot.hold("Angle: %5.1f Degrees",value); break;
		//case 76: pref.arrowny  =value; annot.hold("Tilt: %5.1f Degrees",    value); break;

		case 80: pref.n  =value; annot.hold("#: %d", (int)value); break;

		//case 100: persistence=value; perlin.bind(); perlin_do(persistence, noctaves); break;
		//case 101: noctaves   =value; perlin.bind(); perlin_do(persistence, noctaves); break;

		case 200: game.difficulty=value; annot.hold("Orneriness: %f X.", value);break;
		case 201: {
				static float lastTime=-1, last=-1;

				if (value!=last) {
					last=value;
					mybullet.interval=value;
					hisbullet.interval=value;
				}
				if ((time.fTime-lastTime)>value) {
					game.fire_bullet();
					lastTime=time.fTime;
				}
				annot.hold("Fire Interval: %f Secs.", value);
			}
			break;
		case 202: game.maxlives=value;
			game.him.nlives=value;
			game.me.nlives =value; annot.hold("# Lives: %d", (int)value);
			break;
		case 203: game.hand=value;
			annot.hold("Hand X: %3.1f", value);
			if ((time.fTime-last3)>.25) {
				game.fire_bullet();
				last3=time.fTime;
			}
			break;
		case 204: game.handHeight=value;
			annot.hold("Hand Y: %3.1f", value);
			if ((time.fTime-last3)>.25) {
				game.fire_bullet();
				last3=time.fTime;
			}
			break;
	}
	break;
	//--------------------------------------------------------
	case GENERAL:
	switch(dialogCurrent->activeID) {
		case 28: 
			pref.b1=value;
			annot.hold("Fog Start Distance: %4.2f", value);
			if (pref.b2<pref.b1+.01) {pref.b2=pref.b1+.01;refresh=1;}
			pref.fog=1;
			break;
		case 29:
			pref.b2=value;
			annot.hold("Fog End Distance: %4.2f",   pref.b2);
			if (pref.b2<pref.b1+.01) {pref.b1=pref.b2-.01;refresh=1;}
			pref.fog=1;
			break;
		case 51: pref.fov=value;    annot.hold("Fov: %4.2f",pref.fov); break;
		case 52: pref.brace=value;  annot.hold("CameraZ: %4.2f",value); break;
		case 521: camera.centery=value;  annot.hold("CameraY: %4.2f",value); break;
		case 55:
			pref.c2=value; pref.blank=0;
			if (pref.c2>=1) {
				pref.c2=1;
				pref.blank=1;
				screen.needblank=pref.blank;
			} else if (pref.c2<.02) pref.c2=0;
			annot.hold("Fade: %4.2f",pref.c2);
			break;
		case 56: pref.spd1=value;   annot.hold("Move Speed: %4.2f",value);  break;
		case 57: pref.spd2=value;   annot.hold("Rotate Speed: %4.2f",value);break;
		case 58: pref.wireframe=value;  break;
		case 59: pref.letterbox=value; screen.needblank=1;
			annot.hold("Letterbox: %1d",pref.letterbox); break;

		case 60: pref.home=value;   annot.hold("Home: %4.2f",pref.home);
			break;

		case 96: pref.blank=value; screen.needblank=pref.blank; break;
		case 97: pref.pause=value; break;
		case 98: init_pref(); init_vars(); break;
	}
	break;
	//--------------------------------------------------------
	case SOUND:
	switch(dialogCurrent->activeID) {
		case 70: sound.on=value;
			if (sound.on) sound.play(game.myfire);
			break;
		case 71:
			pref.volume=value;
			FSOUND_SetSFXMasterVolume(value*255);
			if (time.fTime-last2>.5) {
				sound.play(game.hisfire);
				last2=time.fTime;
			}
			break;
		case 711:
			sound.worldScale=value;
			FSOUND_3D_Listener_SetDistanceFactor(value);
			if (time.fTime-last2>.5) {
				sound.play(game.hisfire);
				last2=time.fTime;
			}
			break;
		case 72:
			pref.pan=value;
			sound.set_pan(value);
			if (time.fTime-last2>.5) {
				sound.play(game.myfire);
				last2=time.fTime;
			}
			break;
		case 73:
			pref.reverbMode=value;
			//sound.set_reverb(pref.reverbMode);
			if (time.fTime-last2>.5) {
				sound.play(game.myfire);
				last2=time.fTime;
			}
			break;
	}
	break;
	//--------------------------------------------------------
	case LIGHTMAP:
	switch(dialogCurrent->activeID) {
		case 60: pref.home=value;    annot.hold("Home: %4.2f",pref.home);    break;
		case 61: pref.lightmap=value; doc_lightmap(); refresh=1; break;
		case 73: pref.c1   =value; annot.hold("F9/F10: Vignette Size %5.1f",value); break;
		case 74: pref.lmInterval=value; annot.hold("Lightmap Refresh Interval:%5.1f secs",value); break;

		case 30: pref.lmtextureMagx=value; annot.hold("Texture X Magnification: %4.2f",value);break;
		case 31: pref.lmtextureMagy=value; annot.hold("Texture Y Magnification: %4.2f",value);break;

		case 21: if (value) pref.lightmap|=1;
			else pref.lightmap&=(63-1);
			refresh=1; break;
		case 22: if (value) pref.lightmap|=2;
			else pref.lightmap&=(63-2);
			refresh=1; break;
		case 23: if (value) pref.lightmap|=4;
			else pref.lightmap&=(63-4);
			refresh=1; break;
		case 24: if (value) pref.lightmap|=8;
			else pref.lightmap&=(63-8);
			refresh=1; break;
		case 25: if (value) pref.lightmap|=16;
			else pref.lightmap&=(63-16);
			refresh=1; break;
		case 26: if (value) pref.lightmap|=32;
			else pref.lightmap&=(63-32);
			refresh=1; break;

	}
	break;
	//--------------------------------------------------------
	case MAIN:
	switch(dialogCurrent->activeID) {
		case 3: dialogCurrent->init(OPTIONS,1); break;
		case 1: dialogCurrent->init(SAVE,1); break;
		case 2: dialogCurrent->init(LOAD,1); break;
		case 5:
			game.init(); game.start();
			pref.view=prefview;
			dialogs=0; pref.pause=0;
			dialogCurrent->fadedir=0;
			mouse.set_cursor(&pref, dialogs);
			refresh=2;
			break;
		case 51:
			game.on=0;
			refresh=2;
			break;

		case 80: infoOn=!infoOn;
			if (infoOn) {help=0;}
			break;
	}
	break;
	//--------------------------------------------------------
	case OPTIONS:
	switch(dialogCurrent->activeID) {
		case 1: dialogCurrent->init(THIS,1); break;
		case 2: dialogCurrent->init(COLORLIGHTING,1); break;
		case 3: dialogCurrent->init(SOUND,1); break;
		case 4: dialogCurrent->init(LIGHTMAP,1); break;
		case 5: dialogCurrent->init(GENERAL,1); break;
	}
	break;
	//--------------------------------------------------------
	case COLORLIGHTING:
	switch(dialogCurrent->activeID) {
		case 10: pref.ambient=value;    break;
		case 11: pref.diffuse=value;    break;
		case 12: pref.specular=value;   break;
		case 13: pref.reflective=value; break;
		case 14: pref.shininess=value;  break;
		case 15: pref.cutoff=value;     break;
		case 16: pref.exponent=value;   break;

		case 20: pref.r=value;  break;
		case 21: pref.g=value;  break;
		case 22: pref.b=value;  break;

		case 23: pref.a=value;  break;
		case 31: pref.blendFunc0=value; doc_blend(0,pref.blendFunc0,&annot);	break;
		case 32: pref.blendFunc1=value; doc_blend(1,pref.blendFunc1,&annot);	break;

		case 24: pref.value[0]=value;
			pref.r*=(value/last0);
			pref.g*=(value/last0);
			pref.b*=(value/last0);
			if (pref.r>1) pref.r=1;
			if (pref.g>1) pref.g=1;
			if (pref.b>1) pref.b=1;
			last0=value;
			refresh=1;
			break;
		case 25: pref.value[1]=value;
			pref.r+=(value-last1);
			pref.g+=(value-last1);
			pref.b+=(value-last1);
			pref.r=(pref.r>1)?1:(pref.r<0)?0:pref.r;
			pref.g=(pref.g>1)?1:(pref.g<0)?0:pref.g;
			pref.b=(pref.b>1)?1:(pref.b<0)?0:pref.b;
			last1=value;
			refresh=1;
			break;
	}
	break;
	//--------------------------------------------------------
	case SAVE:
	switch(dialogCurrent->activeID) {
		case 1:
		  if (pref.save(titlestr)) annot.hold("Could NOT Save Prefs");
		  else                     annot.hold("Saved Prefs");
			break;
	}
	break;
	//--------------------------------------------------------
	case LOAD:
	switch(dialogCurrent->activeID) {
		case 1:
			if (pref.load(titlestr)) annot.hold("Could NOT Load Prefs");
			else {
				annot.hold("Loaded Prefs");
				pref.view=prefview;
				switch(pref.option) {
					case 0:	break;
				}
				mesh.tex=&images[pref.image];
				make_bump();
				update_mesh();
				switch(pref.universe) {
					case 0: set_fogcolor(0,0,0,1); break;
					case 1: set_fogcolor(0,0,0,1); break;
					case 2: set_fogcolor(1,1,1,1); break;
					case 3: set_fogcolor(1,1,1,1); break;
				}
				screen.needblank=1;
			}
			pref.pause=0;
			break;
	}
	break;
	}
	if      (refresh==1) dialog_init(dialogCurrent->ID,0); // update dialog
	else if (refresh==2) dialog_init(dialogCurrent->ID,1); // replace dialog
	return;
}

//--------------------------------------------------------------------

void view_this(void) {
	camera.orient();

	if (time.realTime<beginTime) {
	camera.x=0;     camera.y=0;       camera.z=sin(time.realTime*4);
	camera.pitch=0; camera.heading=0;  camera.roll=0;
	}
	if (pref.view==0) {
		camera.heading+=pref.arrownx;
		camera.pitch  +=pref.arrowny;
	}
/*
	if (lastmatrix) {
		float h=time.fTime-lastmatrix;
		if (h>=1) {lastmatrix=0; pref.pause=0;}// pref.view=0;}
		else camera.heading+=sin(h*pi2);
	}
	if (lastmatrix) glTranslatef(0,0,-1);
*/
	if ((pref.view!=3)||(!introDone)) {
		glRotatef(camera.roll ,   0,0,1); // rotate world
		glRotatef(camera.pitch,   1,0,0);
		glRotatef(camera.heading, 0,1,0);
	} else {
		//glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
		glTranslatef(0,0,-pref.brace);
		gluLookAt(app.get(0,10,1000*pref.bracket,-500*pref.bracket),
				  app.get(1,10,300,  50),
				  app.get(2,10,1000*pref.bracket,-500*pref.bracket),
				  0,10,0,
				  0,1,0);
	}
	camera.get_matrix();

	camera.maxthrust=FARZ*.3*(game.me.health*.0066+.33);
	if ((mouse.lbutton)&&(pref.view==1)&&(!game.on)) {
		camera.kick();
		annot.hold("Thrust");
	}
	if ((pref.view==1)&&(introDone)) {
		static float trav=0, lastTime=0;
		Object o;
		
		if ((key.move)&&(!dialogs)) camera.move_keys(&key);
		if (!pref.pause) camera.move(time.fDeltaTime, &pref);

//---------------

		clearance=mesh_grav(&camera,1);
		
//---------------

		//if ((!key.move)&&(mouse.mbutton)) {
		//	camera.drag(10);
	//		annot.hold("Brake");
		//}
		//else
			camera.drag(1.5);

		//trav+=fabs(camera.zi);
		//if ((trav>100)&&((time.fTime-lastTime)>.3)) {
		//	trav=0;
		//	sound.play("pl_snow1.wav");
		//	lastTime=time.fTime;
		//}

		o=camera;
		o.size=camera.size*-.2;
		if (o.collision(&game.him, &pref)&&(game.on)) {
			static float lastTime=0;

			camera.bounce(time.fDeltaTime, &pref);
			camera.shakeampl=1;
			
			if ((time.fTime-lastTime)>1) {
				sound.play("bonk.wav");
				lastTime=time.fTime;
			}
		}
	}
	camera.shake();


/*	if (pref.universe>=2) {
		skybox.r=pref.r;
		skybox.g=pref.g;
		skybox.b=pref.b;
	}
*/
	if ((pref.universe==2)&&(capture<2)&&(time.realTime>beginTime)) {// fixed skybox
		glColor3f(1,1,1);
		//glPushMatrix();			//glTranslatef(0,0,0);
			skybox.draw(550*pref.bracket);		//glPopMatrix();
	}

	if ((pref.view!=3)||(!introDone)) {
		//glTranslatef(-camera.x,-(camera.y+camera.centery),-camera.z);
		glTranslatef(-camera.x,-(camera.y+camera.centery),-camera.z);
		camera.get_matrix();
	}

	if ((pref.universe==3)&&(capture<2)&&(time.realTime>beginTime)) {// movable skybox
		glColor3f(1,1,1);
		//glPushMatrix();			//glTranslatef(0,0,0);
			skybox.draw(550*pref.bracket);		//glPopMatrix();
	}
}

//---------------------------------------------------------------------------

void draw_this(void) {
	static float flare=0;

	if (!ex) screen.maskr=1-game.me.health*.01;
	else     screen.maskr=0;

	if (!pref.pause) app.update(time.fDeltaTime,1);

	if (capture==2) {
		glViewport(0,0,screen.width,screen.height);
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		gluPerspective(90,(GLfloat)screen.width/screen.height,.1,100);
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
		gluLookAt(0,0,.5, 0,0,0,  0,1,0);
		end_shatter(1,&shatter, &screen, &time, &pref, &capture, &skip);
		return;
	}

#if defined (hasstars) // draw starfield
	if (pref.universe==1) {
		glPushMatrix();
			glRotatef(cameraRotate.get(3,.5,720,-360),0,0,1);
			glRotatef(cameraRotate.get(2,.5,720,-360),1,0,0);
			glRotatef(cameraRotate.get(1,.5,720,-360),0,1,0);
			starfield.draw();
			glPopMatrix();
	}
#endif

	lightRotate.update(time.fDeltaTime,pref.paren); // lighting
	if (pref.universe<2) light.position(
		                 lightRotate.get( 1,light.rotateSpeed,1000*pref.bracket,-500*pref.bracket),
		                 lightRotate.get2(1,light.rotateSpeed,pih,1000*pref.bracket,-500*pref.bracket),0);
	else light.position( lightRotate.get( 1,light.rotateSpeed,1000*pref.bracket,-500*pref.bracket),400,
		                 lightRotate.get2(1,light.rotateSpeed,pih,1000*pref.bracket,-500*pref.bracket));
	if (pref.lights) {
		light.ambient   =pref.ambient;//.15;
		light.diffuse   =pref.diffuse;//.4;
		light.specular  =pref.specular;//.7;
		light.reflective=pref.reflective;//.99;
		light.shininess =pref.shininess;
		light.exponent  =pref.exponent;
		light.cutoff    =pref.cutoff;
		//light.direction=1;
		light.set();
	}
	if ((pref.universe<2)&&(!ex)&&(introDone)) {
		sun.x=light.x;
		sun.y=light.y;
		sun.z=light.z;
		//set_suncolor();
	}

/*
	flare+=time.fDeltaTime;
	if (flare>.07) {
		flare=0;
		sunflare(sunMap.texID);
	}
*/
	// -----------------------------------------------

	if ((time.realTime>beginTime)&&(introDone==0)) { // begin
		time.baseTime-=beginTime;
		init_pref();
		init_vars();
		introDone=1;
		game.init();
		dialogCurrent->saveView=pref.view;
		pref.view=3;
		dialogs=1; //pref.pause=1;
		dialogCurrent->fadedir=1;
		mouse.set_cursor(&pref, dialogs);

		sound.play(game.wind,0,0,0);
		game.teleportChannel=sound.lastChannel;
		FSOUND_SetReserved(game.teleportChannel,TRUE);
	}
	if (time.realTime<beginTime) intro();
	else                         draw_thing();// do most of this rendering

	return;
}

//------------------------------------------------------------------------------

void draw_thing(void) {
	static float nextTaunt=rndn(20)+20;

	if (pref.universe==1) sun.draw(&camera, &pref);

	glEnable(GL_DEPTH_TEST);
	if (pref.fog) {
		glFogf(GL_FOG_START,pref.b1);
		glFogf(GL_FOG_END,  pref.b2);
		glEnable(GL_FOG);
	}
	if (pref.lights) glEnable(GL_LIGHTING);

	if ((camera.y+camera.centery)>game.waterHeight) game.draw_water();

	mesh.texmagx=image_details[pref.image].texmagx*pref.a1;
	mesh.texmagy=image_details[pref.image].texmagy*pref.a2;
	mesh.r=image_details[pref.image].r;
	mesh.g=image_details[pref.image].g;
	mesh.b=image_details[pref.image].b;
/*
	mesh.texmagx=pref.a1;
	mesh.texmagy=pref.a2;
	mesh.r=pref.r;
	mesh.g=pref.g;
	mesh.b=pref.b;
*/
	mesh.blend=0;
	mesh.texOffsetx=0;
	mesh.texOffsety=0;
	mesh.tex=&images[pref.image];
	mesh.draw(&pref); // ---------------------------------------------------
	if (pref.universe>1) {
		glEnable(GL_POLYGON_OFFSET_FILL);
		glPolygonOffset(-1,-1);
		mesh.blend=1;
		mesh.blendFunc0=pref.blendFunc0;
		mesh.blendFunc1=pref.blendFunc1;
		mesh.setcolor(pref.r,pref.g,pref.b);
		mesh.a=.5;//pref.a;
		mesh.texmagx=3*pref.a1;
		mesh.texmagy=3*pref.a2;
		mesh.texOffsetx=time.fTime*.075;
		mesh.texOffsety=time.fTime*.075;
		mesh.tex=&cloud;
		mesh.draw(&pref);
		glDisable(GL_POLYGON_OFFSET_FILL); // ------------------------------
	}
/*
	glEnable(GL_ALPHA_TEST);
	glEnable(GL_BLEND);
	glEnable(GL_TEXTURE_2D);
	grassTex.bind();
	glColor4f(0,0,0,pref.a);
	set_blend(pref.blendFunc0,pref.blendFunc1);
	mesh.draw_grass(&pref);
	glDisable(GL_ALPHA_TEST);
	glDisable(GL_BLEND);
	glDisable(GL_TEXTURE_2D);
*/

	grass.draw();

	glDisable(GL_DEPTH_TEST);
	if (pref.fog)    glDisable(GL_FOG);

	game.boost(); // health recovery
	if      (rndn(1)<.06*time.fDeltaTime)  { // occasional cricket
		sound.play("cricket2.wav", mesh.rnd(1), 0, mesh.rnd(1));
		//sound.set_pan(rndn(1));
	}
	else if (rndn(1)<.004*time.fDeltaTime) { // more rare horse
		sound.play("horse1.wav",   mesh.rnd(1), 0, mesh.rnd(1));
		//sound.set_pan(rndn(1));
	}
	else if (rndn(1)<.004*time.fDeltaTime) { // wolf
		sound.play("wolf2.wav",    mesh.rnd(1), 0, mesh.rnd(1));
		//sound.set_pan(rndn(1));
	}
	if ((time.fTime-game.lastTauntTime)>(nextTaunt)) {
		nextTaunt=rndn(15)+15;
		if (dialogs||(!game.on)) game.idle_taunt(); // Sam taunts
		else                     game.taunt();
	}
	if (rndn(1)<.01*time.fDeltaTime) {
		if (rndn(1)<.1) skip=1.01;
		else            skip= .01;
		capture=3;
	}

	mybullet.update();
	hisbullet.update();
	if ((!dialogs)&&(!pref.pause)) { // game running?
		if (game.on) { // game running?
			game.update(dialogs);
			if (!game.on) dialog_init(dialogCurrent->ID,1);
			test_mesh(&mybullet);
			test_mesh(&hisbullet);
		}
	}

	if (rndn(1)<10*time.fDeltaTime) {
		if      (pref.option==1) dust.create(0,5*pref.page,0);
		else if (pref.option==3) dust.create(-135*pref.bracket,130*pref.page,-180*pref.bracket);
	}

	dust.update();   dust.draw(); // particles
	spark.update();	 spark.draw();
	splash.update(); splash.draw();
	
	blood.update(); test_mesh_stick(&blood); blood.draw();
	mybullet.draw();
	hisbullet.draw();

	if (game.proc_teleport()) {
		int r=rndn(10);
		switch(r) {
		case 0:
			pref.lightmap=8;
			pref.lmtextureMagx=1;
			pref.lmtextureMagy=1;
			pref.c1=.8;
			pref.letterbox=0;
			break;
		case 1:
			pref.lightmap=1;
			pref.lmtextureMagx=3;
			pref.lmtextureMagy=3;
			pref.letterbox=2;
			break;
		case 2:
			pref.lightmap=2;
			pref.lmtextureMagx=3;
			pref.lmtextureMagy=3;
			pref.letterbox=2;
			break;
		case 3:
			pref.lightmap=32;
			pref.lmtextureMagx=3;
			pref.lmtextureMagy=3;
			pref.letterbox=2;
			break;
		case 4: default:
			pref.lightmap=4;
			pref.letterbox=2;
			pref.lmtextureMagx=3;
			pref.lmtextureMagy=3;
			break;
		}
		make_bump();
		change_world();
		setc();
	}
	glEnable(GL_DEPTH_TEST);
	if ((game.me.nbullet)&&(game.on)) game.draw_gun();
	if (!((camera.y+camera.centery)>game.waterHeight)) game.draw_water();
	if (pref.lights) glDisable(GL_LIGHTING);
	return;
}

//--------------------------------------------------------------------------

void annot_this(void) { // all 2d rendering

	if (capture<2) update_lm(); //splinterable 2d

	if (capture==0) { // non-splinterable 2d
		char s[80];
		switch(pref.n) {
		case 0: break;
		case 1: sprintf(s,"forwardx %f", camera.forwardx);     annot.hold(s); break;
		case 2: sprintf(s,"forwardy %f", camera.forwardy);     annot.hold(s); break;
		case 3: sprintf(s,"forwardz %f", camera.forwardz);     annot.hold(s); break;
		case 4: sprintf(s,"upx %f", camera.upx);          annot.hold(s); break;
		case 5: sprintf(s,"upy %f", camera.upy);          annot.hold(s); break;
		case 6: sprintf(s,"upz %f", camera.upz);          annot.hold(s); break;
		case 7: sprintf(s,"rightx %f", camera.rightx);       annot.hold(s); break;
		case 8: sprintf(s,"righty %f", camera.righty);       annot.hold(s); break;
		case 9: sprintf(s,"rightz %f", camera.rightz);       annot.hold(s); break;
		case 10: sprintf(s,"heading %f", camera.heading);      annot.hold(s); break;
		case 11: sprintf(s,"pitch %f", camera.pitch);        annot.hold(s); break;
		case 12: sprintf(s,"# playing channels: %d", FSOUND_GetChannelsPlaying()); annot.hold(s); break;
		case 13: sprintf(s,"farz: %f", screen.farz); annot.hold(s); break;
		}

		annot.printHold();
		if (introDone) {
			if (dialogs) dialogCurrent->run();
			else {
				if (dialogCurrent->fade) dialogCurrent->show();
				game.annot();
			}
			if (infoOn) info(titlestr);
			if (time.fTime<10) title.print(titlestr,
				                           screen.width*(time.fTime*.4-1),
										   screen.height*.95,40);
			announce("");
		}
	}
	if (mouse.mbutton==1) zoomon=!zoomon;
	if (zoomon && !ex) zoom();
	game.draw_crosshair();

	sound.update_listener();
	//float pos[]={camera.x,camera.y,camera.z},vel[]={0,0,0};
	//FSOUND_3D_Listener_SetAttributes(pos, vel,
	//	-camera.forwardx,-camera.forwardy,-camera.forwardz,
	//	-camera.upx,     -camera.upy,     -camera.upz);
	//FSOUND_3D_Update();

	return;
}

//---------------------------------------------------------------------------

void set_suncolor(void) {
	float y;

	y=(light.y+300)/700;
	screen.b=sin(y*pih);
	screen.r=screen.b*.2;
	screen.g=screen.b*.2;
}

//--------------------------------------------------------------------------

void info(char *titlestr) {
	short x,y, yinc=-16, size=8;
   
	x=screen.width*.25;
	y=screen.height*.95;
	gen.reset();
	gen.color(1,1,0);
	gen.blend=0;
	gen.alphatest=0;

	gen.bend=0;
	gen.bloom=1;
	gen.print(titlestr, x,y, 30);
	gen.bend=0; gen.bloom=0;

	x+=10; y+=yinc*2;
	gen.color(0,1,0);
	gen.print("Basics:",x,y,size); y+=yinc; x+=20;
	gen.color(1,1,1);
	gen.print("OpenGL Challenge - Cartoon",x,y,size); y+=yinc;
	gen.print("First-Person 3d Shooter w/ Yosemite Sam",x,y,size); y+=yinc;
	gen.print("Gun w/ 25-round capacity, available 25-round packs at Green Orb",x,y,size); y+=yinc;
	gen.print("Teleport to random heightmap/texture/blendFuncs/3d soundEnvironment at Yellow Orb",x,y,size); y+=yinc;
	gen.print("Sam's A.I. includes (adjustable) aggressiveness keyed to his health and proximity",x,y,size); y+=yinc;
	gen.print("5 (adjustable) lives in each DeathMatch",x,y,size); y+=yinc;
	gen.print("Health slowly recharges, and affects mobility, Red Cross Orb tops off",x,y,size); y+=yinc;
	gen.print("~30 adjustable & savable preferences",x,y,size); y+=yinc;

	x-=20; y-=10;
	gen.color(0,1,0);
	gen.print("Controls:",x,y,size); y+=yinc; x+=20;
	gen.color(1,1,1);
	//gen.print("       1-9,0  - Adjust # ",            x,y,size); y+=yinc;
	gen.print("<PageUp/Down> - Adjust Ground Height",      x,y,size); y+=yinc;
	gen.print("           [] - Adjust Size: World",        x,y,size); y+=yinc;
	gen.print("     shift <> - Adjust Ground Tesselation",     x,y,size); y+=yinc;
	gen.print("            i - Image Options: Ground Textures",x,y,size); y+=yinc;
	gen.print("            o - Options: Ground Heightmaps",    x,y,size); y+=yinc;
	y+=yinc-15;
	gen.print("    shift L - Lightmaps: Toggle On/Off",x,y,size); y+=yinc;
	gen.print("shift-alt L - Lightmaps: Cycle None/Vignette/X-Y-Z Noise/X-Y Grating",x,y,size); y+=yinc;
	gen.print(" <Home/End> - Adjust Lightmap Noise Level",               x,y,size); y+=yinc;

	x-=20; y-=10;
	gen.color(0,1,0);
	gen.print("Motion Control:",x,y,size); y+=yinc; x+=20;
	gen.color(1,1,1);
	gen.print("v - ViewCam Cycle: Auto/Manual/Hold/Auto-centered",       x,y,size); y+=yinc;
	gen.print("Quake-type ArrowKey movement controls",        x,y,size); y+=yinc;
	gen.print("Quake-type MouseLook",                         x,y,size); y+=yinc;
	//gen.print("Click <LeftMouseButton> to Move Forward in Manual ViewCam",x,y,size); y+=yinc;
	gen.print("Click <MiddleMouseButton> to Toggle Sniper Zoom",          x,y,size); y+=yinc;
	gen.print("<Control> + ArrowKey converts ArrowKey X/Z to X/Y",        x,y,size); y+=yinc;
	gen.print("    <Alt> + ArrowKey increases thrust 10x",                x,y,size); y+=yinc;

	x-=20; y-=10;
	gen.color(0,1,0);
	gen.print("Misc:",x,y,size); y+=yinc; x+=20;
	gen.color(1,1,1);
	//gen.print("         m - Matrix:   Demo Effect                      ",  x,y,size); y+=yinc;
	gen.print("u - Universe: Cycle None/Stars/Fixed SkyBox/Movable SkyBox", x,y,size); y+=yinc;
	gen.print("x - eXplode:  Toggle Effect (try with Hyper<Space>!)",        x,y,size); y+=yinc;
	gen.print("Select Popups or 'P' to pause",                               x,y,size); y+=yinc;
	gen.print("Press '?' for more info on keyboard controls, and read info.txt",x,y,size); y+=yinc;
	return;
}

//-----------------------------------------------------------------------------

void lm(void) {
	//lightmap.roll=lightmap.roll+120+rndn(120);//faster noise by rotation vs. texgening
	//if (lightmap.roll>=360) lightmap.roll-=360;
	//if (pref.lightmap&8) {lightmap.texmagx=1;lightmap.texmagy=1;}
	//else                 {lightmap.texmagx=3;lightmap.texmagy=3;}
	lightmap.texmagx=pref.lmtextureMagx;
	lightmap.texmagy=pref.lmtextureMagy;
	lightmap.sizex=screen.width *.5;
	lightmap.sizey=screen.height*.5;
	lightmap.x=.5*screen.width;
	lightmap.y=.5*screen.height;
	lightmap.draw(&camera,&pref);
}

//-----------------------------------------------------------------------------

void intro(void) {
	float t=time.fTime*2,r,g,b, size=1, f2;
	int it=((int)t)%3,t2;

	t2=((int)t)%2;
	f2=t-(int)t;
	f2=1-fabs(f2*2-1);
	r=(it==0)*f2;
	g=(it==1)*f2;
	b=(it==2)*f2;

	glColor3f(r,g,b);
	glAlphaFunc(GL_GREATER,.1);
	glEnable(GL_ALPHA_TEST);
	if (pref.textures) {
		glEnable(GL_TEXTURE_2D);
		geom.tex=&logo;
		geom.tex->bind();
	}
	glPushMatrix();
		glTranslatef(0,0,-3);
		glRotatef(app.get(1,200,40,-20),1,0,0);
		glRotatef(app.get(2,200,40,-20),0,1,0);
		glRotatef(app.get(3,200,40,-20),0,0,1);
		geom.draw_quad(-size,-size,0,  size,-size,0,
		                size, size,0, -size, size,0);
		glPopMatrix();
	glDisable(GL_ALPHA_TEST);
	if (pref.textures) glDisable(GL_TEXTURE_2D);
	return;
}

//-----------------------------------------------------------------------------

void make_bump(void) {
	int n=0, i;

	switch(pref.option) {
		case 0: 
			bump.freq=rndn(1)+.5;// sine bumps
			bump.make(pref.option, 128, 15,-5); 
			break;
		case 1:
			bump.freq=rndn(.8)+.2;// rings
			bump.make(pref.option, 128,20,0);
			break;
		case 2: bump.load("bump_128.bmp",        30,5);  break;// rocky
		case 3: bump.load("bump3.bmp",          200,130);break;// pyramid
		case 4: bump.make(pref.option,          128,20,10); break;// playground
		case 5: bump.make(pref.option,          128,0,0); n=rndn(10)+10; break;// flat
		case 6: bump.load("cloudbw2128.bmp",     50,10); break;// cloud
		case 7: bump.load("dirt128.bmp",         60,25);break;// 
		case 8: bump.load("metalplate128.bmp",   25,0);break;// 
		case 9: bump.load("brasstarnish128.bmp", 20,-20);break;// 
		case 10: bump.load("floor.bmp",          60,-10);break;// 
	}
	for (i=0; i<n; i++) { // random cratering
		bump.depress(rndn(110)+10,
		             rndn(110)+10,
					 rndn(60)+30, rndn(10)+10);
	}
}

//------------------------------------------------------------------------------

void update_mesh(void) {
	int n=rndn(98)+1;

	grass.kill();
	mesh.init(&bump);
	grass.size0=rndn(5)+2;
	for (int i=0; i<n; i++) { // place grasses
		float x, y, z;
		x=rndn(1);
		z=rndn(1);
		y=mesh.get(x,z)+grass.size0;
		x=x*mesh.size-.5*mesh.size;
		z=z*mesh.size-.5*mesh.size;
		grass.create(x,y,z);
	}
}

//-----------------------------------------------------------------------------

float mesh_grav(Object *object,int fall) {
	float h,ix,iz,xi,zi,f,w, gravity=500;
	
	w=(.5*mesh.size);
	if (object->x<-w) {object->x=-w; object->xi=-object->xi*.2;}
	if (object->x> w) {object->x= w; object->xi=-object->xi*.2;}
	if (object->z<-w) {object->z=-w; object->zi=-object->zi*.2;}
	if (object->z> w) {object->z= w; object->zi=-object->zi*.2;}
		
	ix=(object->x+w)/mesh.sizex;
	iz=(object->z+w)/mesh.sizey;
	h=mesh.get(ix,iz);                // get area height
	if (fall) mesh.getv( ix,iz, &xi, &zi);         // get normal
	if (((object->y-.5*object->sizey)<h)&&(pref.view==1)) { // object under?
		f=((30./pref.page)/pref.bracket)*time.fDeltaTime*pref.bracket;     //50
		object->y=h+.5*object->sizey; // clamp object
		if (fall) {
			object->xi+=xi*f;
			object->zi+=zi*f;
		}
	}
	else if ((!pref.pause)&&((object->y-.5*object->sizey)>h)&&(pref.view==1))
	   object->yi-=gravity*time.fDeltaTime; // gravity
	return object->y-h;
}

//-----------------------------------------------------------------------------

void announce(char *ins) {
	static char s[80];
	static float t=0;
	static l;

	if (ins[0]==0) {
		if ((time.fTime-t)<3) {annot.reset();
			annot.bend=1;
			annot.blend=1; annot.alphatest=1; annot.alphafunc=.1;
			annot.r=1;
			annot.g=1;
			annot.b=1;
			annot.print(s,.5*screen.width-l*15,.95*screen.height,40);
			annot.reset();
		}
	} else {
		l=strlen(ins);
		t=time.fTime;
		strcpy(s,ins);
	}
}

//------------------------------------------------------------------------------

void update_lm(void) {
	static float lastmaketex=0;

	if ((time.realTime<beginTime)||(pref.lightmap>0)) {
		if ((time.realTime-lastmaketex)>pref.lmInterval) {
			lastmaketex=time.realTime;
			noisy1(lmsphere.texID,pref.home*50,100*pref.c1,pref.lightmap);
		}
		lm();
	}
}

//-------------------------------------------------------------------------------

void test_mesh(Particle *particle) {
	float w, h, ix, iz;
	int i, n=rndn(7)+3, ni;

	w=(.5*mesh.sizex);	
	for (i=0; i<particle->maxparticles; i++) {
		if (particle->p[i].on==0) continue; 
		ix=(particle->p[i].x+w)/mesh.sizex;
		iz=(particle->p[i].z+w)/mesh.sizey;
		h=mesh.get(ix,iz);
		if (particle->p[i].y<h) { // object under?
			particle->p[i].on=0;
			if (h>game.waterHeight) {				
				int r;

				r=rndn(3);
				switch(r) {
					case 0: sound.play(game.rico1,particle->p[i].x,h,particle->p[i].z); break;
					case 1: sound.play(game.rico2,particle->p[i].x,h,particle->p[i].z); break;
					case 2: sound.play(game.twing,particle->p[i].x,h,particle->p[i].z); break;
					case 3:  break;
				}
				sound.set_frequency(rndn(15000)+10000);
				if (!sound.hasReverb) {
					sound.set_pan(rndn(1));
					sound.set_volume(rndn(100)+50);
				}
				dust.create(     particle->p[i].x,h,particle->p[i].z);
				for (ni=0; ni<n; ni++) {
					spark.create(particle->p[i].x,h,particle->p[i].z);
				}
			} else {
				sound.play(game.splash,particle->p[i].x,h,particle->p[i].z);
				sound.set_frequency(rndn(20000)+5000);
				if (!sound.hasReverb) {
					sound.set_pan(rndn(1));
					sound.set_volume(rndn(100)+50);
				}
				for (ni=0; ni<n; ni++) {
					splash.create(particle->p[i].x,game.waterHeight,particle->p[i].z);
				}
			}
		}
	}
	return;
}

//-------------------------------------------------------------------------------

void test_mesh_stick(Particle *particle) {
	float w, h, ix, iz;
	int i;

	w=(.5*mesh.sizex);	
	for (i=0; i<particle->maxparticles; i++) {
		if (particle->p[i].on==0) continue; 
		ix=(particle->p[i].x+w)/mesh.sizex;
		iz=(particle->p[i].z+w)/mesh.sizey;
		h=mesh.get(ix,iz);
		if (particle->p[i].y<h) { // object under?
			particle->p[i].y=h;
			particle->p[i].yi=0;
			particle->p[i].xi*=.9;
			particle->p[i].zi*=.9;

		}
	}
	return;
}

//-------------------------------------------------------------------------------

void setc(void) {
	spark.r0=image_details[pref.image].rp;
	spark.g0=image_details[pref.image].gp;
	spark.b0=image_details[pref.image].bp;
	spark.r1=image_details[pref.image].rp;
	spark.g1=image_details[pref.image].gp;
	spark.b1=image_details[pref.image].bp;
	spark.blendFunc0=image_details[pref.image].b0;
	spark.blendFunc1=image_details[pref.image].b1;

	dust.r0=image_details[pref.image].rp;
	dust.g0=image_details[pref.image].gp;
	dust.b0=image_details[pref.image].bp;
	dust.r1=image_details[pref.image].rp;
	dust.g1=image_details[pref.image].gp;
	dust.b1=image_details[pref.image].bp;
	dust.blendFunc0=image_details[pref.image].b0;
	dust.blendFunc1=image_details[pref.image].b1;
	}

//--------------------------------------------------------------------------------

void zoom(void) {
	int size=36, sizeh;

	sizeh=size*.5;
	glPixelZoom(2,2);
	glRasterPos2f(screen.width*.5-size, screen.height*.5-size);
	glCopyPixels( screen.width*.5-sizeh,screen.height*.5- sizeh,size,size,GL_COLOR);
	glColor3f(1,0,0);
	frame(screen.width*.5-size-1,screen.height*.5-size-1,
		  screen.width*.5+size+1,screen.height*.5+size+1);
}

//--------------------------------------------------------------------------------

void frame(float x0, float y0, float x1, float y1) {
	glLineWidth(1);
	glBegin(GL_LINE_LOOP);
		glVertex2d(x0,y0);
		glVertex2d(x1,y0);
		glVertex2d(x1,y1);
		glVertex2d(x0,y1);
		glEnd();
}

//-------------------------------------------------------------------------------

void change_world(void) {
	starfield.size=600*pref.bracket;
	starfield.hollow=400*pref.bracket;
	starfield.init(2000/starfield.nlevel);// init volume univ

	screen.farz=FARZ*pref.bracket;
	mesh.size =1000*pref.bracket;
	mesh.sizex=mesh.size;
	mesh.sizey=mesh.size;
	update_mesh();
}

//-------------------------------------------------------------------------------




