//-------------------------------------------------------------------
//	File:		   objectn.c
//	Comments:   by George Ziniewicz
//-------------------------------------------------------------------

#include "object.h"
#include "main.h"
#include "utility.h"

#include "snd.h"

extern Snd snd;

//-----------------------------------------------------------------------------------

Object::Object(){
	origin=0; 
	size=1; sizex=1; sizey=1; sizez=1;
	init_bounds(0,0,0);
	xi=0; yi=0; zi=0;
	r=1; g=1; b=1; a=1; on=1;
	pitch =0; heading=0;  roll =0;
	pitchi=0; headingi=0; rolli=0;

	alphatest=1; alphafunc=.1; blendFunc0=2; blendFunc1=3; lightmap=0;
	blend=1;
	depthtest=1; depthwrite=1;	
	faceScreen=1;

	tex=0; texmagx=1; texmagy=1; controller=0;
	setup=1;
	return;
};
Object::~Object() {};

Hardware *Object::hw=0;

//-----------------------------------------------------------------------------------

void Object::init_bounds(float inx, float iny, float inz) {
	x=inx;
	y=iny;
	z=inz;
	switch(origin) {
	case 0:
		x0=x-.5*sizex;
		y0=y-.5*sizey;
		z0=z-.5*sizez;
		break;
	case 1:
		x0=x;
		y0=y;
		z0=z;
		break;
	}
	x1=x0+sizex;
	y1=y0+sizey;
	z1=z0+sizez;
	return;
}

//-----------------------------------------------------------------

void Object::move(float fDeltaTime, Pref *pref) {
	float spd=fDeltaTime;

	x+=xi*spd;
	y+=yi*spd;
	z+=zi*spd;
	pitch  +=pitchi  *spd;
	heading+=headingi*spd;
	roll   +=rolli   *spd;
	return;
}

//------------------------------------------------------------------------------

int Object::collision(Object *obj, Pref *pref) {
	float size2=(size+obj->size)*pref->bracket;

	if ((fabs(x-obj->x)<size2)&&
		(fabs(y-obj->y)<size2)&&
		(fabs(z-obj->z)<size2)) return 1;
	else return 0;
}

//------------------------------------------------------------------------------

void Object::bounce(float fDeltaTime, Pref *pref) {

	xi*=-1.1;
	yi*=-1.1;
	zi*=-1.1;
	move(fDeltaTime*2, pref);
	return;
	}

//------------------------------------------------------------------------------

void Object::draw(Camera *camera, Pref *pref) {
	if (!on) return;
	if (setup) set(pref);
	glPushMatrix();
		glTranslatef(x,y,z);
		//if (roll) glRotatef(roll,0,0,1);
		if (faceScreen) {
			glRotatef(-camera->heading,0,1,0);
			glRotatef(-camera->pitch  ,1,0,0);
		}
		//else {
			if (roll) glRotatef(roll,0,0,1);
			glRotatef(heading,0,1,0);
			glRotatef(pitch  ,1,0,0);
		//}
		glBegin(GL_QUADS);
			glTexCoord2f(0,0);             glVertex2f(-sizex,-sizey);
			glTexCoord2f(texmagx,0);       glVertex2f( sizex,-sizey);
			glTexCoord2f(texmagx,texmagy); glVertex2f( sizex, sizey);
			glTexCoord2f(0,      texmagy); glVertex2f(-sizex, sizey);
			glEnd();
		glPopMatrix();
	if (setup) unset(pref);
	return;
}

//------------------------------------------------------------------------------

void Object::set(Pref *pref) {
	int bfa[]={GL_ZERO,GL_ONE,
				  GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA,
				  GL_DST_ALPHA,GL_ONE_MINUS_DST_ALPHA,
				  GL_DST_COLOR,GL_ONE_MINUS_DST_COLOR,
				  GL_SRC_ALPHA_SATURATE};
	int bfb[]={GL_ZERO,GL_ONE,
				  GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA,
				  GL_DST_ALPHA,GL_ONE_MINUS_DST_ALPHA,
				  GL_SRC_COLOR,GL_ONE_MINUS_SRC_COLOR};

	if (alphatest) {
		glAlphaFunc(GL_GREATER,alphafunc);
		glEnable(GL_ALPHA_TEST);
	}
	if (blend) {
		glBlendFunc(bfa[blendFunc0],bfb[blendFunc1]);
		glEnable(GL_BLEND);
	}
	if (depthtest)   glEnable(GL_DEPTH_TEST);
	if (!depthwrite) glDepthMask(GL_FALSE);
	else             glDepthMask(GL_TRUE);
	if ((pref->textures)&&(tex)) {
		glEnable(GL_TEXTURE_2D);
		tex->bind();
	}
	glColor4f(r,g,b,a);
	return;
}

//------------------------------------------------------------------------------

void Object::setcolor(float inr, float ing, float inb, float ina) {
	r=inr;
	g=ing;
	b=inb;
	a=ina;
}

//------------------------------------------------------------------------------

void Object::unset(Pref *pref) {
	if (alphatest) {
		glDisable(GL_ALPHA_TEST);
		//glAlphaFunc(GL_GREATER,0);
	}
	if (blend) {
		glDisable(GL_BLEND);
		//glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	}
	if (depthtest)      glDisable(GL_DEPTH_TEST);
	if (!depthwrite)    glDepthMask(GL_TRUE);
	if (pref->textures) glDisable(GL_TEXTURE_2D);
	return;
}

//------------------------------------------------------------------------------

void Object::drag(float inspeed) {
	float damp;

	damp=1-hw->time->fDeltaTime*inspeed;
	xi*=damp;
	yi*=damp;
	zi*=damp;
}

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------

Camera::Camera() {
	size=80; sizex=size; sizey=size; sizez=size;
	r=0; g=0; b=0;
	cameraMove=0;
	fov=90; shakeampl=0; jogx=0;jogy=0;jogz=0;
	forwardx=0;forwardy=0;forwardz=0;
	rightx=0;righty=0;rightz=0;
	upx=0;upy=0;upz;
	moveSpeed=10; rotateSpeed=1;
	moveRangex=1;moveRangey=1;moveRangez=1;
	pitchRange=360; headingRange=360; rollRange=360;
	speedLimit=10;
	pref=0; time=0; mouse=0; maxthrust=.0002;
	center=(FARZ-NEARZ)*.5+NEARZ;
	translatex=0;
	translatey=0;
	translatez=0;
	centerx=0;
	centery=0;
	centerz=0;
};
Camera::~Camera() {};

//--------------------------------------------------------------------------

void Camera::shake(void) {
	shakeampl*=(1-2*time->fDeltaTime);
	x+=(shakeampl*cameraMove->get(3,.35*shakeampl,8,-4)+jogx);
	y+=(shakeampl*cameraMove->get(4,.35*shakeampl,8,-4)+jogy);
	z+=(shakeampl*cameraMove->get(5,.35*shakeampl,8,-4)+jogz);
	jogx*=(1-2*time->fDeltaTime); // dampen jiggle
	jogy*=(1-2*time->fDeltaTime);
	jogz*=(1-2*time->fDeltaTime);
	return;
}

//---------------------------------------------------------------------------

void Camera::get_matrix(void) {
	glGetFloatv(GL_MODELVIEW_MATRIX,model);
	rightx  = model[0];	righty  = model[4];	rightz  = model[ 8];
	upx     = model[1];	upy     = model[5];	upz     = model[ 9];
	forwardx=-model[2];	forwardy=-model[6];	forwardz=-model[10];
	//rightx  = model[0];	righty  = model[1];	rightz  = model[ 2];
	//upx     = model[4];	upy     = model[5];	upz     = model[ 6];
	//forwardx= model[8];	forwardy= model[9];	forwardz= model[10];
	translatex=model[12];
	translatey=model[13];
	translatez=model[14];
}

//---------------------------------------------------------------------------

void Camera::orient(void) {
	static int lastview=-1;

	switch(pref->view) {
		case 0:
			if (mouse->moved) mouse->moved=0;
			if (!pref->pause) {
				pitch  =cameraRotate->get(0,rotateSpeed  ,pitchRange,-pitchRange*.5);
				heading=cameraRotate->get(1,rotateSpeed,  headingRange,-headingRange*.5);
				//roll   =cameraRotate->get(2,rotateSpeed,rollRange,-rollRange*.5);

				x=cameraMove->get(0,moveSpeed,moveRangex,-moveRangex*.5)+centerx;
				y=cameraMove->get(1,moveSpeed,moveRangey,-moveRangey*.5)+centery;
				z=cameraMove->get(2,moveSpeed,moveRangez,-moveRangez*.5)+centerz;
			}
			break;
		case 1:
			if (mouse->moved) {
			if (lastview==0) {
					//pitch  =cameraRotate->get(0,rotateSpeed,100,-50);
					//heading=cameraRotate->get(1,rotateSpeed,720,-360);
				}
				pitchi=mouse->y*.1*pref->mouseSens;//.15
				pitch  +=pitchi;
				headingi=mouse->x*.1*pref->mouseSens;
				heading+=headingi;
				while(pitch>360) pitch-=360;
				while(pitch<  0) pitch+=360;
				while(heading>360) heading-=360;
				while(heading<  0) heading+=360;
				glutWarpPointer(mouse->snapx,mouse->snapy);
				mouse->moved=0;
			}
			break;
		case 2:
			if (mouse->moved) mouse->moved=0;
			break;
	}
	lastview=pref->view;
}

//---------------------------------------------------------------------------

void Camera::move_keys(Key *key) {
	float thrust;

	if (!key->move) return;

	thrust=maxthrust*time->fDeltaTime*pref->moveSens;
	if (key->alt) thrust*=10;
 
	if (key->move&1)  {xi-=rightx*thrust;  yi-=righty*thrust;  zi-=rightz*thrust;}// left
	if (key->move&2)  {xi+=rightx*thrust;  yi+=righty*thrust;  zi+=rightz*thrust;}// right
	if (key->move&4)  {xi+=forwardx*thrust;yi+=forwardy*thrust;zi+=forwardz*thrust;}// forward
	if (key->move&8)  {xi-=forwardx*thrust;yi-=forwardy*thrust;zi-=forwardz*thrust;}// back
	if (key->move&16) {xi+=upx*thrust*3;   yi+=upy*thrust*3;   zi+=upz*thrust*3;}// up
	if (key->move&32) {xi-=upx*thrust*3;   yi-=upy*thrust*3;   zi-=upz*thrust*3;}// down

	limitv(&xi,&yi,&zi,speedLimit+speedLimit*9*key->alt); // limit velocity
	return;
}

//---------------------------------------------------------------------------
void Camera::kick(void) {
	float thrust;

	thrust=maxthrust*time->fDeltaTime*pref->moveSens;
	xi+=forwardx*thrust;
	yi+=forwardy*thrust;
	zi+=forwardz*thrust;// forward
	limitv(&xi,&yi,&zi,speedLimit+speedLimit); // limit velocity
	return;
}

//---------------------------------------------------------------------------

int Camera::frustum_check(Vector v) {
	float fx,fy,fz,center2;

	center2=center*center*2;
	fx=x+forwardx*center;
	fy=y+forwardy*center;
	fz=z+forwardz*center;
	if ( ((fx-v.x)*(fx-v.x)+(fy-v.y)*(fy-v.y)+(fz-v.z)*(fz-v.z) )>center2) return 0;
	else return ((x-v.x)*(x-v.x)+
					 (y-v.y)*(y-v.y)+
					 (z-v.z)*(z-v.z));
}

//----------------------------------------------------------------------------



//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------

Key::Key() {key=0; skey=0; alt=0; shift=0; control=0; move=0; moved=0;};

Key::~Key() {};

void Key::set(int inkey) {
	key    =inkey;
	alt    =glutGetModifiers()&GLUT_ACTIVE_ALT;
	control=glutGetModifiers()&GLUT_ACTIVE_CTRL;
	shift  =glutGetModifiers()&GLUT_ACTIVE_SHIFT;
	return;
}

//-------------------------------------------------------------------------

void Key::set_move(int inkey) {
	skey=inkey;
	alt    =glutGetModifiers()&GLUT_ACTIVE_ALT;
	control=glutGetModifiers()&GLUT_ACTIVE_CTRL;
	shift  =glutGetModifiers()&GLUT_ACTIVE_SHIFT;

	switch(skey) {
		case GLUT_KEY_LEFT: move|=1; move&=(63-2); break;
		case GLUT_KEY_RIGHT:move|=2; move&=(63-1); break;
		case GLUT_KEY_UP:
			if (!control) {move|=4;  move&=(63-8-16-32);}
			else          {move|=16; move&=(63-4-8-32); }
			break;
		case GLUT_KEY_DOWN:
			if (!control) {move|=8;  move&=(63-4-16-32);}
			else          {move|=32; move&=(63-4-8-16); }
			break;
		//case GLUT_KEY_PAGE_UP:   move|=16;break;
		//case GLUT_KEY_PAGE_DOWN: move|=32;break;
	}
	return;
}

//-------------------------------------------------------------------------

void Key::unset_move(int inkey) {
	//alt    =glutGetModifiers()&GLUT_ACTIVE_ALT;
	//control=glutGetModifiers()&GLUT_ACTIVE_CTRL;
	//shift  =glutGetModifiers()&GLUT_ACTIVE_SHIFT;
	switch(inkey) {
		case GLUT_KEY_LEFT:      move&=(63-1); break;
		case GLUT_KEY_RIGHT:     move&=(63-2); break;
		case GLUT_KEY_UP:
			if (!control)        move&=(63-4);
			else                 move&=(63-16);
			break;
		case GLUT_KEY_DOWN:
			if (!control)        move&=(63-8);
			else                 move&=(63-32);
			break;
		//case GLUT_KEY_PAGE_UP:   move&=(63-16);break;
		//case GLUT_KEY_PAGE_DOWN: move&=(63-32);break;
	}
	return;
}

//------------------------------------------------------------------------
//------------------------------------------------------------------------

Light::Light() {
	lightn=0; direction=1; // 1-spot
	cutoff=180; exponent=1;
	r=1; g=1; b=1; inten=1;
	x=1; y=1; z=1;
	rotateSpeed=5;
};

Light::~Light() {};

//-------------------------------------------------------------------

void Light::position(float inx, float iny, float inz) {
	x=inx; y=iny; z=inz;
	return;
}

//-------------------------------------------------------------------

void Light::set(void) {
   int i;
   int linfo[]={GL_LIGHT0,GL_LIGHT1,GL_LIGHT2,GL_LIGHT3,
                GL_LIGHT4,GL_LIGHT5,GL_LIGHT6,GL_LIGHT7};
	GLfloat amba[4]={0,0,0,1.}, diffa[4]={0,0,0,1.},
		    speca[4]={0,0,0,1.}, refla[4]={0,0,0,1.},
		      pos[4]={0,0,0,0},  dir[3]={0,0,0};
	float c[3];

	c[0]=r*inten;
	c[1]=g*inten;
	c[2]=b*inten;

	if (!direction) {
		pos[0]=-x;
		pos[1]=-y;
		pos[2]=-z;
	} else {
		pos[0]=x; dir[0]=-x;
		pos[1]=y; dir[1]=-y;
		pos[2]=z; dir[2]=-z;
	}
	pos[3]=direction;
	for (i=0; i<3; i++) {
		 amba[i]= ambient   *c[i];
	    diffa[i]= diffuse   *c[i]; 
		speca[i]= specular  *c[i];
		refla[i]= reflective*c[i];
	}
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT,amba);
	glMaterialfv(  GL_FRONT_AND_BACK,GL_SPECULAR,refla);
	glMateriali(GL_FRONT_AND_BACK,GL_SHININESS,shininess);//5bigspot-50smallspot

	glLightfv(linfo[lightn],GL_AMBIENT, amba);
	glLightfv(linfo[lightn],GL_DIFFUSE, diffa);
	glLightfv(linfo[lightn],GL_SPECULAR,speca);

	glLightf(linfo[lightn],GL_SPOT_EXPONENT,exponent);
	glLightf(linfo[lightn],GL_SPOT_CUTOFF,cutoff);

	//glPushMatrix();
		//glLoadIdentity(); //if viewer relative
		//glTranslatef(x,y,z);
		glLightfv(linfo[lightn],GL_POSITION,       pos);
		glLightfv(linfo[lightn],GL_SPOT_DIRECTION, dir);
		//glPopMatrix();
	glEnable(linfo[lightn]);
	return;
}

//---------------------------------------------------------------------------------
//---------------------------------------------------------------------------------

Material::Material() {ambient=.25; diffuse=.5; specular=.75; reflective=1; shininess=25;};

Material::~Material() {};

//---------------------------------------------------------------------------------
//---------------------------------------------------------------------------------

Mouse::Mouse() {
	snapx=450; snapy=450;
	x=0;
	y=0;
	x_cartesian=0;
	y_cartesian=0;
	x_center=0;
	y_center=0;
	x_exp=0;
	y_exp=0;
	//lastx=0;lasty=0;
	mode=0; moved=0;
	mouseSensitivity=1;
	lbutton=0; mbutton=0; rbutton=0;
};

Mouse::~Mouse() {};

//--------------------------------------------------------------------------

void Mouse::set(int glutx, int gluty, Pref *pref, Screen *screen) {
	xcart=(float)                glutx   /screen->width;
	ycart=(float)(screen->height-gluty-1)/screen->height;

	switch(pref->view) {
		//0=auto-movement, cartesian ortho, mouse unused
		case 0: case 3:
		
		// 2=hold, cartesian ortho for popups
		case 2: x=glutx;       y=screen->height-gluty-1; // return cartesian
			break;
		
		// 1=relative mouse for 3d movement
		case 1: x=glutx-snapx; y=               gluty-snapy;
			break;
	}
	moved=1;
}

//---------------------------------------------------------------------------

void Mouse::set_button(int button, int state) {
	switch(button) {
		case GLUT_LEFT_BUTTON:
			switch(state) {
			case GLUT_DOWN:
				if (lbutton==0) lbutton=1;
				break;
			case GLUT_UP:
				if (lbutton==2) lbutton=3;
				break;
			}
			break;

		case GLUT_MIDDLE_BUTTON:
			switch(state) {
			case GLUT_DOWN:
				if (mbutton==0) mbutton=1;
				break;
			case GLUT_UP:
				if (mbutton==2) mbutton=3;
				break;
			}
			break;

		case GLUT_RIGHT_BUTTON:
			switch(state) {
			case GLUT_DOWN:
				if (rbutton==0) rbutton=1;
				break;
			case GLUT_UP:
				if (rbutton==2) rbutton=3;
				break;
			}
			break;
	}
	return;
}

//--------------------------------------------------------------------------

void Mouse::buttonstep(void) {
	if      (lbutton==1) lbutton=2;
	else if (lbutton==3) lbutton=0;
	if      (mbutton==1) mbutton=2;
	else if (mbutton==3) mbutton=0;
	if      (rbutton==1) rbutton=2;
	else if (rbutton==3) rbutton=0;
}

//--------------------------------------------------------------------------


void Mouse::set_cursor(Pref *pref, int dialogs) {
	if ((dialogs)||((pref->fullscreen==0)&&(pref->view==0))) 
		 glutSetCursor(GLUT_CURSOR_LEFT_ARROW);
	else {
		glutSetCursor(GLUT_CURSOR_NONE);
		glutWarpPointer(snapx,snapy);
	}
	return;
}

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------

Screen::Screen() {
	width=100; height=100;
	clipy0=height; clipy1=0; letterbox=0; colordepth=32;
	pref=0; nearz=NEARZ; farz=FARZ; needblank=0; r=0; g=0; b=0;
	camera=0; pref=0;
	maskr=0; maskg=0; maskb=0;
};

Screen::~Screen() {};

//---------------------------------------------------------------------------

void Screen::clear(void) {
	glClear(GL_DEPTH_BUFFER_BIT);
	if (pref->blank) {
		glClearColor(r,g,b, 1);
		if (pref->letterbox) fade(1);
		else if (needblank==0) {
			glClear(GL_COLOR_BUFFER_BIT);
		}
	}
	return;
}

//-------------------------------------------------------------------------------

void Screen::color(float inr, float ing, float inb) {
	r=inr; g=ing; b=inb;
	return;
}

//-------------------------------------------------------------------------------

void Screen::set_letterbox(int inletterbox) {
	letterbox=inletterbox;
	clipy0=   letterbox*.05 *height;
	clipy1=(1-letterbox*.05)*height;
	clipheight=clipy1-clipy0;
	//aspect=(GLfloat) width/clipheight;
	return;
}

//---------------------------------------------------------------------------

void Screen::set_viewport(void) {
	glViewport(0,clipy0,width,clipheight);
	//glMatrixMode(GL_PROJECTION);
	//glLoadIdentity();
	return;
}

//---------------------------------------------------------------------------

void Screen::set_ortho(void) {
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(0,width, clipy0,clipy1, -1,1); // l-r-b-t
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	
/*	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glFrustum(0,width, clipy0,clipy1, 1, 2);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
   glTranslatef(0,0,-1);*/
	return;
}

//---------------------------------------------------------------------------

void Screen::set_perspective(float aspect) {
	float v;
	//aspect=width/clipheight;
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	v=pow((letterbox/6.),2);
	gluPerspective(camera->fov-(v*44),aspect,//(GLfloat)width/clipheight,
					   nearz,(farz+pref->brace));//+pref->bracket); //farz
	//gluPerspective(camera->fov,aspect,//(GLfloat)width/clipheight,
	//				   nearz,(farz+pref->brace)+pref->bracket); //farz
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	gluLookAt(0,0,pref->brace, 0,0,pref->brace-1, 0,1,0);
	return;
}

//	v=(4.5*letterbox+letterbox*letterbox*.5)*fov/100.*1.2;
//	gluPerspective(fov-v, (GLfloat)width/clipheight,
//					   near, (far+pref->brace)+pref->bracket); 

//---------------------------------------------------------------------------

void Screen::fade(float alpha) { // variable screen fade F11/F12 control
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
	glDepthMask(GL_FALSE);
	glColor4f(r,g,b,alpha);
	glPushAttrib(GL_POLYGON_MODE);
		glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
		glRectf(0, clipy0, width, clipy1);
		glPopAttrib();
	glDisable(GL_BLEND);
	glDepthMask(GL_TRUE);
	return;
}

//---------------------------------------------------------------------------

void Screen::mask(void) { // letterbox mask
	glColor3f(maskr,maskg,maskb);
	glPushAttrib(GL_POLYGON_MODE);
		glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
		glRectf(0,0, width, clipy0);
		glRectf(0, clipy1, width, height);
		glPopAttrib();
	return;
}

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------

Time::Time() {
	fTime=0;
	fDeltaTime=.01;
	msTime=0;
	realTime=0;
	baseTime=0;
	lastTime=0;
};

Time::~Time() {};

//---------------------------------------------------------------------------

void Time::update(void) {
	msTime=get_time();
	realTime=msTime*.001;
	fTime=baseTime+realTime;
	fDeltaTime=fTime-lastTime;
	if (fDeltaTime>.1) {
		baseTime-=(fDeltaTime-.05);
		fDeltaTime=.05;
		fTime=baseTime+realTime;
	}
	lastTime=fTime;
	return;
}

//-------------------------------------------------------------------------------
//-------------------------------------------------------------------------------

Pref::Pref() {};
Pref::~Pref() {};

int Pref::load(const char* title) {
	char s[80];
	
	strcpy(s,title);
	strcat(s,".pref");
	ifstream file(s,ios::binary|ios::nocreate);
	if (file.bad()||(file==0)) return 1;
	file.read((char *) this, sizeof(Pref));
	file.close();
	return 0;
}

//-----------------------------------------------------------------------------------

int Pref::save(const char* title) {
	char s[80];

	strcpy(s,title);
	strcat(s,".pref");
	ofstream file(s,ios::binary|ios::out);
	if (file.bad()||(file==0)) return 1;
	file.write((char *) this, sizeof(Pref));
	file.close();
	return 0;
}

//-----------------------------------------------------------------------------------
//-----------------------------------------------------------------------------------
//-----------------------------------------------------------------------------------

Matrix::Matrix() {
	pitch=0; heading=0; roll=0;
	for (int i=0; i<16; i++) m[i]=0;
};

Matrix::~Matrix() {};

//----------------------------------------------------------------------------------

void Matrix::set_rot_axis(Vector axis, float value) {
	m[0]=                   cos(value)*axis.y+  cos(value)*axis.z;
	m[1]=                                       sin(value)*axis.z;
	m[2]=                  -sin(value)*axis.y;
	m[3]=0;
	m[4]=                                      -sin(value)*axis.z;
	m[5]= cos(value)*axis.x                    +cos(value)*axis.z;
	m[6]= sin(value)*axis.x;
	m[7]=0;
	m[8]=                   sin(value)*axis.y;
	m[9]=-sin(value)*axis.x;
	m[10]=cos(value)*axis.x+cos(value)*axis.y;
	m[11]=0;
	m[15]=1;
	if      (axis.x==1) m[0]=1;
	else if (axis.y==1) m[5]=1;
	else if (axis.z==1) m[10]=1;
}

//-----------------------------------------------------------------------------------

Vector Matrix::rotatev(Vector v) {
	Vector result;
	//result.x=v.x*m[0]+v.y*m[1]+v.z*m[2] +m[12];
	//result.y=v.x*m[4]+v.y*m[5]+v.z*m[6] +m[13];
	//result.z=v.x*m[8]+v.y*m[9]+v.z*m[10]+m[14];
	result.x=v.x*m[0]+v.y*m[4]+v.z*m[8] +m[12];
	result.y=v.x*m[1]+v.y*m[5]+v.z*m[9] +m[13];
	result.z=v.x*m[2]+v.y*m[6]+v.z*m[10]+m[14];
	return result;
}

//-----------------------------------------------------------------------------------

void Matrix::v2m(Camera *camera) {
	m[0]=camera->rightx;
	m[1]=camera->righty;
	m[2]=camera->rightz;
	m[3]=0;

	m[4]=camera->upx;
	m[5]=camera->upy;
	m[6]=camera->upz;
	m[7]=0;

	m[8] =camera->forwardx;
	m[9] =camera->forwardy;
	m[10]=camera->forwardz;
	m[11]=0;

	m[12]=camera->translatex;
	m[13]=camera->translatey;
	m[14]=camera->translatez;

	m[15]=1;
	glLoadMatrixf(m);
}

//-----------------------------------------------------------------------------------

Hardware::Hardware() {};
Hardware::~Hardware() {};

//----------------------------------------------------------------------------------
//----------------------------------------------------------------------------------
//----------------------------------------------------------------------------------
/*
Controller::Controller() {
	scalex=1;  scaley=1;  scalez=1;	scalexi=1; scaleyi=1; scalezi=1;
	scalepitch=1; scaleheading=1; scaleroll=1;
	offsetx=0;  offsety=0;  offsetz=0;	offsetxi=0; offsetyi=0; offsetzi=0;
	offsetpitch=0; offsetheading=0; offsetroll=0;
	speedx=1; speedy=1; speedz=1; speedpitch=1; speedheading=1; speedroll=1;
	counter=0;
	x=ARROWX;
	y=-1;
	z=ARROWY;
	pitch=MOUSEY;
	heading=MOUSEX;
	roll=-1;
	xi=-1;
	yi=-1;
	zi=-1;
};
Controller::~Controller() {};

Hardware *Controller::hw=0;

//-----------------------------------------------------------------------------------

void Controller::control(Object object) {
	switch(x) {
	case MOUSEX: object.x=hw->mouse->x     *hw->pref->mouseSens*scalex+offsetx;break;
	case ARROWX: object.x=hw->pref->arrownx*hw->pref->moveSens *scalex+offsetx;break;
	case AUTO:   object.x=counter->get(counterx,speedx,scalex,offsetx);break;
	}
	switch(y) {
	case MOUSEY: object.y     = hw->mouse->y     *hw->pref->mouseSens*scaley+offsety;break;
	//case MOUSEMINUSY: object.y=-hw->mouse->y     *hw->pref->mouseSens*scaley+offsety;break;
	case ARROWY: object.y     = hw->pref->arrowny*hw->pref->moveSens *scaley+offsety;break;
	//case ARROWMINUSY: object.y=-hw->pref->arrowny*hw->pref->moveSens *scaley+offsety;break;
	case AUTO:   object.y=counter->get(countery,speedy,scaley,offsety);break;
	}
	switch(z) {
	case MOUSEY:      object.z     = hw->mouse->y     *hw->pref->mouseSens*scalez+offsetz;break;
	//case MOUSEMINUSY: object.z=-hw->mouse->y     *hw->pref->mouseSens*scalez+offsetz;break;
	case ARROWY:      object.z     = hw->pref->arrowny*hw->pref->moveSens *scalez+offsetz;break;
	//case ARROWMINUSY: object.z=-hw->pref->arrowny*hw->pref->moveSens *scalez+offsetz;break;
	case LEFTBUTTON:  object.z=(hw->mouse->lbutton>0)*scalez+offsetz;	break;
	case AUTO:        object.z=counter->get(counterz,speedz,scalez,offsetz);break;
	}

	switch(xi) {
	case MOUSEX:     object.xi=hw->mouse->x     *hw->pref->mouseSens*scalexi+offsetxi;break;
	case ARROWXDOWN: object.xi=((((hw->key->move&3)-1)*2)-1)*hw->pref->moveSens *scalexi+offsetxi;break;
	}
	switch(yi) {
	case MOUSEY:          object.yi     = hw->mouse->y     *hw->pref->mouseSens*scaleyi+offsetyi;break;
	//case MOUSEMINUSY:     object.yi=-hw->mouse->y     *hw->pref->mouseSens*scaleyi+offsetyi;break;
	case ARROWYDOWN:      object.yi     = ((((hw->key->move&3)-1)*2)-1)*hw->pref->moveSens *scaleyi+offsetyi;break;
	//case ARROWMINUSYDOWN: object.yi=-((((hw->key->move&3)-1)*2)-1)*hw->pref->moveSens *scaleyi+offsetyi;break;
	}
	switch(zi) {
	case MOUSEY:          object.zi     = hw->mouse->y     *hw->pref->mouseSens*scalezi+offsetzi;break;
	//case MOUSEMINUSY:     object.zi=-hw->mouse->y     *hw->pref->mouseSens*scalezi+offsetzi;break;
	case ARROWYDOWN:      object.zi     = ((((hw->key->move&3)-1)*2)-1)*hw->pref->moveSens *scalezi+offsetzi;break;
	//case ARROWMINUSYDOWN: object.zi=-((((hw->key->move&3)-1)*2)-1)*hw->pref->moveSens *scalezi+offsetzi;break;
	case LEFTBUTTON:      object.zi=(hw->mouse->lbutton>0)*hw->pref->moveSens*scalezi+offsetzi;break;
	}

	switch(heading) {
	case MOUSEX: object.heading=hw->mouse->x     *hw->pref->mouseSens*scaleheading+offsetheading;break;
	case ARROWX: object.heading=hw->pref->arrownx*hw->pref->moveSens *scaleheading+offsetheading;break;
	case AUTO:   object.heading=counter->get(counterheading,speedheading,scaleheading,offsetheading);break;
	}
	switch(pitch) {
	case MOUSEY:      object.pitch     = hw->mouse->y     *hw->pref->mouseSens*scalepitch+offsetpitch;break;
	case ARROWY:      object.pitch     = hw->pref->arrowny*hw->pref->moveSens *scalepitch+offsetpitch;break;
	//case MOUSEMINUSY: object.pitch=-hw->mouse->y     *hw->pref->mouseSens*scalepitch+offsetpitch;break;
	//case ARROWMINUSY: object.pitch=-hw->pref->arrowny*hw->pref->moveSens *scalepitch+offsetpitch;break;
	case AUTO:        object.pitch=counter->get(counterpitch,speedpitch,  scalepitch,offsetpitch);break;
	}
	switch(roll) {
	case MOUSEX: object.roll= hw->mouse->x     *hw->pref->mouseSens*scaleroll+offsetroll;break;
	case ARROWX: object.roll= hw->pref->arrownx*hw->pref->moveSens *scaleroll+offsetroll;break;
	case AUTO:   object.roll=counter->get(counterroll,speedroll,    scaleroll,offsetroll);break;
	}
}
*/
//------------------------------------------------------------------------------------



