//-------------------------------------------------------------------
//	File:		   starfield.cpp
//	Comments:   by George Ziniewicz
//-------------------------------------------------------------------

#include "starfield.h"

//---------------------------------------------------------------------

Starfield::Starfield() {
	listID=0;
	blend=0;	depthtest=0; depthwrite=0; n=0;
	type=1; nlevel=1;
	size=1; hollow=0;
	minc=.3; maxc=.7;
	boundRight=2; boundLeft=-2;
	sizex=15; sizey=15; tex=0;
	camera=0;
};
Starfield::~Starfield() {
   if (listID!=0) {
      glDeleteLists(listID,1);
      listID=0;
      }
};

//---------------------------------------------------------------------

void Starfield::init(int n) {
   double x,y,z, size2, dist;
   int i;
   
   if (listID!=0) {
      glDeleteLists(listID,1);
      listID=0;
      }
   size2=size*2;
   listID=glGenLists(1);
   glNewList(listID,GL_COMPILE); // star points
    	glBegin(GL_POINTS);
         for(i=0;i<n;i++) {
			switch (type) {
			   default: 
			   case 0: dist=-.1; // hollow center sphere
				   while( (dist<hollow) || (dist>size) ) {
                     x=rndn(size2)-size;
                     y=rndn(size2)-size;
                     z=rndn(size2)-size;
					 dist=sqrt(x*x+y*y+z*z);
				  }
			  	  break;
			   case 1: x=0; y=0; z=0;// hollow center cube
                  while( (fabs(x)<hollow+.01)&&
                         (fabs(y)<hollow+.01)&&
                         (fabs(z)<hollow+.01) ) {
                     x=rndn(size2)-size;
                     y=rndn(size2)-size;
                     z=rndn(size2)-size;
				  }
			      break;
			   case 2: dist=-1; // hollow center dome
				   while( (dist<hollow) || (dist>size) || (y<0) ) {
                     x=rndn(size2)-size;
                     y=rndn(size2)-size;
                     z=rndn(size2)-size;
					 dist=sqrt(x*x+y*y+z*z);
				  }
			  break;
			}
            if (rndn(1)<.1) glColor3f((minc+rndn(maxc)),(minc+rndn(maxc)),(minc+rndn(maxc))); // few colored
               else         glColor3f((maxc+rndn(minc)),(maxc+rndn(minc)),(maxc+rndn(minc))); // mostly whitish
            glVertex3d(x,y,z);
            }
         glEnd();	
      glEndList();
   return;
   }

//----------------------------------------------------

void Starfield::draw(void) {
	if (blend) {
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		glEnable(GL_BLEND); // need for smoothing
	}
	if (depthtest) {
		if (depthwrite) glDepthMask(GL_TRUE);
		glEnable(GL_DEPTH_TEST);
	}
	glPushMatrix();
		glPointSize(1);
		glCallList(listID);
		if (nlevel>=2) {
			glRotatef(90,0,1,0); // flip over and reuse
			glPointSize(2);
			glCallList(listID);
		}
		if (nlevel>=3) {
			glRotatef(90,0,1,0);
			glPointSize(3);
			glCallList(listID);
		}
		glPopMatrix();

	if (blend) {
		glDisable(GL_BLEND);
	}
	if (depthtest) {
		glDisable(GL_DEPTH_TEST);
	}
	return;
}

//---------------------------------------------------------------------

void Starfield::destroy(void) {
   if (listID!=0) {
      glDeleteLists(listID,1);
      listID=0;
      }
   return;
   }

//---------------------------------------------------------------------
//---------------------------------------------------------------------
//---------------------------------------------------------------------

void Starfield::init2(int inn) {
   double x,y,z, size2, dist;
   int i;

   n=inn;
	if (n>=maxarray) n=maxarray;
   size2=size*2;
   for(i=0; i<n; i++) {
		switch (type) {
			default:
			case 0: // hollow center sphere
				dist=-.1;
				while( (dist<hollow) || (dist>size) ) {
					x=rndn(size2)-size;
					y=rndn(size2)-size;
					z=rndn(size2)-size;
					dist=sqrt(x*x+y*y+z*z);
				} break;
			case 1: // hollow center cube
				x=0; y=0; z=0;
				while( (fabs(x)<hollow+.01)&&
						 (fabs(y)<hollow+.01)&&
						 (fabs(z)<hollow+.01) ) {
					x=rndn(size2)-size;
					y=rndn(size2)-size;
					z=rndn(size2)-size;
				} break;
			case 2: // hollow center dome
				dist=-1;
				while( (dist<hollow) || (dist>size) || (y<0) ) {
						x=rndn(size2)-size;
						y=rndn(size2)-size;
						z=rndn(size2)-size;
						dist=sqrt(x*x+y*y+z*z);
				} break;
		}
		if (rndn(1)<.1) {
			rs[i]=minc+rndn(maxc);
			gs[i]=minc+rndn(maxc);
			bs[i]=minc+rndn(maxc);
		} else {
         rs[i]=maxc+rndn(minc);
			gs[i]=maxc+rndn(minc);
			bs[i]=maxc+rndn(minc);
		}
		xs[i]=x;
		ys[i]=y;
		zs[i]=z;
	}
	return;
}

//--------------------------------------------------------------------------

void Starfield::update2(float fDeltaTime, float page) {
   float spd, max2,min2;
   int i;
	
	//mins=-maxs;
	max2=boundRight*2;
	min2=boundLeft*2;

   spd=fDeltaTime;//*page;
   for(i=0; i<10; i++) {
		xs[i]-=camera->xi*spd;
		ys[i]-=camera->yi*spd;
		zs[i]-=camera->zi*spd;
		if      (xs[i]>(camera->x+boundRight)) xs[i]-=max2;
		else if (xs[i]<(camera->x+boundLeft))  xs[i]-=min2;
		if      (ys[i]>(camera->y+boundRight)) ys[i]-=max2;
		else if (ys[i]<(camera->y+boundLeft))  ys[i]-=min2;
		if      (zs[i]>(camera->z+boundRight)) zs[i]-=max2;
		else if (zs[i]<(camera->z+boundLeft))  zs[i]-=min2;
	}
	return;
}

//----------------------------------------------------

void Starfield::draw2(Pref *pref) {
	int i;

	if (blend) {
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		glEnable(GL_BLEND); // need for smoothing
	}
	if (depthtest) {
		if (!depthwrite) glDepthMask(GL_FALSE);
		glEnable(GL_DEPTH_TEST);
	}
	if (pref->textures) {
		glEnable(GL_TEXTURE_2D);
		tex->bind();
	}
   for (i=0; i<n; i++) {
		Vector v;
		v.x=xs[i];
		v.y=ys[i];
		v.z=zs[i];
		//if (camera->frustum_check(v)) {
			glColor3f(rs[i],gs[i],bs[i]);
			//glColor4f(1,1,1,1);
			glPushMatrix();
				glTranslatef(v.x,v.y,v.z);
				//glRotatef(roll   ,0,0,1);
				glRotatef(-camera->heading,0,1,0);
				glRotatef(-camera->pitch  ,1,0,0);

				glBegin(GL_QUADS);
					glTexCoord2f(0,0); glVertex2f(-sizex*pref->bracket,-sizey*pref->bracket);
					glTexCoord2f(1,0); glVertex2f( sizex*pref->bracket,-sizey*pref->bracket);
					glTexCoord2f(1,1); glVertex2f( sizex*pref->bracket, sizey*pref->bracket);
					glTexCoord2f(0,1); glVertex2f(-sizex*pref->bracket, sizey*pref->bracket);
					glEnd();
				glPopMatrix();
		//}
	}
	if (blend) {
		glDisable(GL_BLEND);
		//glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	}
	if (depthtest) {
		glDisable(GL_DEPTH_TEST);
		if (!depthwrite) glDepthMask(GL_TRUE);
	}
	if (pref->textures) {
		glDisable(GL_TEXTURE_2D);
	}
	return;
}

//---------------------------------------------------------------------

