#include "text.h"

//----------------------------------------------------------------------------

Text::Text() {
	size=10;
	reset();

};

Text::~Text() {};

Texture *Text::tex    = 0;
Counter *Text::counter= 0;
Time    *Text::time   = 0;
Screen  *Text::screen = 0;

//---------------------------------------------------------------------------

void Text::reset(void) {
	alphatest=1; alphafunc=.1;
	blend=0; duration=3; fmode=0;
	r=1; g=1; b=1; a=1; inten=1;
	x=20; y=20; xinc=.9;
	endTime=0; italic=3;
	size=10; 

	bend=0;     bendspeedx=1;   bendspeedy=1;   bendamplx=1;   bendamply=1;
	drift=0;   driftspeedx=1;  driftspeedy=1;  driftamplx=1;  driftamply=1;
	glitch=0; glitchspeedx=1; glitchspeedy=1; glitchamplx=1; glitchamply=1;
	bloom=0;   bloomspeedx=1;  bloomspeedy=1;  bloomamplx=1;  bloomamply=1;
	fmodeamplx=0; fmodeamply=0; 
	bloomcrawl=1;
}

//---------------------------------------------------------------------------

void Text::hold(void) {
	endTime=time->fTime+duration;	return;
}

//---------------------------------------------------------------------------

void Text::hold(char *ins) {
	strcpy(s,ins);	endTime=time->fTime+duration;	return;
}

//---------------------------------------------------------------------------

void Text::hold(char *format, float value) {
	sprintf(s,format,value);	endTime=time->fTime+duration;	return;
}

//---------------------------------------------------------------------------

void Text::hold(char *format, double value) {
	sprintf(s,format,value);	endTime=time->fTime+duration ;return;
}

//---------------------------------------------------------------------------

void Text::hold(char *format, int value) {
	sprintf(s,format,value);	endTime=time->fTime+duration;	return;
}

//---------------------------------------------------------------------------

void Text::hold(char *format, char *value) {
	sprintf(s,format,value);	endTime=time->fTime+duration;	return;
}

//---------------------------------------------------------------------------

void Text::printHold(void) {
	if (endTime) {
		if (time->fTime<endTime) {
			if (screen->letterbox) screen->needblank=1;
			print(s, x, y, size);
		}
		else {
			endTime=0;
			screen->needblank=1;
		}
	}
}

//---------------------------------------------------------------------------

void Text::print(char *ins, float inx, float iny, float insize) {
	unsigned char c;
	unsigned int i;

	float dx,dy, tx,ty, xsize,ysize, txsize=1/16.,tysize=1/8.;
	float bendx,bendy, driftx,drifty, bloomx,bloomy;
	float tx2,ty2,fx,fy;

	strcpy(s,ins);
	x=inx;
	y=iny;
	size=insize;

	if (bend) {
		bendx=counter->get(1,bendspeedx*100,2,-1)*insize*bendamplx*.2;
		bendy=counter->get(2,bendspeedy*100,2,-1)*insize*bendamply*.2;
	} else {
		bendx=0;
		bendy=0;
	}
	if (drift) {
		driftx=counter->get(3,driftspeedx*100,2,-1)*insize*driftamplx*.5;
		drifty=counter->get(4,driftspeedy*100,2,-1)*insize*driftamply*.5;
	} else {
		driftx=0;
		drifty=0;
	}
	if (glitch) {
		if (rndn(1)*rndn(1)<.005*glitchspeedx) driftx+=(rndn(1)-.5)*insize*glitchamplx*2;
		if (rndn(1)*rndn(1)<.005*glitchspeedy) drifty+=(rndn(1)-.5)*insize*glitchamply*2;
	}
	dx=inx+driftx;
	dy=iny+drifty;

	if (alphatest) {
		glEnable(GL_ALPHA_TEST);
		glAlphaFunc(GL_GREATER, alphafunc);
	}
	if (blend) {
		glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	}
	glDepthMask(GL_FALSE);
	glEnable(GL_TEXTURE_2D);
	tex->bind();
	glColor4d(r*inten,g*inten,b*inten,a);

	glPushAttrib(GL_POLYGON_BIT);
	glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
	//glPushMatrix();
		for (i=0; i<strlen(ins); i++) {
			if (bloom) {
				bloomx=counter->get2(5,bloomspeedx*25,(i-time->fTime*5)*bloomcrawl,1.5,.75)*bloomamplx*.5;
				bloomy=counter->get2(6,bloomspeedy*25,(i-time->fTime*5)*bloomcrawl,1.5,.75)*bloomamply*.5;
			} else {
				bloomx=1;
				bloomy=1;
			}
			xsize=insize*.5*bloomx;
			ysize=insize*   bloomy;
			c=ins[i]-32;// don't do space?
			tx=(c%16)*txsize;
			ty=(1-tysize)-(int)(c*txsize)*tysize;
			tx2=tx+txsize;
			ty2=ty+tysize;
			glBegin(GL_QUADS);
				glTexCoord2f(tx ,ty );
				fx=dx-xsize+bendx; fy=dy-ysize-bendy; f(&fx,&fy); glVertex2f(fx,fy);

				glTexCoord2f(tx2,ty );
				fx=dx+xsize       -bendx; fy=dy-ysize+bendy; f(&fx,&fy); glVertex2f(fx,fy);

				glTexCoord2f(tx2,ty2);
				fx=dx+xsize+italic+bendx; fy=dy+ysize-bendy; f(&fx,&fy); glVertex2f(fx,fy);

				glTexCoord2f(tx ,ty2);
				fx=dx-xsize+italic-bendx; fy=dy+ysize+bendy; f(&fx,&fy); glVertex2f(fx,fy);

				glEnd();
			dx+=insize*xinc;//*bloomx;
		}
		//glPopMatrix();
	glPopAttrib();
	if (alphatest) {
		glDisable(GL_ALPHA_TEST);
		glAlphaFunc(GL_GREATER,0);
	}
	if (blend) {
		glDisable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	}
	glDepthMask(GL_TRUE);
	glDisable(GL_TEXTURE_2D);
	return;
	}

//--------------------------------------------------------------------------------

void Text::f(float *inx, float *iny) {
	float x,y, x2,y2;

	if ((fmode==0)||(fmodeamplx==0)) return;
	switch(fmode) {
	case 1:
		x=(*inx-screen->width *.5)*.002;
		y=(*iny-screen->height*.5)*.002;

		x2=( cos(counter->get(1,1,720,-360))*x+
			  sin(counter->get(2,1,720,-360))*y);
		y2=(-sin(counter->get(3,1,720,-360))*x+
			  cos(counter->get(4,1,720,-360))*y);

		x2=x2*500+screen->width *.5;
		y2=y2*500+screen->height*.5;

		*inx=lin_interp(*inx,x2,fmodeamplx);
		*iny=lin_interp(*iny,y2,fmodeamply);

		break;
	case 2:
		break;
	}
	return;
}

//---------------------------------------------------------------------------

void Text::color(float red, float green, float blue) {
	r=red; g=green; b=blue;
	return;
}

//--------------------------------------------------------------------------------

void Text::color(float red, float green, float blue, float inalpha) {
	r=red; g=green; b=blue; a=inalpha;
	return;
}

//--------------------------------------------------------------------------------
//--------------------------------------------------------------------------------
//--------------------------------------------------------------------------------
	
Clock::Clock() {
	nextsecond=0;
	framecnt=0, fps=0;
	blend=0; alphatest=0;
};

Clock::~Clock() {};

//-------------------------------------------------------------------------------

void Clock::draw(void) {
	hold("FPS: %d",fps);
	printHold();
}

//--------------------------------------------------------------------------

void Clock::update(void) {
	framecnt++;
	if (time->fTime >= nextsecond) {
 		nextsecond+=1;
 		fps=framecnt;
 		framecnt=0;
	}
	return;
}

//---------------------------------------------------------------------------





