//-------------------------------------------------------------------
//	File:		   texture.cpp
//	Comments:   by George Ziniewicz
//-------------------------------------------------------------------

#include "texture.h"
#include "main.h"
extern float perlin_noise2(float x, float y, float persistence=.25, int noctave=6);

//-------------------------------------------------------------------

Texture::Texture() {
	texID=0;
	repeatMode=GL_REPEAT;
	blendMode=GL_MODULATE;
	return;
};
Texture::~Texture() {
	if (texID) glDeleteTextures(1, &texID);
};

GLuint Texture::lastTexID=0;

//--------------------------------------------------------------------

void Texture::bind(void) {

	if (texID==0) glGenTextures(1, &texID);
	if (texID==lastTexID) return; // avoid binding current texture again
	lastTexID=texID;
	glBindTexture(GL_TEXTURE_2D, texID);
	if (repeatMode) { // if custom repeatmode *** move back 2 loader
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,repeatMode);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,repeatMode);
	}
	if (blendMode) glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,blendMode); // if custom blend
	return;
}

//--------------------------------------------------------------------

void Texture::maketextype(int type, int mipmap, float inf, int ini) {
	unsigned char image[128][128][4];
	float f[128][128];
	int i, x,y, rx, ry, niter=64;
	double size=16;
	float  r,g,b, lum, len, max=-1;

	if (type==SMOKE) {// smoke
		for (i=0; i<niter; i++) {
			rx=sqrt(rndn(1))*rndn(128-size*2)+size;
			ry=sqrt(rndn(1))*rndn(128-size*2)+size;
			for    (y = 0; y <128; y++) {
				for (x = 0; x <128; x++) {
					if (i==0) f[y][x]=0;
					lum=sqrt((x-rx)*(x-rx)+(y-ry)*(y-ry))/size;
					if (lum>1) lum=1;
					lum=1-lum;
					f[y][x]+=lum;
					if (f[y][x]>max) max=f[y][x];
				}
			}
		}
		max=(255./max);
		for     (y = 0; y <128; y++) {
			for (x = 0; x <128; x++) {
				lum=f[y][x]*max;
				image[y][x][0]=image[y][x][1]=image[y][x][2]=image[y][x][3]=lum;
			}
		}
	}
	else {
		for    (y = 0; y <128; y++) {
			int grain=ini;
			for (x = 0; x <128; x++) {
				switch(type) {
					float x1,y1;
				case SPHERE:        // sphere
					x1=x-64;
					y1=y-64;
					lum=sqrt(x1*x1+y1*y1)/inf;
					if (lum>1) lum=1;
					lum=lum*lum;
					lum=(1-lum)*((254-grain)+rndn(grain));//////////////////////
					r=g=b=lum;
					break;
				case NOTSPHERE:        // sphere
					x1=x-64;
					y1=y-64;
					lum=sqrt(x1*x1+y1*y1)/inf;
					if (lum>1) lum=1;
					lum=lum*lum;
					lum=(1-lum)*((254-grain)+rndn(grain));//////////////////////
					lum=255-lum;//
					r=g=b=lum;
					break;
				case GRADIENT:        // gradient
					lum=(16.-sqrt(y*2.))*15;
					r=g=b=lum;
					break;
				case HALO:        // halo
					len=sqrt((x-64)*(x-64)+(y-64)*(y-64));
					lum=len/58;
					if (lum>1) lum=9-lum*8;
					if (lum<0) lum=0;
					else lum=pow(lum,1)*255;
					r=g=b=lum;
					break;
				case FIREHALO:       // fire halo 
					if (y>64) lum=0; else lum=y-48+sin(x*.5)+sin(x*1.1);
					lum=(lum< 0) ? 0 : (lum>16) ? 16 : lum;
					lum=cos_interp(0,255, lum /16.);
					r=g=b=lum;
					break;
				case STAR:       // star
					float lx,ly;
					lx=(64.-fabs(x-64.))/64.;
					ly=(64.-fabs(y-64.))/64.;
					lum=lx*ly;
					lum= (lum<0) ? 0 : (lum>1) ? 1 : lum;
					//lum=(1-lum);
					lum*=255;
					r=g=b=lum;
					break;
				case STARFIELD:
					if (rndn(1)>.04) {
						r=0; g=0; b=0; lum=0;
					} else {
						r=rndn(128)+127;
						g=rndn(128)+127;
						b=rndn(128)+127; lum=255;
					}
					break;
				case GRID:
					{
						int nx,ny, n=4;
						nx=((float)x/n==(int)x/n);
						ny=((float)y/n==(int)y/n);
						lum=(nx|ny)*255;
						r=g=b=lum;
					}
					break;
				case CHECK:
					{
						int nx,ny, n=8;
						nx=((int)x/n)&1;
						ny=((int)y/n)&1;
						lum=(nx^ny)*255;
						r=g=b=lum;
					}
					break;
				case SAND:
					{
						float n;
						lum=255;
						n=rndn(1);
						if      (n<.001) r=g=b=0;
						else if (n<.002) r=g=b=255;
						else {
							r=rndn(127)+127;
							g=r*.8;
							b=r*.6;
						}
					}
					break;
				case PERLIN:
					lum=(perlin_noise2(x,y,inf,ini)*.5+.5)*255;
					r=g=b=lum;
					break;

				case CROSS:
					x1=x-64; // sphere
					y1=y-64;
					lum=sqrt(x1*x1+y1*y1)/inf;
					if (lum>1) lum=1;
					lum=lum*lum;
					lum=(1-lum)*((254-grain)+rndn(grain));
					if ((abs(x1)<20)||(abs(y1)<20)) {
						lum=255;
						r=255;
						g=0;
						b=0;
					} else r=g=b=lum;
					break;
				
				case SPARK:
					lum=(64-fabs((x)-64));
					lum=sqrt(lum);
					lum=lum/7;
					lum=maxv(0,minv(lum,1));
					lum=pow(lum,2);
					//if (lum>6) lum=6;
					lum*=255;//42;//31;

					//if (sqrt((x-64)*(x-64)+
					//	      (y-64)*(y-64))<40) lum=255; else lum=0;
					if (rndn(255-lum)<30)	lum=255;
					r=lum;
					g=lum;
					b=lum;
					//lum=255;
					break;
				}
				image[y][x][0] = r;
				image[y][x][1] = g;
				image[y][x][2] = b;
				image[y][x][3] = lum;
			}
		}
	}

   bind();
	mip(mipmap);
	glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,128,128,0,GL_RGBA,GL_UNSIGNED_BYTE,image);
	if (mipmap) gluBuild2DMipmaps(GL_TEXTURE_2D,4,128,128,GL_RGBA,GL_UNSIGNED_BYTE,image);
	return;
}

//---------------------------------------------------------------------------------------

void Texture::mip(int mipmap) {
   glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
	if (mipmap) {
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST_MIPMAP_NEAREST);
	} else {
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	}
	return;
}

//----------------------------------------------------------------------------------------

void Texture::maketextypeint(int type, int mipmap, float inf, int ini) {
	unsigned char image[128][128][2];
	float f[128][128];
	int i, x,y, rx, ry, niter=64;
	double size=16;
	float  r, lum, len, max=-1;

	if (type==SMOKE) {// smoke
		for (i=0; i<niter; i++) {
			rx=sqrt(rndn(1))*rndn(128-size*2)+size;
			ry=sqrt(rndn(1))*rndn(128-size*2)+size;
			for    (y = 0; y <128; y++) {
				for (x = 0; x <128; x++) {
					if (i==0) f[y][x]=0;
					lum=sqrt((x-rx)*(x-rx)+(y-ry)*(y-ry))/size;
					if (lum>1) lum=1;
					lum=1-lum;
					f[y][x]+=lum;
					if (f[y][x]>max) max=f[y][x];
				}
			}
		}
		max=(255./max);
		for    (y = 0; y <128; y++) {
			for (x = 0; x <128; x++) {
				lum=f[y][x]*max;
				image[y][x][0]=image[y][x][1]=lum;
			}
		}
	}
	else {
		for    (y = 0; y <128; y++) {
			int grain=ini;
			for (x = 0; x <128; x++) {
				switch(type) {
					float x1,y1;
				case SPHERE:        // sphere
					x1=x-64;
					y1=y-64;
					lum=sqrt(x1*x1+y1*y1)/inf;
					if (lum>1) lum=1;
					lum=lum*lum;
					lum=(1-lum)*((254-grain)+rndn(grain));//////////////////////
					r=lum;
					break;
				case NOTSPHERE:        // sphere
					x1=x-64;
					y1=y-64;
					lum=sqrt(x1*x1+y1*y1)/inf;
					if (lum>1) lum=1;
					lum=lum*lum;
					lum=(1-lum)*((254-grain)+rndn(grain));//////////////////////
					lum=255-lum;//
					r=lum;
					break;
				case GRADIENT:        // gradient
					lum=(16.-sqrt(y*2.))*15;
					r=lum;
					break;
				case HALO:        // halo
					len=sqrt((x-64)*(x-64)+(y-64)*(y-64));
					lum=len/58;
					if (lum>1) lum=9-lum*8;
					if (lum<0) lum=0;
					else lum=pow(lum,1)*255;
					r=lum;
					break;
				case FIREHALO:       // fire halo 
					if (y>64) lum=0; else lum=y-48+sin(x*.5)+sin(x*1.1);
					lum=(lum< 0) ? 0 : (lum>16) ? 16 : lum;
					lum=cos_interp(0,255, lum /16.);
					r=lum;
					break;
				case STAR:       // star
					float lx,ly;
					lx=(64.-fabs(x-64.))/64.;
					ly=(64.-fabs(y-64.))/64.;
					lum=lx*ly;
					lum= (lum<0) ? 0 : (lum>1) ? 1 : lum;
					//lum=(1-lum);
					lum*=255;
					r=lum;
					break;
				case STARFIELD:
					if (rndn(1)>.04) {
						r=0;
					} else {
						r=rndn(128)+127;
					}
					break;
				case GRID:
					{
						int nx,ny, n=4;
						nx=((float)x/n==(int)x/n);
						ny=((float)y/n==(int)y/n);
						lum=(nx|ny)*255;
						r=lum;
					}
					break;
				case CHECK:
					{
						int nx,ny, n=8;
						nx=((int)x/n)&1;
						ny=((int)y/n)&1;
						lum=(nx^ny)*255;
						r=lum;
					}
					break;
				case SAND:
					{
						float n;
						lum=255;
						n=rndn(1);
						if      (n<.001) r=0;
						else if (n<.002) r=255;
						else {
							r=rndn(127)+127;
						}
					}
					break;
				case PERLIN:
					lum=(perlin_noise2(x,y,inf,ini)*.5+.5)*255;
					r=lum;
					break;
				
				case SPARK:
					lum=(64-fabs((x)-64));
					lum=sqrt(lum);
					lum=lum/7;
					lum=maxv(0,minv(lum,1));
					lum=pow(lum,2);
					//if (lum>6) lum=6;
					lum*=255;//42;//31;

					//if (sqrt((x-64)*(x-64)+
					//	      (y-64)*(y-64))<40) lum=255; else lum=0;
					if (rndn(255-lum)<30)	lum=255;
					r=lum;
					//lum=255;
					break;

				}
				image[y][x][0] = r;
				image[y][x][1] = lum;
			}
		}
	}

   bind();
	mip(mipmap);
	glTexImage2D(GL_TEXTURE_2D,0,GL_LUMINANCE_ALPHA,128,128,0,GL_LUMINANCE_ALPHA,GL_UNSIGNED_BYTE,image);
	if (mipmap) gluBuild2DMipmaps(GL_TEXTURE_2D,4,128,128,GL_LUMINANCE_ALPHA,GL_UNSIGNED_BYTE,image);
	return;
}

//---------------------------------------------------------------------------------------

void Texture::maketextype2(int type, int mipmap, float inf, int ini) {
	unsigned char image[256][256][4];
	float f[256][256];
	int i, x,y, rx, ry, niter=64;
	double size=32;
	float  r,g,b, lum, len, max=-1;

	if (type==SMOKE) {// smoke
		for (i=0; i<niter; i++) {
			rx=sqrt(rndn(1))*rndn(256-size*2)+size;
			ry=sqrt(rndn(1))*rndn(256-size*2)+size;
			for    (y = 0; y <256; y++) {
				for (x = 0; x <256; x++) {
					if (i==0) f[y][x]=0;
					lum=sqrt((x-rx)*(x-rx)+(y-ry)*(y-ry))/size;
					if (lum>1) lum=1;
					lum=1-lum;
					f[y][x]+=lum;
					if (f[y][x]>max) max=f[y][x];
				}
			}
		}
		max=(255./max);
		for    (y = 0; y <256; y++) {
			for (x = 0; x <256; x++) {
				lum=f[y][x]*max;
				image[y][x][0]=image[y][x][1]=image[y][x][2]=image[y][x][3]=lum;
			}
		}
	}
	else {
		for    (y = 0; y <256; y++) {
			for (x = 0; x <256; x++) {
				int grain=20;
				switch(type) {
				case SPHERE: case NOTSPHERE:        // sphere
					lum=sqrt((x-128)*(x-128)+(y-128)*(y-128))/128.;
					if (lum>1) lum=1;
					lum=lum*lum;
					lum=(1-lum)*(254-grain)+rndn(grain);///////////////////////////
					if (type==NOTSPHERE) lum=255-lum;
					r=g=b=lum;
					break;
				case GRADIENT:        // gradient
					lum=(32.-sqrt(y*2.))*15;
					r=g=b=lum;
					break;
				case HALO:        // halo
					len=sqrt((x-128)*(x-128)+(y-128)*(y-128));
					lum=len/58;
					if (lum>1) lum=9-lum*8;
					if (lum<0) lum=0;
					else lum=pow(lum,1)*255;
					r=g=b=lum;
					break;
				case FIREHALO:       // fire halo 
					if (y>128) lum=0; else lum=y-96+sin(x*.5)+sin(x*1.1);
					lum=(lum< 0) ? 0 : (lum>16) ? 16 : lum;
					lum=cos_interp(0,255, lum /16.);
					r=g=b=lum;
					break;
				case STAR:       // star
					float lx,ly;
					lx=(128.-fabs(x-128.))/128.;
					ly=(128.-fabs(y-128.))/128.;
					lum=lx*ly;
					lum= (lum<0) ? 0 : (lum>1) ? 1 : lum;
					//lum=(1-lum);
					lum*=255;
					r=g=b=lum;
					break;
				case STARFIELD:
					if (rndn(1)>.04) {
						r=0; g=0; b=0;
					} else {
						r=rndn(128)+127;
						g=rndn(128)+127;
						b=rndn(128)+127;
					}
					break;
				case GRID:
					{
						int nx,ny, n=8;
						nx=((float)x/n==(int)x/n);
						ny=((float)y/n==(int)y/n);
						lum=(nx|ny)*255;
						r=g=b=lum;
					}
					break;
				case CHECK:
					{
						int nx,ny, n=16;
						nx=((int)x/n)&1;
						ny=((int)y/n)&1;
						lum=(nx^ny)*255;
						r=g=b=lum;
					}
					break;
				case SAND:
					{
						float n;
						lum=255;
						n=rndn(1);
						if      (n<.001) r=g=b=0;
						else if (n<.002) r=g=b=255;
						else {
							r=rndn(127)+127;
							g=r*.8;
							b=r*.6;
						}
					}
					break;
				case PERLIN:
					lum=(perlin_noise2(x,y,inf,ini)*.5+.5)*255;
					r=g=b=lum;
					break;
				
				case SPARK:
					lum=( 128-fabs((x)-128) );
					lum=sqrt(lum);
					lum=lum/7;
					lum=maxv(0,minv(lum,1));
					lum=pow(lum,2);
					//if (lum>6) lum=6;
					lum*=255;//42;//31;

					//if (sqrt((x-64)*(x-64)+
					//	      (y-64)*(y-64))<40) lum=255; else lum=0;
					if (rndn(255-lum)<2) lum=255;
					r=lum;
					g=lum;
					b=lum;
					//lum=255;
					break;

				}
				image[y][x][0] = r;
				image[y][x][1] = g;
				image[y][x][2] = b;
				image[y][x][3] = lum;
			}
		}
	}
   bind();
	mip(mipmap);
	glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,256,256,0,GL_RGBA,GL_UNSIGNED_BYTE,image);
	if (mipmap) gluBuild2DMipmaps(GL_TEXTURE_2D,4,256,256,GL_RGBA,GL_UNSIGNED_BYTE,image);
	return;
}

//---------------------------------------------------------------------------------------

int Texture::loadtex(const char *picname, int alpha, int threshold, int mipmap) {
	FILE *file;
	unsigned char image[65536][4];
	int x,y, addr, red,green,blue, sizex, sizey;

	file= fopen(picname, "rb");
    if (file == 0) return 1;
    fseek(file, 18, SEEK_SET);
	 sizex=fgetc(file)+fgetc(file)*256;
    fseek(file, 22, SEEK_SET);
	 sizey=fgetc(file)+fgetc(file)*256;
	 if (sizex*sizey>65536) return 2;

    fseek(file, 54, SEEK_SET);
    for     (y = 0; y <sizey; y++) {
        for (x = 0; x <sizex; x++) {
			addr=x+y*sizey;
			blue =fgetc(file);
			green=fgetc(file);
			red  =fgetc(file);
			image[addr][0] = red;
			image[addr][1] = green;
			image[addr][2] = blue;
			if (alpha<0) image[addr][3]=(red+green+blue)/3;
			else {
				if ((red  <threshold) &&
					 (green<threshold) &&
					 (blue <threshold)) image[addr][3]=0;
				else					     image[addr][3]=alpha;
			}
		}
	}
	fclose(file);

   bind();
	mip(mipmap);
	glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,sizex,sizey,0,GL_RGBA,GL_UNSIGNED_BYTE,image);
	if (mipmap) gluBuild2DMipmaps(GL_TEXTURE_2D,4,sizex,sizey,GL_RGBA,GL_UNSIGNED_BYTE,image);
	return 0;
}

//---------------------------------------------------------------------------

int Texture::loadtexInt(const char *picname, int alpha, int threshold, int mipmap) {
	FILE *file;
	unsigned char image[65536][2];
	int x,y, addr, red,green,blue, sizex, sizey;

	file= fopen(picname, "rb");
    if (file == 0) return 1;
    fseek(file, 18, SEEK_SET);
	 sizex=fgetc(file)+fgetc(file)*256;
    fseek(file, 22, SEEK_SET);
	 sizey=fgetc(file)+fgetc(file)*256;
	 if (sizex*sizey>65536) return 2;

    fseek(file, 54, SEEK_SET);
    for     (y = 0; y <sizey; y++) {
        for (x = 0; x <sizex; x++) {
			addr=x+y*sizey;
			blue =fgetc(file);
			green=fgetc(file);
			red  =fgetc(file);
			image[addr][0]=red;
			if (alpha<0) image[addr][1]=(red+green+blue)/3.;
			else {
				if ( (red <threshold) &&
					 (green<threshold) &&
					 (blue <threshold) ) image[addr][1]=0;
				else					      image[addr][1]=alpha;
			}
		}
	}
	fclose(file);

   bind();
	mip(mipmap);
	glTexImage2D(GL_TEXTURE_2D,0,GL_LUMINANCE_ALPHA,sizex,sizey,0,GL_LUMINANCE_ALPHA,GL_UNSIGNED_BYTE,image);
	if (mipmap) gluBuild2DMipmaps(GL_TEXTURE_2D,4,sizex,sizey,GL_LUMINANCE_ALPHA,GL_UNSIGNED_BYTE,image);
	return 0;
}

//--------------------------------------------------------------------------------------

#define nflare 20
#define size 256
#define halfsize 128
#define qtrsize 64

void sunflare(GLuint texID) {
	static float rot[nflare], roti[nflare], inten[nflare];
	float  sinx,cosy, f;
	static float lineinten[halfsize];
	static int init=0;
	int i, l, sqrsize, x,y,z;
	static unsigned char array[size][size][4];

	sqrsize=halfsize*halfsize;

	if (init==0) { // init flare dir & inten
		init=1;
		for (i=0; i<nflare; i++) {
			rot[i]=rndn(pi2);
			roti[i]=rndn(.2)-.1;
			inten[i]=rndn(.3)+.7;
		}
		for (i=0; i<halfsize; i++) lineinten[i]=(256.-i*(256./halfsize))/256.;//((float)(i*i)/sqrsize)*255;
	}
	for    (x=0; x<size; x++) {
		int x0,y0;
		x0=x-halfsize;
		for (y=0; y<size; y++) {
			y0=y-halfsize;
			f=(1-sqrt(x0*x0+y0*y0)/qtrsize);
			f=(f>.95)?1:f*1.05;
			z=(f<0)?0:f*250;
			array[x][y][0]=array[x][y][1]=array[x][y][2]=array[x][y][3]=z;
		}
	}
	for (i=0; i<nflare; i++) {
		rot[i]+=roti[i];
		sinx=sin(rot[i]);
		cosy=cos(rot[i]);
		for (l=0; l<halfsize-1; l++) {
			x=sinx* l+halfsize;
			y=cosy* l+halfsize;
			z=255*lineinten[l]*inten[i];
			if (z>array[x][y][0]) {
				array[x][y][0]=array[x][y][1]=array[x][y][2]=array[x][y][3]=z;
				x=sinx*-l+halfsize;
				y=cosy*-l+halfsize;
				array[x][y][0]=array[x][y][1]=array[x][y][2]=array[x][y][3]=z;
			}
		}
	}
	glBindTexture(GL_TEXTURE_2D, texID);
	glTexSubImage2D(GL_TEXTURE_2D,0, 0,0,size,size, GL_RGBA,GL_UNSIGNED_BYTE,array);
	return;
}

//----------------------------------------------------------------------------
#define texSize 128
#define nlines 30

void noisy1(GLuint texID, int grain, int insize, int lightmap) {
	unsigned char array[texSize][texSize][2];
	int xnoisev[texSize];
	int i, x,y, ynoise, texSizeh, grainh=grain*.5;
	float r, x1,y1, lum;

	texSizeh=texSize*.5;
	if (insize<1) insize=1;// min size

	if (lightmap&1) { // modify x noise -----------------------------
		for (i=0; i<texSize; i++) {
			xnoisev[i]=rndn(grain)-grainh;// (xnoise[i]+xnoisei[i])%texSize; // move
		}
	} // --------------------------------------------------------

	for    (y = 0; y <texSize; y++) { // for every line
		if (lightmap&2) ynoise=rndn(grain)-grainh;//-rndn(grain);//rndn(grain)-grainh; // rnd tv noise
		y1=(y-texSizeh)*(y-texSizeh);
		for (x = 0; x <texSize; x++) { // for every pixel in x
			x1=x-texSizeh;
			lum=255;

			if (lightmap&1) { // x noise
				lum+=xnoisev[x];
			}

			if (lightmap&2) { // y noise
				lum+=ynoise;
			}

			if (lightmap&4) { // z noise
				lum+=rndn(grain)-grainh;
			}

			lum=(lum<0)?0:(lum>255)?255:lum;

			if (lightmap&8) { // vignette
				r=1-pow(sqrt(x1*x1+y1)/insize,2);// 0-1  ** adj sharpness
				if (r<0) r=0;
				lum*=r;
			}

			if (lightmap&16) { // x grating
				if (x&1) lum-=grain;
			}
			if (lightmap&32) { // y grating
				if (y&1) lum-=grain;
			}
			lum=(lum<0)?0:lum;
			array[y][x][0]=array[y][x][1]=lum;
		}
	}
	glBindTexture(GL_TEXTURE_2D, texID);
	glTexSubImage2D(GL_TEXTURE_2D,0, 0,0,texSize,texSize,GL_LUMINANCE_ALPHA,GL_UNSIGNED_BYTE,array);
}

//-------------------------------------------------------------------------------



