//-------------------------------------------------------------------
//	File:		   utility.c       standard functions
//	Comments:   by George Ziniewicz
//-------------------------------------------------------------------


#include "utility.h"

//-------------------------------------------------------------------------------
//-------------------------------------------------------------------------------

float cos_interp(float a, float b, float x) {
	float f;

	f=(1-cos(x*pi1))*.5;
	return a*(1-f)+b*f;
}

//---------------------------------------------------------------------------------------

float lin_interp(float a, float b, float x) {
	return  a*(1-x)+b*x;
}

//---------------------------------------------------------------------------------------

void delay(int milliseconds) {
	int now, then;

	now=get_time();
	then=now+milliseconds;
	while(get_time()<then) {;}
	return;
}

//-------------------------------------------------------------------------------

int get_time(void){
	return glutGet(GLUT_ELAPSED_TIME);
}

//------------------------------------------------------------------------

void normalize(float *x, float *y, float *z) {
   float len;
   len=sqrt(*x**x+*y**y+*z**z);
   *x/=len;
   *y/=len;
   *z/=len;
   return;
}

//---------------------------------------------------------------------------------

double rndn(double range) {	 //  0->.99999 * range
	//return rand()*.0000305185*range;
	return ((double)rand()/(RAND_MAX+1))*range;
}

//---------------------------------------------------------------------------------

void rot_point(float pt1,float hd1,float yw1,float *xf,float *yf,float *zf) {
   float x,y;
   float m1,m2,m3,m4,m5,m6,m7,m8,m9;
   float sinpt,cospt, sinhd,coshd, sinyw,cosyw, spsh,cpsh;
   
   sinpt=sin(pt1);   cospt=cos(pt1);
   sinhd=sin(-hd1);  coshd=cos(-hd1);
   sinyw=sin(yw1);   cosyw=cos(yw1);
     
   spsh=-sinpt*sinhd;
   cpsh= cospt*sinhd;

   m1=( coshd*cosyw);
   m2=( coshd*-sinyw);
   m3=(-sinhd);
   m4=( spsh * cosyw+cospt*sinyw);
   m5=( spsh *-sinyw+cospt*cosyw);
   m6=(-sinpt* coshd);
   m7=( cpsh * cosyw+sinpt*sinyw);
   m8=( cpsh *-sinyw+sinpt*cosyw);
   m9=( cospt* coshd);

   x=*xf * m1 + *yf * m2 + *zf * m3;
   y=*xf * m4 + *yf * m5 + *zf * m6;
 *zf=*xf * m7 + *yf * m8 + *zf * m9;
   *xf=x;*yf=y;
   }

//----------------------------------------------------------------------------

void showmatrix(float *m, float iny, Text *text) {
	int x,y;
	char s[80];

//	sprintf(s,"Heading %3.0f",heading); textObj.printGL(s,10,40,20);
//	sprintf(s,"Pitch   %3.0f",pitch);   textObj.printGL(s,10,60,20);
	//glGetFloatv(GL_MODELVIEW_MATRIX,m);
	for    (x=0; x<4; x++) {
		for (y=0; y<4; y++) {
			sprintf(s,"%2d: %6.3f",x*4+y,m[x*4+y]);
			text->print(s,x*120+50,iny-(y*30),10);
		}
	}
	return;
}

//-------------------------------------------------------------------------

void show_rot(float pitch, float heading) {
	float x=0,y=0,z=1;
	//char s[80];

//	sprintf(s,"Heading %3.0f",heading); textObj.printGL(s,10,40,20);
//	sprintf(s,"Pitch   %3.0f",pitch);   textObj.printGL(s,10,60,20);
	rot_point(pitch*rc,heading*rc,0, &x,&y,&z);
//	sprintf(s,"x: %6.3f",x); textObj.printGL(s,50,100,20);
//	sprintf(s,"y: %6.3f",y); textObj.printGL(s,50,140,20);
//	sprintf(s,"z: %6.3f",z); textObj.printGL(s,50,180,20);
	return;
}

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

void partial_sphere(float size, int tesx, int tesy, float toffx, float toffy,
					float startx, float starty,	float endx, float endy) {
   int xn,yn,yn2, x0,x1, y0,y1;
   float radincx,radincy, dx,dy,dz, dx2,dy2,dz2;
   double anglex,angley,angley2;

   x0=(tesx+1)*startx;
   y0=tesy*starty;
   x1=(tesx+1)*endx;
   y1=tesy*endy;
   if (x1<=x0) x1=x0+1;
   if (y1<=y0) y1=y0+1;
   if ((x1-x0)>tesx+1) x1=x0+tesx;
   if ((y1-y0)>tesy-1) y1=y0+tesy-1;

   radincx=pi2/ tesx; // compute quad x radius
   radincy=pi1/(tesy-1);//  "         y
   for (yn=y0; yn<y1; yn++) {
      yn2=yn+1;
      angley =yn *radincy;
      angley2=yn2*radincy;
	  dy =-cos(angley );
	  dy2=-cos(angley2);

      glBegin(GL_TRIANGLE_STRIP);
      for (xn=x1; xn>=x0; xn--) {
            anglex=xn*radincx;
            dx = (sin(anglex)*sin(angley )); dz =(cos(anglex)*sin(angley ));
            dx2= (sin(anglex)*sin(angley2)); dz2=(cos(anglex)*sin(angley2));
            
    	    glNormal3f(dx2,dy2,dz2);
     	    glTexCoord2f((float)xn/tesx+toffx,(float)yn2/tesy+toffy);
     	    glVertex3f(dx2*size, dy2*size, dz2*size);
     	    
            glNormal3f(dx ,dy ,dz );
    	    glTexCoord2f((float)xn/tesx+toffx,(float)yn /tesy+toffy);
    	    glVertex3f(dx *size, dy *size, dz *size);
         }
         glEnd();	
		}
	return;
}

//---------------------------------------------------------------------------------

// create sphere of size, tesselated by tesx/y,
//   with texcoord starting at texoff* - texmag*, (1,0 mag,off is "normal" mapping)
void solid_sphere(double size, int tesx, int tesy,
						double texmagx, double texmagy,
						double texoffx, double texoffy) {
	int aix, ix,iy, iy2;
	double rx,ry, cx, cy,cy2, dx,dy,dz, dx2,dy2,dz2;
	double fx,fy,fy2, texx, texy, texy2;
	if ((tesx<3)||(tesy<3)) return;
	if ((tesx>100)||(tesy>100)) return;
	rx=pi2/ tesx;
	ry=pi1/(tesy-1);
	for    (iy=0; iy<(tesy-1); iy++) {
		iy2=iy+1;
		fy =iy*ry;
		fy2=iy2*ry;
		dy = -cos(fy );
		dy2= -cos(fy2);
		cy =(double)iy /tesy;
		cy2=(double)iy2/tesy;
		texy =texoffy+texmagy*cy;
		texy2=texoffy+texmagy*cy2;

		glBegin(GL_TRIANGLE_STRIP);
			for (ix=0; ix<=tesx; ix++) {
				cx=(double)ix /tesx;
				aix=(ix==tesx) ? 0 : ix;
				fx=aix*rx;
				dx =sin(fx)*sin(fy ); dz =cos(fx)*sin(fy );
            dx2=sin(fx)*sin(fy2); dz2=cos(fx)*sin(fy2);
				texx=texoffx+texmagx*cx;

				glNormal3d(dx2,dy2,dz2);
				glTexCoord2d(texx, texy2);
				glVertex3d(dx2*size, dy2*size, dz2*size);

				glNormal3d(dx ,dy ,dz);
				glTexCoord2d(texx, texy);
				glVertex3d(dx *size, dy *size, dz *size);
			}
			glEnd();
	}
	return;
}

//------------------------------------------------------------------------------

void solid_torus(int tes, double size, double thick) {
   int i,j,k, tes2;
   double s,t, x,y,z, angleincr;
   
   tes2=tes*2;
   angleincr=pi2/tes;
   for          (i=0; i< tes ; i++) {// around thickness
      glBegin(GL_QUAD_STRIP);
         for    (j=0; j<=tes2; j++) {// around ring
            for (k=1; k>= 0; k--) {
               s=(i+k) % tes + 0.5;
               t=j % tes2;
               x=(size+thick*cos(s*angleincr))*cos(t*angleincr);
               y=(size+thick*cos(s*angleincr))*sin(t*angleincr);
               z=      thick*sin(s*angleincr);
               glTexCoord2d(((double)j/tes2)*8,((double)i/tes+k));
			   glVertex3d(x,y,z);
               }
            }
         glEnd();
      }
   return;
   }

//------------------------------------------------------------------------------

void set_blend(int blend0, int blend1) {
	int blendFunc0[]={GL_ZERO,
					  GL_ONE,
					  GL_SRC_ALPHA,
					  GL_ONE_MINUS_SRC_ALPHA,
					  GL_DST_ALPHA,
					  GL_ONE_MINUS_DST_ALPHA,
					  GL_DST_COLOR,
					  GL_ONE_MINUS_DST_COLOR,
					  GL_SRC_ALPHA_SATURATE};
	int blendFunc1[]={GL_ZERO,
					  GL_ONE,
					  GL_SRC_ALPHA,
					  GL_ONE_MINUS_SRC_ALPHA,
					  GL_DST_ALPHA,
					  GL_ONE_MINUS_DST_ALPHA,
					  GL_SRC_COLOR,
					  GL_ONE_MINUS_SRC_COLOR};

	glBlendFunc(blendFunc0[blend0],blendFunc1[blend1]);
	return;
}

//----------------------------------------------------------------------

void doc_blend(int type, int n, Text *text) {
	switch(type) {
	case 0:
		switch(n) {
		case 0: text->hold("0:0-GL_ZERO");     break;
		case 1: text->hold("0:1-GL_ONE");      break;
		case 2: text->hold("0:2-GL_SRC_ALPHA");           break;
		case 3: text->hold("0:3-GL_ONE_MINUS_SRC_ALPHA"); break;
		case 4: text->hold("0:4-GL_DST_ALPHA");           break;
		case 5: text->hold("0:5-GL_ONE_MINUS_DST_ALPHA"); break;
		case 6: text->hold("0:6-GL_DST_COLOR");           break;
		case 7: text->hold("0:7-GL_ONE_MINUS_DST_COLOR"); break;
		case 8: text->hold("0:8-GL_SRC_ALPHA_SATURATE"); break;
		}
		break;
	case 1:
		switch(n) {
		case 0: text->hold("1:0-GL_ZERO");      break;
		case 1: text->hold("1:1-GL_ONE");       break;
		case 2: text->hold("1:2-GL_SRC_ALPHA"); break;
		case 3: text->hold("1:3-GL_ONE_MINUS_SRC_ALPHA"); break;
		case 4: text->hold("1:4-GL_DST_ALPHA");           break;
		case 5: text->hold("1:5-GL_ONE_MINUS_DST_ALPHA"); break;
		case 6: text->hold("1:6-GL_SRC_COLOR");           break;
		case 7: text->hold("1:7-GL_ONE_MINUS_SRC_COLOR"); break;
		}
		break;
	}
	return;
}

//---------------------------------------------------------------------

void limitv(float *x, float *y, float *z, float limit) {
	float v;
	
	v=sqrt(*x**x+*y**y+*z**z);
	if (v>limit) {
		v=limit/v;
		*x*=v;
		*y*=v;
		*z*=v;
	}
	return;
}

//-----------------------------------------------------------------------------

void set_polymode(Pref *pref) {
	int modes[3]={GL_FILL,GL_LINE,GL_POINT};

	glPolygonMode(GL_FRONT_AND_BACK,modes[pref->wireframe]);
	return;
}

//	switch(pref->wireframe) { // set polymode
//		case 0: glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);  break;
//		case 1: glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);  break;
//		case 2: glPolygonMode(GL_FRONT_AND_BACK,GL_POINT); break;
//		default: break;
//	}


//--------------------------------------------------------------------------

float clamp_high(float value, float limit) {
	if (value<limit) return value;
	else             return limit;
}

//--------------------------------------------------------------------

float clamp_low(float value, float limit) {
	if (value<limit) return limit;
	else             return value;
}

//--------------------------------------------------------------------

float minv(float value0, float value1) {
	if (value0<value1) return value0;
	else               return value1;
}

//--------------------------------------------------------------------

float maxv(float value0, float value1) {
	if (value0>value1) return value0;
	else               return value1;
}

//--------------------------------------------------------------------

float wrapMax(float value, float compare) {
	while (value>=compare)  value-=compare;
	return value;
}

//--------------------------------------------------------------------

float wrapMin(float value, float compare) {
	while (value<compare)  value+=compare;
	return value;
}

//--------------------------------------------------------------------

int wrapMax(int value, int compare) {
	while (value>=compare)  value-=compare;
	return value;
}

//--------------------------------------------------------------------

int wrapMin(int value, int compare) {
	while (value<compare)  value+=compare;
	return value;
}

//--------------------------------------------------------------------

/* The function below takes an arbitrary plane,
defined as a plane equation in Ax + By + Cz + D = 0 form, and a light position in homogeneous
coordinates. If the light is directional, the w value should be 0. The function concatenates the
shadow matrix with the current matrix. */

void shadowMatrix(float plane[4], float light[4]) {
    float  dot;
    float  m[16];

    dot=plane[0]*light[0]+plane[1]*light[1]+plane[2]*light[2]+plane[3]*light[3];
    m[0] = dot - light[0] * plane[0];
    m[4] = 0.0 - light[0] * plane[1];
    m[8] = 0.0 - light[0] * plane[2];
    m[12]= 0.0 - light[0] * plane[3];
    m[1] = 0.0 - light[1] * plane[0];
    m[5] = dot - light[1] * plane[1];
    m[9] = 0.0 - light[1] * plane[2];
    m[13]= 0.0 - light[1] * plane[3];
    m[2] = 0.0 - light[2] * plane[0];
    m[6] = 0.0 - light[2] * plane[1];
    m[10]= dot - light[2] * plane[2];
    m[14]= 0.0 - light[2] * plane[3];
    m[3] = 0.0 - light[3] * plane[0];
    m[7] = 0.0 - light[3] * plane[1];
    m[11]= 0.0 - light[3] * plane[2];
    m[15]= dot - light[3] * plane[3];
    glMultMatrixf((const GLfloat*)m);
	 return;
}

//----------------------------------------------------------------
/*
void shadowMatrix(float ground[4], float light[4]) {
    float  dot;
    float  shadowMat[4][4];

    dot = ground[0] * light[0] +
          ground[1] * light[1] +
          ground[2] * light[2] +
          ground[3] * light[3];
    shadowMat[0][0] = dot - light[0] * ground[0];
    shadowMat[1][0] = 0.0 - light[0] * ground[1];
    shadowMat[2][0] = 0.0 - light[0] * ground[2];
    shadowMat[3][0] = 0.0 - light[0] * ground[3];
    shadowMat[0][1] = 0.0 - light[1] * ground[0];
    shadowMat[1][1] = dot - light[1] * ground[1];
    shadowMat[2][1] = 0.0 - light[1] * ground[2];
    shadowMat[3][1] = 0.0 - light[1] * ground[3];
    shadowMat[0][2] = 0.0 - light[2] * ground[0];
    shadowMat[1][2] = 0.0 - light[2] * ground[1];
    shadowMat[2][2] = dot - light[2] * ground[2];
    shadowMat[3][2] = 0.0 - light[2] * ground[3];
    shadowMat[0][3] = 0.0 - light[3] * ground[0];
    shadowMat[1][3] = 0.0 - light[3] * ground[1];
    shadowMat[2][3] = 0.0 - light[3] * ground[2];
    shadowMat[3][3] = dot - light[3] * ground[3];
    glMultMatrixf((const GLfloat*)shadowMat);
	 return;
*/

//----------------------------------------------------------------------------
void logo321(float t, Texture *tex, Text *gen) {
	float angle, angle2, fract, x,y, c[3][3]={0,0,1, 0,1,0, 1,0,0};
	int second;
	char s[80];

	glColor3f(1,1,1);
	glLineWidth(3);
	second=t;
	second=3-(second%4);
	if (second<1) return;
	fract=t-(int)t;

	glPushMatrix();
		glLoadIdentity();
		glTranslatef(0,0,-.75);

		glEnable(GL_TEXTURE_2D);
		tex->bind();
		glBegin(GL_TRIANGLE_STRIP);// textured circle
			for(angle=0; angle<pi2; angle+=pi2/50.001) {
				angle2=angle-pi1*.25;
				if ((angle/pi2)>fract) glColor3f(0,0,0);
				else                   glColor3f(c[second-1][0],c[second-1][1],c[second-1][2]);
				x= cos(angle2)+sin(angle2);
				y=-sin(angle2)+cos(angle2);
				glTexCoord2f(x*.4+.5,  y*.4 +.5); glVertex3f(x*.3,  y*.3,0);
				glTexCoord2f(x*.35+.5, y*.35+.5); glVertex3f(x*.225, y*.225,0);
			}
			glEnd();
		glDisable(GL_TEXTURE_2D);

		glColor3f(.5,.5,.5);
		glBegin(GL_LINES);
			glVertex3f(0,-.5,0);// vert line
			glVertex3f(0, .5,0);
			glVertex3f(-.5,0,0);// hor line
			glVertex3f( .5,0,0);
			glEnd();

		glBegin(GL_LINE_LOOP);//circle
			for(angle=0; angle<pi2; angle+=pi2/50.001) {
				glVertex3f(( cos(angle)+sin(angle))*.3,
							  (-sin(angle)+cos(angle))*.3,0);
			}
			glEnd();
		glBegin(GL_LINE_LOOP);//circle
			for(angle=0; angle<pi2; angle+=pi2/50.001) {
				glVertex3f(( cos(angle)+sin(angle))*.225,
							  (-sin(angle)+cos(angle))*.225,0);
			}
			glEnd();

		glColor3f(1,1,1);
			glBegin(GL_LINES);// sweep
			angle2=pi2*fract-pi1*.25;
				glVertex3f(0,0,0);
				glVertex3f(( cos(angle2)+sin(angle2))*.35,
							  (-sin(angle2)+cos(angle2))*.35,0);
			glEnd();
		sprintf(s,"%d",second);
		gen->italic=0;
		gen->print(s,0,0,.3);
		gen->reset();
		glPopMatrix();
	return;
}

//-----------------------------------------------------------------------------

#define nshatter 61

void end_shatter(float init, Texture *tex, Screen *screen, Time *time, Pref *pref,
				 int *capture, float *skip) {
	int x,y;
	static int type=128, nx=1, ny=1;
	float fx,fy, tincx, tincy, spd, xh,yh;
	static float xinc,yinc, xsize,ysize;
	static float  xs[nshatter][nshatter],
		          ys[nshatter][nshatter],
		          zs[nshatter][nshatter],
				 xis[nshatter][nshatter],
				 yis[nshatter][nshatter],
				 zis[nshatter][nshatter],
				   p[nshatter][nshatter],
				  pi[nshatter][nshatter],
				   h[nshatter][nshatter],
				  hi[nshatter][nshatter],
				   r[nshatter][nshatter],
				  ri[nshatter][nshatter];
	static float am=1,speed=1, spd1000;
	float xsizeam, ysizeam, width=((float)screen->width/screen->height), height=1;

	if (init==0) {
		float r1=rndn(1);

		speed=rndn(1)+.5;
		am=1;
		if      (r1<.1) {nx=1; ny=1;} // 10% big block
		else if (r1<.2) {nx=nshatter-1;ny=nshatter-1;}// 10% tiny
		else if (r1<.4) {nx=rndn(nshatter-3)+2; ny=nx;}// 20% squares
		else            {nx=rndn(nshatter-3)+2; ny=rndn(nshatter-3)+2;} //60% rects

		xinc=width/nx;
		yinc=height/ny;
		xsize=.5*xinc;
		ysize=.5*yinc;
		xh=.5*width;
		yh=.5*height;
		r1=rndn(1);
		if (*skip==0) {
			if      (r1<.1) {
				if (rndn(1)<.5) *skip=.1;
				else            *skip=1.1;
			}
			else if (r1<.5) type=rndn(8191)+1;
			else {
				type=1+2+4; // move x/y/z
				type+=pow(2,3+rndn(2));// +=x
				type+=pow(2,5+rndn(2));// +-y
				type+=pow(2,7+rndn(2));// +-z
				type+=512+1024+2048;//rotate all 3 axis
			}
		}
		for     (x=0; x<nx; x++) {
			for (y=0; y<ny; y++) {
				xs[x][y]=x*xinc+xsize-xh;
				ys[x][y]=y*yinc+ysize-yh;
				zs[x][y]=0;
				xis[x][y]=sin(rndn(pi2))*(rndn(.5)+.5)*.2;
				yis[x][y]=sin(rndn(pi2))*(rndn(.5)+.5)*.2;
				zis[x][y]=sin(rndn(pi2))*(rndn(.5)+.5)*.1;
				p[x][y]=0;
				pi[x][y]=rndn(200)-100;
				h[x][y]=0;
				hi[x][y]=rndn(200)-100;
				r[x][y]=0;
				ri[x][y]=rndn(200)-100;
			}
		}
	return;
	}
	glColor4f(1,1,1,1);
	glEnable(GL_TEXTURE_2D);
	tex->bind();
	spd=time->fDeltaTime*speed;
	if (*skip) {
		if (*skip<1) {
			*skip+=2.5*time->fDeltaTime;
			if (*skip>=1) {
				if (rndn(1)<.5) {
					*skip=0;
					*capture=0;
				} else *skip=.01;
			}
		} else {
			*skip+=2.5*time->fDeltaTime;
			if (*skip>=2) {
				if (rndn(1)<.5) {
					*skip=0;
					*capture=0;
				} else *skip=1.01;
			}
		}

		glPushMatrix();
			glTranslatef(-.5*width,-.5*height,0);
			if (*skip>1) {
				glBegin(GL_QUADS);
					glTexCoord2f(-*skip,0);   glVertex2f(0,0);
					glTexCoord2f(1-*skip,0);  glVertex2f(width,0);
					glTexCoord2f(1-*skip,1);  glVertex2f(width,height);
					glTexCoord2f(-*skip,1);   glVertex2f(0,height);
					glEnd();
			} else {
				glBegin(GL_QUADS);
					glTexCoord2f(0,-*skip);  glVertex2f(0,0);
					glTexCoord2f(1,-*skip);  glVertex2f(width,0);
					glTexCoord2f(1,1-*skip); glVertex2f(width,height);
					glTexCoord2f(0,1-*skip); glVertex2f(0,height);
					glEnd();
			}
			glPopMatrix();
		return;
	}
	tincx=1./nx;
	tincy=1./ny;
	spd1000=spd*2;
	if (type&4096) {
		am-=time->fDeltaTime*.25;
		if (am<0) am=0;
	}
	xsizeam=xsize*am;
	ysizeam=ysize*am;
	for    (x=0; x<nx; x++) { 
		fx=(float)x/nx;
		for (y=0; y<ny; y++) {
			fy=(float)y/ny;
			if (type&1)  xs[x][y] +=xis[x][y]*spd;
			if (type&2)  ys[x][y] +=yis[x][y]*spd;
			if (type&4)  zs[x][y] +=zis[x][y]*spd;
			if (type&8)  xis[x][y]-=spd1000;
			if (type&16) xis[x][y]+=spd1000;
			if (type&32) yis[x][y]-=spd1000;
			if (type&64) yis[x][y]+=spd1000;
			if (type&128) zis[x][y]-=spd1000*.5;
			if (type&256) zis[x][y]+=spd1000*.5;
			if (type&512) p[x][y]+=pi[x][y]*spd;
			if (type&1024) h[x][y]+=hi[x][y]*spd;
			if (type&2048) r[x][y]+=ri[x][y]*spd;
			glPushMatrix();
				glTranslatef(xs[x][y],ys[x][y],zs[x][y]);
				glRotatef(p[x][y],1,0,0);
				glRotatef(h[x][y],0,1,0);
				glRotatef(r[x][y],0,0,1);
				glBegin(GL_QUADS);
					glTexCoord2f(fx      ,fy);       glVertex2f(-xsizeam,-ysizeam);
					glTexCoord2f(fx+tincx,fy);       glVertex2f(+xsizeam,-ysizeam);
					glTexCoord2f(fx+tincx,fy+tincy); glVertex2f(+xsizeam,+ysizeam);
					glTexCoord2f(fx      ,fy+tincy); glVertex2f(-xsizeam,+ysizeam);
					glEnd();
				glPopMatrix();
		}
	}
	glDisable(GL_TEXTURE_2D);
	return;
}

//-----------------------------------------------------------------------------

void draw_cube(float w0,float h0,float d0) {
	float w,h,d;

	w=w0*.5;
	h=h0*.5;
	d=d0*.5;
	glBegin(GL_QUADS);
		glNormal3f(0,0,-1);
		glVertex3f(-w,-h,-d); glVertex3f( w,-h,-d); glVertex3f( w, h,-d); glVertex3f(-w, h,-d);//f
		glNormal3f(0,0,1);
		glVertex3f(-w,-h, d); glVertex3f( w,-h, d); glVertex3f( w, h, d); glVertex3f(-w, h, d);//b
		glNormal3f(-1,0,0);
		glVertex3f(-w,-h,-d); glVertex3f(-w,-h, d); glVertex3f(-w, h, d); glVertex3f(-w, h,-d);//l
		glNormal3f( 1,0,0);
		glVertex3f( w,-h,-d); glVertex3f( w,-h, d); glVertex3f( w, h, d); glVertex3f( w, h,-d);//r
		glNormal3f(0,1,0);
		glVertex3f(-w, h,-d); glVertex3f( w, h,-d); glVertex3f( w, h, d); glVertex3f(-w, h, d);//t
		glNormal3f(0,-1,0);
		glVertex3f(-w,-h,-d); glVertex3f( w,-h,-d); glVertex3f( w,-h, d); glVertex3f(-w,-h, d);//b
		glEnd();
}

//-----------------------------------------------------------------------------

void draw_pill(float size, float thick) {
	GLUquadric *q;
	q=gluNewQuadric();
	gluQuadricTexture(q,GL_TRUE);
	glPushMatrix();
		glTranslatef(0,0,-thick*.5);

		gluQuadricOrientation(q, GLU_OUTSIDE);
		gluCylinder(q,size,size,thick,30,3);

		gluQuadricOrientation(q, GLU_INSIDE);
		gluDisk(q,0,size,30,3);

		glTranslatef(0,0,thick);
		gluDisk(q,0,size,30,3);

	glPopMatrix();
	gluDeleteQuadric(q);
	return;
}

//-----------------------------------------------------------------------------

void set_fog(int universe) {
	switch(universe) {
		case 0: set_fogcolor(0,0,0,1); break; // stars
		case 1: set_fogcolor(0,0,0,1); break;
		case 2: set_fogcolor(1,1,1,1); break; // skybox
		case 3: set_fogcolor(1,1,1,1); break;
	}
	return;
}

//-----------------------------------------------------------------------------

void set_fogcolor(float r, float g, float b, float a) {
	GLfloat fogcolor[4];

	fogcolor[0]=r;
	fogcolor[1]=g;
	fogcolor[2]=b;
	fogcolor[3]=a;
	glFogfv(GL_FOG_COLOR,fogcolor);
}

//-----------------------------------------------------------------------------

int log_message(char *ins) {
	char s[200];
	static int created=0;

	cout << ins;
	cout << "\n";
	strcpy(s,"app.log");
	ofstream file(s,(!created)? ios::out : ios::app);
	created=1;

	if (file.bad()||(file==0)) return 1;

	strcpy(s,ins);
	strcat(s,"\n");

	file.write((const signed char *) s, strlen(s));
	file.close();
	return 0;
}

//-----------------------------------------------------------------------------





